@@ -1,7 +1,9 @@
-/*
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
-|  Part of the gtkpod project.
+/* Time-stamp: <2005-06-17 22:12:19 jcs>
 |
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
+|  Part of the gtkpod project.
+| 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
@@ -22,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: confirmation.c,v 1.16 2004/07/25 15:55:37 jcsjcs Exp $
+|  $Id: confirmation.c,v 1.21 2005/06/17 13:30:02 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -30,16 +32,16 @@
 #endif
 
 #include <string.h>
-#include "support.h"
+#include "misc.h"
 #include "confirmation.h"
 #include "prefs.h"
-#include "interface.h"
 
 
 static GHashTable *id_hash = NULL;
 
 typedef struct {
     GtkWidget *window;
+    GladeXML *window_xml;
     gboolean  scrolled;
     ConfHandlerOpt option1_handler;
     gboolean option1_invert;
@@ -277,6 +279,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     ConfData *cd;
     gint defx, defy;
     gint *idp;
+    GladeXML *confirm_xml;
 
     if (id_hash == NULL)
     {  /* initialize hash table to store IDs */
@@ -288,8 +291,8 @@ GtkResponseType gtkpod_confirmation (gint id,
 	if ((cd = g_hash_table_lookup (id_hash, &id)))
 	{ /* window with same ID already open -- add @text and return
 	   * */
-	    if (text &&
-		cd->window && ((w = lookup_widget (cd->window, "text"))))
+	    if (text && *text &&
+		cd->window && ((w = glade_xml_get_widget (cd->window_xml, "text"))))
 	    {
 		GtkTextIter ti;
 		GtkTextBuffer *tb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(w));
@@ -300,7 +303,7 @@ GtkResponseType gtkpod_confirmation (gint id,
 		    gtk_text_view_set_buffer(GTK_TEXT_VIEW(w), tb);
 		    gtk_text_view_set_editable(GTK_TEXT_VIEW(w), FALSE);
 		    gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), FALSE);
-		    if ((w1 = lookup_widget (cd->window, "scroller")))
+		    if ((w1 =  glade_xml_get_widget (cd->window_xml, "scroller")))
 			gtk_widget_show (w1);
 		    cd->scrolled = TRUE;
 		}
@@ -333,13 +336,16 @@ GtkResponseType gtkpod_confirmation (gint id,
 	else         return GTK_RESPONSE_OK;
     }
 
-    window = create_confirm_dialog ();
+    /* window = create_confirm_dialog (); */
+    confirm_xml = glade_xml_new (xml_file, "confirm_dialog", NULL);
+    window = glade_xml_get_widget (confirm_xml, "confirm_dialog");
 
     /* insert ID into hash table */
     idp = g_malloc (sizeof (gint));
     *idp = id;
     cd = g_malloc (sizeof (ConfData));
     cd->window = window;
+    cd->window_xml = confirm_xml;
     cd->option1_handler = option1_handler;
     cd->option2_handler = option2_handler;
     cd->confirm_again_handler = confirm_again_handler;
@@ -357,11 +363,11 @@ GtkResponseType gtkpod_confirmation (gint id,
 	gtk_window_set_title (GTK_WINDOW(window), _("Confirmation Dialogue"));
 
     /* Set label */
-    if (label && (w = lookup_widget (window, "label")))
+    if (label && (w = glade_xml_get_widget (confirm_xml, "label")))
 	gtk_label_set_text(GTK_LABEL(w), label);
 
     /* Set text */
-    w = lookup_widget (window, "text");
+    w = glade_xml_get_widget (confirm_xml, "text");
     if (text)
     {
 	if (w)
@@ -377,7 +383,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     }
     else
     { /* no text -> hide widget */
-	if ((w = lookup_widget (window, "scroller")))
+	if ((w = glade_xml_get_widget (confirm_xml, "scroller")))
 	    gtk_widget_hide (w);
 	cd->scrolled = FALSE;
 	prefs_get_size_conf (&defx, &defy);
@@ -385,7 +391,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     gtk_window_set_default_size (GTK_WINDOW (window), defx, defy);
 
     /* Set "Option 1" checkbox */
-    w = lookup_widget (window, "option_vbox");
+    w = glade_xml_get_widget (confirm_xml, "option_vbox");
     if (w && option1_handler && option1_text)
     {
 	gboolean state, invert;
@@ -411,7 +417,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     }
 
     /* Set "Option 2" checkbox */
-    w = lookup_widget (window, "option_vbox");
+    w = glade_xml_get_widget (confirm_xml, "option_vbox");
     if (w && option2_handler && option2_text)
     {
 	gboolean state, invert;
@@ -437,7 +443,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     }
 
     /* Set "Never Again" checkbox */
-    w = lookup_widget(window, "never_again");
+    w = glade_xml_get_widget (confirm_xml, "never_again");
     if (w && confirm_again_handler)
     { /* connect signal */
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w),
@@ -455,7 +461,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     /* Hide and set "default" button that can be activated by pressing
        ENTER in the window (usually OK)*/
     /* Hide or default CANCEL button */
-    if ((w = lookup_widget (window, "cancel")))
+    if ((w = glade_xml_get_widget (confirm_xml, "cancel")))
     {
 	GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
 	gtk_widget_grab_default (w);
@@ -463,7 +469,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     }
 
     /* Hide or default APPLY button */
-    if ((w = lookup_widget (window, "apply")))
+    if ((w = glade_xml_get_widget (confirm_xml, "apply")))
     {
 	GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
 	gtk_widget_grab_default (w);
@@ -471,7 +477,7 @@ GtkResponseType gtkpod_confirmation (gint id,
     }
 
     /* Hide or default OK button */
-    if ((w = lookup_widget (window, "ok")))
+    if ((w = glade_xml_get_widget (confirm_xml, "ok")))
     {
 	GTK_WIDGET_SET_FLAGS (w, GTK_CAN_DEFAULT);
 	gtk_widget_grab_default (w);
