@@ -1,7 +1,10 @@
-/*
+/* Time-stamp: <2005-06-25 01:05:39 jcs>
+|
 |  Copyright (C) 2003 Corey Donohoe <atmos at atmos dot org>
+|  Copyright (C) 2003-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -22,8 +25,9 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: context_menus.c,v 1.30 2004/12/06 15:21:04 jcsjcs Exp $
+|  $Id: context_menus.c,v 1.42 2005/06/24 16:17:18 jcsjcs Exp $
 */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -31,16 +35,16 @@
 #include "display.h"
 #include "file.h"
 #include "misc.h"
-#include "playlist.h"
+#include "misc_track.h"
 #include "prefs.h"
-#include "support.h"
 #include "tools.h"
-#include "track.h"
+#include "itdb.h"
 
 static guint entry_inst = -1;
 static GList *selected_tracks = NULL;
 static Playlist *selected_playlist = NULL;
 static TabEntry *selected_entry = NULL; 
+static iTunesDB *active_itdb = NULL;
 /* types of context menus (PM/ST/TM) */
 typedef enum {
     CM_PM = 0,
@@ -59,7 +63,7 @@ static void
 export_entries(GtkWidget *w, gpointer data)
 {
     if(selected_tracks)
-	export_files_init(selected_tracks);
+	export_files_init (selected_tracks, NULL, FALSE, NULL);
 }
 
 /**
@@ -123,11 +127,11 @@ static void
 update_entries(GtkMenuItem *mi, gpointer data)
 {
     if (selected_playlist)
-	do_selected_playlist (update_trackids);
+	gp_do_selected_playlist (update_tracks);
     else if(selected_entry)
-	do_selected_entry (update_trackids, entry_inst);
+	gp_do_selected_entry (update_tracks, entry_inst);
     else if(selected_tracks)
-	do_selected_tracks (update_trackids);
+	gp_do_selected_tracks (update_tracks);
 }
 
 /*
@@ -140,11 +144,11 @@ static void
 sync_dirs_entries(GtkMenuItem *mi, gpointer data)
 {
     if (selected_playlist)
-	do_selected_playlist (sync_trackids);
+	gp_do_selected_playlist (sync_tracks);
     else if(selected_entry)
-	do_selected_entry (sync_trackids, entry_inst);
+	gp_do_selected_entry (sync_tracks, entry_inst);
     else if(selected_tracks)
-	do_selected_tracks (sync_trackids);
+	gp_do_selected_tracks (sync_tracks);
 }
 
 /**
@@ -154,39 +158,16 @@ sync_dirs_entries(GtkMenuItem *mi, gpointer data)
  * @data - ignored, should be NULL
  */
 static void 
-delete_entries(GtkMenuItem *mi, gpointer data)
+delete_entries(GtkMenuItem *mi, DeleteAction deleteaction)
 {
     if (selected_playlist)
-	delete_playlist_head (FALSE);
+	delete_playlist_head (deleteaction);
     else if(selected_entry)
-	delete_entry_head (entry_inst, FALSE);
+	delete_entry_head (entry_inst, deleteaction);
     else if(selected_tracks)
-	delete_track_head (FALSE);
+	delete_track_head (deleteaction);
 }
 
-/**
- * delete_entries_full - delete the currently selected entry, be it a playlist,
- * items in a sort tab, or tracks. All member tracks are removed from
- * the iPod completely.
- * @mi - the menu item selected
- * @data - ignored, should be NULL
- */
-static void 
-delete_entries_full(GtkMenuItem *mi, gpointer data)
-{
-    if (selected_playlist)
-    {
-	delete_playlist_head (TRUE);
-    }
-    else if (selected_entry)
-    {
-	delete_entry_head (entry_inst, TRUE);
-    }
-    else if (selected_tracks)
-	delete_track_head (TRUE);
-}
-
-
 /* Edit selected smart playlist */
 static void edit_spl (GtkMenuItem *mi, gpointer data)
 {
@@ -203,7 +184,7 @@ static void edit_spl (GtkMenuItem *mi, gpointer data)
 static void
 create_playlist_from_entries (GtkMenuItem *mi, gpointer data)
 {
-    generate_new_playlist (selected_tracks);
+    generate_new_playlist (active_itdb, selected_tracks);
 }
 
 /**
@@ -252,7 +233,7 @@ static void normalize_entries (GtkMenuItem *mi, gpointer data)
  * @func - the callback for when the item is selected (or NULL)
  * @mi - the GtkWidget we're gonna hook into the menu
  */
-GtkWidget *hookup_mi (GtkWidget *m, gchar *str, gchar *stock, GCallback func)
+GtkWidget *hookup_mi (GtkWidget *m, gchar *str, gchar *stock, GCallback func, gpointer userdata)
 {
     GtkWidget *mi;
 
@@ -272,7 +253,7 @@ GtkWidget *hookup_mi (GtkWidget *m, gchar *str, gchar *stock, GCallback func)
     gtk_widget_set_sensitive(mi, TRUE);
     gtk_widget_add_events(mi, GDK_BUTTON_RELEASE_MASK);
     if (func)
-	g_signal_connect(G_OBJECT(mi), "activate", func, NULL);
+	g_signal_connect(G_OBJECT(mi), "activate", func, userdata);
     gtk_container_add (GTK_CONTAINER (m), mi);
     return mi;
 }
@@ -297,10 +278,18 @@ void
 create_context_menu(CM_type type)
 {
     static GtkWidget *menu[CM_NUM];
-    static GtkWidget *mi_d[CM_NUM];   /* delete but keep track menu */
-    static GtkWidget *mi_df[CM_NUM];  /* delete with tracks menu */
-    static GtkWidget *mi_sep[CM_NUM]; /* separator */
-    static GtkWidget *mi_spl[CM_NUM]; /* edit smart playlist */
+    static GtkWidget *mi_exp[CM_NUM];  /* Export Tracks */
+    static GtkWidget *mi_pl[CM_NUM];   /* DELETE_ACTION_PLAYLIST */
+    static GtkWidget *mi_ipod[CM_NUM]; /* DELETE_ACTION_IPOD     */
+    static GtkWidget *mi_local[CM_NUM];/* DELETE_ACTION_LOCAL    */
+    static GtkWidget *mi_db[CM_NUM];   /* DELETE_ACTION_DATABASE */
+    static GtkWidget *mi_sep[CM_NUM];  /* separator              */
+    static GtkWidget *mi_spl[CM_NUM];  /* edit smart playlist    */
+    static GtkWidget *mi_ipod_all[CM_NUM];/* DELETE_ACTION_IPOD (all
+					   * tracks)      */
+    static GtkWidget *mi_db_all[CM_NUM];  /* DELETE_ACTION_DATABASE
+					   * (all tracks  */
+
     Playlist *pl;
 
     if(!menu[type])
@@ -310,24 +299,31 @@ create_context_menu(CM_type type)
 	hookup_mi (menu[type], _("Edit"), NULL, G_CALLBACK (edit_entries));
 #endif
 	hookup_mi (menu[type], _("Play Now"), "gtk-cdrom",
-		   G_CALLBACK (play_entries_now));
+		   G_CALLBACK (play_entries_now), NULL);
 	hookup_mi (menu[type], _("Enqueue"), "gtk-cdrom",
-		   G_CALLBACK (play_entries_enqueue));
-	hookup_mi (menu[type], _("Copy from iPod"), "gtk-floppy",
-		   G_CALLBACK (export_entries));
+		   G_CALLBACK (play_entries_enqueue), NULL);
+	mi_exp[type] = hookup_mi (menu[type],
+				  _("Export Tracks"), "gtk-floppy",
+				  G_CALLBACK (export_entries), NULL);
 	hookup_mi (menu[type], _("Create Playlist File"), "gtk-floppy",
-		   G_CALLBACK (create_playlist_file));
+		   G_CALLBACK (create_playlist_file), NULL);
 	hookup_mi (menu[type], _("Update"), "gtk-refresh",
-		   G_CALLBACK (update_entries));
+		   G_CALLBACK (update_entries), NULL);
 	hookup_mi (menu[type], _("Sync Dirs"), "gtk-refresh",
-		   G_CALLBACK (sync_dirs_entries));
+		   G_CALLBACK (sync_dirs_entries), NULL);
 	hookup_mi (menu[type], _("Normalize"), NULL,
-		   G_CALLBACK (normalize_entries));
+		   G_CALLBACK (normalize_entries), NULL);
+	hookup_mi (menu[type], _("Create new Playlist"),
+		   "gtk-justify-left",
+		   G_CALLBACK (create_playlist_from_entries), NULL);
+	mi_spl[type] = hookup_mi (menu[type], _("Edit Smart Playlist"),
+				  "gtk-properties",
+				  G_CALLBACK (edit_spl), NULL);
 
 	if (type == CM_ST)
 	{
 	    hookup_mi (menu[type], _("Alphabetize"), "gtk-sort-ascending",
-		       G_CALLBACK (alphabetize));
+		       G_CALLBACK (alphabetize), NULL);
 /* example for sub menus!
 	    GtkWidget *mi;
 	    GtkWidget *sub;
@@ -337,70 +333,130 @@ create_context_menu(CM_type type)
 	    gtk_widget_show (sub);
 	    gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), sub);
 	    hookup_mi (sub, _("Ascending"), "gtk-sort-ascending",
-		       G_CALLBACK (alphabetize_ascending));
+		       G_CALLBACK (alphabetize_ascending), NULL);
 	    hookup_mi (sub, _("Descending"), "gtk-sort-descending",
-		       G_CALLBACK (alphabetize_descending));
+		       G_CALLBACK (alphabetize_descending), NULL);
 	    hookup_mi (sub, _("Reset"), "gtk-undo",
-	               G_CALLBACK (reset_alphabetize));
+	               G_CALLBACK (reset_alphabetize), NULL);
 */
 	}
 	if ((type == CM_ST) || (type == CM_TM))
 	{
-	    hookup_mi (menu[type], _("Create new Playlist"), "gtk-justify-left",
-		       G_CALLBACK (create_playlist_from_entries));
 	    mi_sep[type] = add_separator (menu[type]);
-	    mi_d[type] = hookup_mi (menu[type], _("Delete From Playlist"),
-				    "gtk-delete",
-				    G_CALLBACK (delete_entries));
-	    mi_df[type] = hookup_mi (menu[type], _("Delete From iPod"),
+	    mi_ipod[type] = hookup_mi (menu[type],
+				       _("Delete From iPod"),
+				       "gtk-delete",
+				       G_CALLBACK (delete_entries),
+				       (gpointer)DELETE_ACTION_IPOD);
+	    mi_local[type] = hookup_mi (menu[type],
+					_("Delete From Harddisk"),
+					"gtk-delete",
+					G_CALLBACK (delete_entries),
+					(gpointer)DELETE_ACTION_LOCAL);
+	    mi_db[type] = hookup_mi (menu[type],
+				     _("Delete From Database"),
 				     "gtk-delete",
-				     G_CALLBACK (delete_entries_full));
-	    mi_spl[type] = hookup_mi (menu[type], _("Edit Smart Playlist"),
-				      "gtk-properties",
-				      G_CALLBACK (edit_spl));
+				     G_CALLBACK (delete_entries),
+				     (gpointer)DELETE_ACTION_DATABASE);
+	    mi_pl[type] = hookup_mi (menu[type],
+				     _("Delete From Playlist"),
+				     "gtk-delete",
+				     G_CALLBACK (delete_entries),
+				     (gpointer)DELETE_ACTION_PLAYLIST);
 	}
 	if (type == CM_PM)
 	{
 	    mi_sep[type] = add_separator (menu[type]);
-	    mi_d[type] = hookup_mi (menu[type], _("Delete But Keep Tracks"),
-				    "gtk-delete",
-				    G_CALLBACK (delete_entries));
-	    mi_df[type] = hookup_mi (menu[type], _("Delete Including Tracks"),
+	    mi_ipod[type] = hookup_mi (menu[type],
+				       _("Delete Including Tracks"),
+				       "gtk-delete",
+				       G_CALLBACK (delete_entries),
+				       (gpointer)DELETE_ACTION_IPOD);
+	    mi_local[type] = hookup_mi (menu[type],
+					_("Delete Including Tracks (Harddisk)"),
+					"gtk-delete",
+					G_CALLBACK (delete_entries),
+					(gpointer)DELETE_ACTION_LOCAL);
+	    mi_db[type] = hookup_mi (menu[type],
+				     _("Delete Including Tracks (Database)"),
 				     "gtk-delete",
-				     G_CALLBACK (delete_entries_full));
-	    mi_spl[type] = hookup_mi (menu[type], _("Edit Smart Playlist"),
-				      "gtk-properties",
-				      G_CALLBACK (edit_spl));
+				     G_CALLBACK (delete_entries),
+				     (gpointer)DELETE_ACTION_DATABASE);
+	    mi_pl[type] = hookup_mi (menu[type],
+				     _("Delete But Keep Tracks"),
+				     "gtk-delete",
+				     G_CALLBACK (delete_entries),
+				     (gpointer)DELETE_ACTION_PLAYLIST);
+	    mi_ipod_all[type] = hookup_mi (menu[type],
+					   _("Remove All Tracks from iPod"),
+					   "gtk-delete",
+					   G_CALLBACK (delete_entries),
+					   (gpointer)DELETE_ACTION_IPOD);
+	    mi_db_all[type] = hookup_mi (menu[type],
+					 _("Remove All Tracks from Database"),
+					 "gtk-delete",
+					 G_CALLBACK (delete_entries),
+					 (gpointer)DELETE_ACTION_DATABASE);
 	}
     }
-    /* Make sure, only available delete options are displayed */
+    /* Make sure, only available options are displayed */
     pl = pm_get_selected_playlist();
     if (pl)
     {
+	iTunesDB *itdb = pl->itdb;
+	g_return_if_fail (itdb);
+
+	/* Make sure, only available delete options are displayed */
 	switch (type)
 	{
 	case CM_PM:
 	    if (pl->is_spl)
 	    {
-		gtk_widget_show (mi_sep[type]);
-		gtk_widget_show (mi_d[type]);
-		gtk_widget_hide (mi_df[type]);
 		gtk_widget_show (mi_spl[type]);
 	    }
 	    else
 	    {
 		gtk_widget_hide (mi_spl[type]);
-		if (pl->type == PL_TYPE_NORM)
+	    }
+	    if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	    {
+		gtk_widget_hide (mi_local [type]);
+		gtk_widget_hide (mi_db [type]);
+		gtk_widget_hide (mi_db_all [type]);
+		if (pl->type == ITDB_PL_TYPE_MPL)
 		{
-		    gtk_widget_show (mi_sep[type]);
-		    gtk_widget_show (mi_d[type]);
-		    gtk_widget_show (mi_df[type]);
+		    gtk_widget_hide (mi_sep [type]);
+		    gtk_widget_hide (mi_ipod [type]);
+		    gtk_widget_hide (mi_pl [type]);
+		    gtk_widget_show (mi_ipod_all [type]);
 		}
 		else
 		{
-		    gtk_widget_hide (mi_sep[type]);
-		    gtk_widget_hide (mi_d[type]);
-		    gtk_widget_hide (mi_df[type]);
+		    gtk_widget_show (mi_sep [type]);
+		    gtk_widget_show (mi_ipod [type]);
+		    gtk_widget_show (mi_pl [type]);
+		    gtk_widget_hide (mi_ipod_all [type]);
+		}
+	    }
+	    if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	    {
+		gtk_widget_hide (mi_ipod [type]);
+		gtk_widget_hide (mi_ipod_all [type]);
+		if (pl->type == ITDB_PL_TYPE_MPL)
+		{
+		    gtk_widget_hide (mi_sep [type]);
+		    gtk_widget_hide (mi_local [type]);
+		    gtk_widget_hide (mi_db [type]);
+		    gtk_widget_hide (mi_pl [type]);
+		    gtk_widget_show (mi_db_all [type]);
+		}
+		else
+		{
+		    gtk_widget_show (mi_sep [type]);
+		    gtk_widget_show (mi_local [type]);
+		    gtk_widget_show (mi_db [type]);
+		    gtk_widget_show (mi_pl [type]);
+		    gtk_widget_hide (mi_db_all [type]);
 		}
 	    }
 	    break;
@@ -408,29 +464,60 @@ create_context_menu(CM_type type)
 	case CM_TM:
 	    if (pl->is_spl)
 	    {
-		gtk_widget_hide (mi_d[type]);
-		gtk_widget_hide (mi_df[type]);
 		gtk_widget_show (mi_spl[type]);
 	    }
 	    else
 	    {
 		gtk_widget_hide (mi_spl[type]);
-		if (pl->type == PL_TYPE_NORM)
+	    }
+	    if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	    {
+		gtk_widget_hide (mi_local [type]);
+		gtk_widget_hide (mi_db [type]);
+		if (pl->type == ITDB_PL_TYPE_MPL)
 		{
-		    gtk_widget_show (mi_d[type]);
-		    gtk_widget_show (mi_df[type]);
+		    gtk_widget_show (mi_ipod [type]);
+		    gtk_widget_hide (mi_pl [type]);
 		}
 		else
 		{
-		    gtk_widget_hide (mi_d[type]);
-		    gtk_widget_show (mi_df[type]);
+		    gtk_widget_show (mi_ipod [type]);
+		    gtk_widget_show (mi_pl [type]);
+		}
+	    }
+	    if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	    {
+		gtk_widget_hide (mi_ipod [type]);
+		if (pl->type == ITDB_PL_TYPE_MPL)
+		{
+		    gtk_widget_show (mi_local [type]);
+		    gtk_widget_show (mi_db [type]);
+		    gtk_widget_hide (mi_pl [type]);
+		}
+		else
+		{
+		    gtk_widget_show (mi_local [type]);
+		    gtk_widget_show (mi_db [type]);
+		    gtk_widget_show (mi_pl [type]);
 		}
 	    }
 	    break;
 	case CM_NUM:  /* to avoid compiler warning */
 	    break;
 	}
+	/* turn 'export tracks' insensitive when necessary */
+	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	{
+	    gtk_widget_set_sensitive (mi_exp[type],
+				      !prefs_get_offline ());
+	}
+	else
+	{
+	    gtk_widget_set_sensitive (mi_exp[type], TRUE);
+	}
     }
+
+
     /* 
      * button should be button 0 as per the docs because we're calling
      * from a button release event
@@ -451,6 +538,7 @@ tm_context_menu_init(void)
 
     selected_entry = NULL; 
     selected_playlist = NULL;
+    active_itdb = gp_get_active_itdb ();
     entry_inst = -1;
     if (selected_tracks)  g_list_free (selected_tracks);
     selected_tracks = tm_get_selected_tracks();
@@ -474,6 +562,7 @@ pm_context_menu_init(void)
     selected_entry = NULL;
     entry_inst = -1;
     selected_playlist = pm_get_selected_playlist();
+    active_itdb = gp_get_active_itdb ();
     if(selected_playlist)
     {
 	selected_tracks = g_list_copy (selected_playlist->members);
@@ -495,6 +584,7 @@ st_context_menu_init(gint inst)
     selected_tracks = NULL;
     selected_playlist = NULL;
     selected_entry = st_get_selected_entry (inst);
+    active_itdb = gp_get_active_itdb ();
     if(selected_entry)
     {
 	entry_inst = inst;
