@@ -1,8 +1,9 @@
-/* Time-stamp: <2004-12-30 12:58:34 jcs>
+/* Time-stamp: <2005-07-02 00:32:38 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display.h,v 1.83 2004/12/30 04:33:57 jcsjcs Exp $
+|  $Id: display.h,v 1.100 2005/07/01 16:53:43 jcsjcs Exp $
 */
 
 #ifndef __DISPLAY_H__
@@ -34,9 +35,12 @@
 #endif
 
 #include <gtk/gtk.h>
-#include "info.h"
-#include "track.h"
-#include "playlist.h"
+#include <glade/glade.h>
+#include "itdb.h"
+#include "display_itdb.h"
+
+/* Main XML glade file */
+GladeXML *main_window_xml;
 
 /* pointer to main window */
 extern GtkWidget *gtkpod_window;
@@ -88,7 +92,7 @@ typedef struct {
   GList *members;  /* GList with member tracks (pointer to "Track") */
 } TabEntry;
 
-/* struct for time info (created, played, modified) */
+/* struct for time info (added, played, modified) */
 typedef struct {
     gchar *int_str;   /* copy of string specified in the sort tab */
     gboolean valid;   /* is current string valid? */
@@ -116,10 +120,12 @@ typedef struct {
   GList *sp_members;                 /* list of tracks in sorttab */
   GList *sp_selected;                /* list of tracks selected */
   gboolean is_go;                    /* pass new members on automatically */
-  TimeInfo ti_created;               /* TimeInfo "created" (sp)  */
-  TimeInfo ti_modified;              /* TimeInfo "modified" (sp) */
-  TimeInfo ti_played;                /* TimeInfo "played" (sp)   */
-  GtkTooltips *sp_tooltips;          /* pointer to tooltips in special st */
+  TimeInfo ti_added;                 /* TimeInfo "added" (sp)         */
+  TimeInfo ti_modified;              /* TimeInfo "modified" (sp)      */
+  TimeInfo ti_played;                /* TimeInfo "played" (sp)        */
+  GtkTooltipsData *sp_tooltips_data; /* ptr to tooltips in special st */
+  /* function used for string comparisons, set in on_st_switch_page   */
+  gint (*string_compare_func) (gchar *str1, gchar *str2);
 } SortTab;
 
 /* "Column numbers" in sort tab model */
@@ -158,7 +164,7 @@ typedef enum  {
   TM_COLUMN_VOLUME,
   TM_COLUMN_YEAR,
   TM_COLUMN_CD_NR,
-  TM_COLUMN_TIME_CREATED,
+  TM_COLUMN_TIME_ADDED,
   TM_COLUMN_IPOD_PATH,        /* 20 */
   TM_COLUMN_SOUNDCHECK,
   TM_COLUMN_SAMPLERATE,
@@ -169,12 +175,47 @@ typedef enum  {
   TM_NUM_COLUMNS
 } TM_item;
 
+/* A means to address the fields by uniform IDs. May be extended as
+ * needed. You should extend "track_get_item_pointer()" defined in
+ * track.c as well for string fields. */
+/* Used in prefs_window.c to label the sort_ign_field<num> buttons */
+typedef enum {
+    T_ALL = 0,      /* all fields */
+    T_ALBUM,
+    T_ARTIST,
+    T_TITLE,
+    T_GENRE,
+    T_COMMENT,      /*  5 */
+    T_COMPOSER,
+    T_FDESC,
+    T_PC_PATH,
+    T_IPOD_PATH,
+    T_IPOD_ID,      /* 10 */
+    T_TRACK_NR,
+    T_TRANSFERRED,
+    T_SIZE,
+    T_TRACKLEN,
+    T_BITRATE,      /* 15 */
+    T_SAMPLERATE,
+    T_BPM,
+    T_PLAYCOUNT,
+    T_RATING,
+    T_TIME_ADDED,   /* 20 */
+    T_TIME_PLAYED,
+    T_TIME_MODIFIED,
+    T_VOLUME,
+    T_SOUNDCHECK,
+    T_YEAR,         /* 25 */
+    T_CD_NR,
+    T_GROUPING,
+    T_COMPILATION,
+    T_ITEM_NUM,
+} T_item;
+
+
 /* number of entries with "autoset empty tag to filename " feature */
 #define TM_NUM_TAGS_PREFS (5)
 
-extern const gchar *tm_col_strings[];
-extern const gchar *tm_col_tooltips[];
-
 /* "Column numbers" in playlist model */
 enum  {
   PM_COLUMN_PLAYLIST = 0,
@@ -183,9 +224,10 @@ enum  {
 
 /* Drag and drop types */
 enum {
-    DND_GTKPOD_IDLIST = 1000,
+    DND_GTKPOD_TRACKLIST = 1000,
     DND_GTKPOD_TM_PATHLIST,
-    DND_GTKPOD_PM_PATHLIST,
+    DND_GTKPOD_PLAYLISTLIST,
+    DND_TEXT_URI_LIST,
     DND_TEXT_PLAIN
 };
 
@@ -194,7 +236,7 @@ enum {
 #define SP_SHIFT 9
 #define SP_MASK ((1<<SP_SHIFT)-1)
 
-void display_create (GtkWidget *gtkpod);
+void display_create (void);
 void display_cleanup (void);
 void display_adjust_delete_menus (void);
 void display_disable_gtkpod_import_buttons(void);
@@ -203,10 +245,14 @@ void display_reset (gint inst);
 GList *display_get_selection (guint32 inst);
 GList *display_get_selected_members (gint inst);
 void display_enable_disable_view_sort (gboolean enable);
+void display_remove_autoscroll_row_timeout (GtkWidget *widget);
+void display_install_autoscroll_row_timeout (GtkWidget *widget);
 
 Playlist* pm_get_selected_playlist(void);
+GtkTreePath *pm_get_path (Playlist *pl);
 void pm_remove_playlist (Playlist *playlist, gboolean select);
 void pm_add_playlist (Playlist *playlist, gint position);
+void pm_add_itdb (iTunesDB *itdb, gint pos);
 void pm_select_playlist (Playlist *playlist);
 void pm_unselect_playlist (Playlist *playlist);
 void pm_remove_track (Playlist *playlist, Track *track);
@@ -215,8 +261,6 @@ void pm_name_changed (Playlist *playlist);
 void pm_track_changed (Track *track);
 void pm_sort (GtkSortType order);
 void pm_stop_editing (gboolean cancel);
-gboolean pm_move_pathlist (gchar *data, GtkTreePath *path,
-			   GtkTreeViewDropPosition pos);
 
 void st_stop_editing (gint inst, gboolean cancel);
 void st_page_selected (GtkNotebook *notebook, guint page);
@@ -232,18 +276,6 @@ void cal_open_calendar (gint inst, T_item item);
 void sp_go (guint32 inst);
 void sp_conditions_changed (guint32 inst);
 
-void on_tm_dnd_get_id_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
-			      GtkTreeIter *i, gpointer data);
-void on_dnd_get_path_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
-			     GtkTreeIter *i, gpointer data);
-void on_tm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
-				GtkTreeIter *i, gpointer data);
-void on_pm_dnd_get_id_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
-			      GtkTreeIter *i, gpointer data);
-void on_pm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
-				GtkTreeIter *i, gpointer data);
-void on_st_listing_drag_foreach(GtkTreeModel *tm, GtkTreePath *tp, 
-				GtkTreeIter *i, gpointer data);
 TimeInfo *sp_update_date_interval_from_string (guint32 inst,
 					       T_item item,
 					       gboolean force_update);
@@ -251,8 +283,6 @@ void tm_add_track_to_track_model (Track *track, GtkTreeIter *into_iter);
 guint tm_get_nr_of_tracks(void);
 void tm_rows_reordered(void);
 void pm_rows_reordered(void);
-gboolean tm_move_pathlist (gchar *data, GtkTreePath *path,
-			   GtkTreeViewDropPosition pos);
 gboolean tm_add_filelist (gchar *data, GtkTreePath *path,
 			  GtkTreeViewDropPosition pos);
 void tm_stop_editing (gboolean cancel);
@@ -261,6 +291,7 @@ void tm_store_col_order (void);
 void tm_adopt_order_in_sorttab (void);
 
 void tm_sort (TM_item col, GtkSortType order);
+gint tm_sort_counter (gint inc);
 GList* tm_get_selected_trackids(void);
 GList* tm_get_selected_tracks(void);
 GList* tm_get_all_trackids(void);
@@ -274,5 +305,5 @@ void display_set_check_ipod_menu (void);
 void display_stop_update (gint inst);
 
 void spl_edit (Playlist *spl);
-void spl_edit_new (gchar *name, gint32 pos);
+void spl_edit_new (iTunesDB *itdb, gchar *name, gint32 pos);
 #endif 
