@@ -1,8 +1,9 @@
-/* Time-stamp: <2005-01-03 17:52:00 jcs>
+/* Time-stamp: <2005-06-14 22:12:11 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_private.h,v 1.15 2004/11/19 16:40:02 jcsjcs Exp $
+|  $Id: display_private.h,v 1.21 2005/06/17 13:30:03 jcsjcs Exp $
 */
 
 #ifndef __DISPLAY_PRIVATE_H__
@@ -66,6 +67,7 @@ extern gint stop_add;
 void pm_create_treeview (void);
 void pm_set_selected_playlist(Playlist *pl);
 void pm_remove_all_playlists (gboolean clear_sort);
+void pm_add_all_playlists (void);
 void tm_create_treeview (void);
 void tm_track_changed (Track *track);
 void tm_remove_track (Track *track);
@@ -91,32 +93,14 @@ void tm_enable_disable_view_sort (gboolean enable);
 
 /* Drag and drop definitions */
 #define TGNR(a) (guint)(sizeof(a)/sizeof(GtkTargetEntry))
-#define DND_GTKPOD_IDLIST_TYPE "application/gtkpod-idlist"
+#define DND_GTKPOD_TRACKLIST_TYPE "application/gtkpod-tracklist"
 #define DND_GTKPOD_TM_PATHLIST_TYPE "application/gtkpod-tm_pathlist"
-#define DND_GTKPOD_PM_PATHLIST_TYPE "application/gtkpod-pm_pathlist"
+#define DND_GTKPOD_PLAYLISTLIST_TYPE "application/gtkpod-playlistlist"
+
 struct asf_data
 {
     GtkTreeIter *to_iter;
     GtkTreeViewDropPosition pos;
 };
 
-typedef enum
-{
-    TRACK_TREEVIEW,
-    PLAYLIST_TREEVIEW,
-    SORTTAB_TREEVIEW
-} TreeViewType;
-
-void  tm_list_store_move_before (GtkListStore *store,
-				 GtkTreeIter  *iter,
-				 GtkTreeIter  *position);
-void  tm_list_store_move_after (GtkListStore *store,
-				GtkTreeIter  *iter,
-				GtkTreeIter  *position);
-gboolean pmtm_move_pathlist (GtkTreeView *treeview,
-			     gchar *data,
-			     GtkTreePath *path,
-			     GtkTreeViewDropPosition pos,
-			     TreeViewType tvt);
-
 #endif /* __DISPLAY_PRIVATE_H__ */
