@@ -1,6 +1,6 @@
-/* Time-stamp: <2005-03-07 23:08:49 jcs>
+/* Time-stamp: <2005-05-21 11:27:14 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users.sourceforge.net>
 |  Part of the gtkpod project.
 |
 |  URL: http://gtkpod.sourceforge.net/
@@ -23,7 +23,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: display_spl.c,v 1.2 2004/12/13 15:57:05 jcsjcs Exp $
+|  $Id: display_spl.c,v 1.9 2005/05/21 04:47:23 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -35,12 +35,10 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "itunesdb.h"
+#include "itdb.h"
 #include "misc.h"
 #include "display.h"
 #include "prefs.h"
-#include "support.h"
-#include "interface.h"
 
 
 
@@ -58,6 +56,7 @@ typedef struct
     const gchar *str;
 } ComboEntry;
 
+GladeXML *spl_window_xml;
 
 static const ComboEntry splat_inthelast_units_comboentries[] =
 {
@@ -249,6 +248,21 @@ static void spl_set_combobox (GtkComboBox *cb,
     if (g_object_get_data (G_OBJECT (cb), "combo_set") == NULL)
     {   /* the combo has not yet been initialized */
 	const ComboEntry *ce = centries;
+	GtkCellRenderer *cell;
+	GtkListStore *store;
+
+	/* since the transition to libglade we have to set the model
+	   ourselves */
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_combo_box_set_model (cb, GTK_TREE_MODEL (store));
+	g_object_unref (store);
+
+	cell = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (cb), cell, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (cb), cell,
+					"text", 0,
+					NULL);
+
 	while (ce->str != NULL)
 	{
 	    gtk_combo_box_append_text (cb, _(ce->str));
@@ -275,7 +289,7 @@ static void spl_all_radio_toggled (GtkToggleButton *togglebutton,
     g_return_if_fail (spl);
     if (gtk_toggle_button_get_active (togglebutton))
     {
-	GtkWidget *frame = lookup_widget (spl_window, "spl_rules_frame");
+	GtkWidget *frame = glade_xml_get_widget (spl_window_xml, "spl_rules_frame");
 	g_return_if_fail (frame);
 	gtk_widget_set_sensitive (frame, TRUE);
 	spl->splpref.checkrules = TRUE;
@@ -293,7 +307,7 @@ static void spl_any_radio_toggled (GtkToggleButton *togglebutton,
     g_return_if_fail (spl);
     if (gtk_toggle_button_get_active (togglebutton))
     {
-	GtkWidget *frame = lookup_widget (spl_window, "spl_rules_frame");
+	GtkWidget *frame = glade_xml_get_widget (spl_window_xml, "spl_rules_frame");
 	g_return_if_fail (frame);
 	gtk_widget_set_sensitive (frame, TRUE);
 	spl->splpref.checkrules = TRUE;
@@ -310,7 +324,7 @@ static void spl_none_radio_toggled (GtkToggleButton *togglebutton,
     g_return_if_fail (spl_window);
     spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
     g_return_if_fail (spl);
-    frame = lookup_widget (spl_window, "spl_rules_frame");
+    frame = glade_xml_get_widget (spl_window_xml, "spl_rules_frame");
     g_return_if_fail (frame);
     if (gtk_toggle_button_get_active (togglebutton))
     {
@@ -514,7 +528,7 @@ static void splr_entry_changed (GtkEditable *editable,
     case spl_ET_FROMVALUE_DATE:
 	t = time_string_to_fromtime (str);
 	if (t != -1)
-	    splr->fromvalue = itunesdb_time_host_to_mac (t);
+	    splr->fromvalue = itdb_time_host_to_mac (t);
 	break;
     case spl_ET_FROMDATE:
 	splr->fromdate = atol (str);
@@ -529,7 +543,7 @@ static void splr_entry_changed (GtkEditable *editable,
     case spl_ET_TOVALUE_DATE:
 	t = time_string_to_totime (str);
 	if (t != -1)
-	    splr->tovalue = itunesdb_time_host_to_mac (t);
+	    splr->tovalue = itdb_time_host_to_mac (t);
 	break;
     case spl_ET_TODATE:
 	splr->todate = atol (str);
@@ -540,9 +554,6 @@ static void splr_entry_changed (GtkEditable *editable,
     case spl_ET_STRING:
 	g_free (splr->string);
 	splr->string = g_strdup (str);
-	g_free (splr->string_utf16);
-	splr->string_utf16 = g_utf8_to_utf16 (str, -1,
-					      NULL, NULL, NULL);
 	break;
     default:
 	/* must not happen */
@@ -611,7 +622,7 @@ static void spl_button_minus_clicked (GtkButton *button,
     row = g_list_index (spl->splrules.rules, splr);
     g_return_if_fail (row != -1);
 
-    splr_remove (spl, splr);
+    itdb_splr_remove (spl, splr);
     spl_update_rules_from_row (spl_window, row);
 }
 
@@ -634,7 +645,7 @@ static void spl_button_plus_clicked (GtkButton *button,
     row = g_list_index (spl->splrules.rules, splr);
     g_return_if_fail (row != -1);
 
-    splr_add_new (spl, row+1);
+    itdb_splr_add_new (spl, row+1);
     spl_update_rules_from_row (spl_window, row+1);
 }
 
@@ -645,11 +656,18 @@ static void spl_cancel (GtkButton *button, GtkWidget *spl_window)
 					   "spl_work");
     Playlist *spl_orig = g_object_get_data (G_OBJECT (spl_window),
 					    "spl_orig");
-    pl_free (spl_dup);
+    iTunesDB *itdb = g_object_get_data (G_OBJECT (spl_window),
+					"spl_itdb");
+
+    g_return_if_fail (spl_dup != NULL);
+    g_return_if_fail (spl_orig != NULL);
+    g_return_if_fail (itdb != NULL);
+
+    itdb_playlist_free (spl_dup);
     /* does playlist already exist in display? */
-    if (!playlist_exists (spl_orig))
+    if (!itdb_playlist_exists (itdb, spl_orig))
     {   /* Delete */
-	pl_free (spl_orig);
+	itdb_playlist_free (spl_orig);
     }
     gtk_widget_destroy (spl_window);
 
@@ -670,33 +688,30 @@ static void spl_ok (GtkButton *button, GtkWidget *spl_window)
     GtkWidget *w;
     Playlist *spl_dup;
     Playlist *spl_orig;
-    Playlist *mpl;
+    iTunesDB *itdb;
     gint32 pos;
 
-    g_return_if_fail (spl_window != NULL);
+    g_return_if_fail (spl_window_xml != NULL);
 
     spl_dup = g_object_get_data (G_OBJECT (spl_window), "spl_work");
     spl_orig = g_object_get_data (G_OBJECT (spl_window), "spl_orig");
     pos =  (gint32)g_object_get_data (G_OBJECT (spl_window), "spl_pos");
+    itdb = g_object_get_data (G_OBJECT (spl_window), "spl_itdb");
 
     g_return_if_fail (spl_dup != NULL);
     g_return_if_fail (spl_orig != NULL);
+    g_return_if_fail (itdb != NULL);
 
     /* Read out new playlist name */
-    if ((w = lookup_widget (spl_window, "spl_name_entry")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_name_entry")))
     {
 	g_free (spl_orig->name);
 	spl_orig->name = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
-	g_free (spl_orig->name_utf16);
-	spl_orig->name_utf16 = g_utf8_to_utf16 (spl_orig->name, -1,
-						NULL, NULL, NULL);
     }
 
-    pl_copy_spl_rules (spl_orig, spl_dup);
+    itdb_spl_copy_rules (spl_orig, spl_dup);
 
-    mpl = get_playlist_by_nr (0);  /* contains all tracks */
-
-    spl_update (spl_orig, mpl->members);
+    itdb_spl_update (itdb, spl_orig);
 
     if (pm_get_selected_playlist () == spl_orig)
     {   /* redisplay */
@@ -704,13 +719,13 @@ static void spl_ok (GtkButton *button, GtkWidget *spl_window)
 	pm_select_playlist (spl_orig);
     }
 
-    /* does playlist already exist in display? */
-    if (!playlist_exists (spl_orig))
+    /* does playlist already exist in itdb? */
+    if (!itdb_playlist_exists (itdb, spl_orig))
     {   /* Insert at specified position */
-	add_playlist (spl_orig, pos);
+	gp_playlist_add (itdb, spl_orig, pos);
     }
 
-    pl_free (spl_dup);
+    itdb_playlist_free (spl_dup);
     gtk_widget_destroy (spl_window);
 
     release_widgets ();
@@ -727,7 +742,7 @@ static void spl_display_checklimits (GtkWidget *spl_window)
     spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
     g_return_if_fail (spl);
 
-    if ((w = lookup_widget (spl_window, "spl_checklimits_button")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_checklimits_button")))
     {
 	gtk_toggle_button_set_active (
 	    GTK_TOGGLE_BUTTON (w), spl->splpref.checklimits);
@@ -736,7 +751,7 @@ static void spl_display_checklimits (GtkWidget *spl_window)
 			  spl_window);
     }
 
-    if ((w = lookup_widget (spl_window, "spl_limitvalue_entry")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_limitvalue_entry")))
     {
 	gchar str[WNLEN];
 	snprintf (str, WNLEN, "%d", spl->splpref.limitvalue);
@@ -747,7 +762,7 @@ static void spl_display_checklimits (GtkWidget *spl_window)
 			  spl_window);
     }
 
-    if ((w = lookup_widget (spl_window, "spl_limittype_combobox")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_limittype_combobox")))
     {
 	spl_set_combobox (GTK_COMBO_BOX (w), 
 			  limittype_comboentries,
@@ -757,12 +772,12 @@ static void spl_display_checklimits (GtkWidget *spl_window)
 	gtk_widget_set_sensitive (w, spl->splpref.checklimits);
     }
 
-    if ((w = lookup_widget (spl_window, "spl_limitsort_label")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_limitsort_label")))
     {
 	gtk_widget_set_sensitive (w, spl->splpref.checklimits);
     }
 
-    if ((w = lookup_widget (spl_window, "spl_limitsort_combobox")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_limitsort_combobox")))
     {
 	spl_set_combobox (GTK_COMBO_BOX (w), 
 			  limitsort_comboentries,
@@ -800,7 +815,7 @@ void set_timestring (gchar *str, guint64 value, enum entrytype et)
 
     g_return_if_fail (str != NULL);
 
-    t = itunesdb_time_mac_to_host (value);
+    t = itdb_time_mac_to_host (value);
     if (et == spl_ET_FROMVALUE_DATE)
     {
 	resstr = time_fromtime_to_string (t);
@@ -932,9 +947,11 @@ GtkWidget *spl_create_hbox (GtkWidget *spl_window, SPLRule *splr)
     GtkWidget *hbox = NULL;
     SPLActionType at;
     GtkWidget *entry, *label, *combobox;
-    gint index, i, n;
+    gint index;
     GArray *pl_ids = NULL;
     Playlist *spl_orig;
+    iTunesDB *itdb;
+    GList *gl;
 
 
     g_return_val_if_fail (spl_window, NULL);
@@ -943,7 +960,10 @@ GtkWidget *spl_create_hbox (GtkWidget *spl_window, SPLRule *splr)
     spl_orig =  g_object_get_data (G_OBJECT (spl_window), "spl_orig");
     g_return_val_if_fail (spl_orig, NULL);
 
-    at = itb_splr_get_action_type (splr);
+    itdb =  g_object_get_data (G_OBJECT (spl_window), "spl_itdb");
+    g_return_val_if_fail (itdb, NULL);
+
+    at = itdb_splr_get_action_type (splr);
     g_return_val_if_fail (at != splat_unknown, NULL);
     g_return_val_if_fail (at != splat_invalid, NULL);
 
@@ -1011,7 +1031,7 @@ GtkWidget *spl_create_hbox (GtkWidget *spl_window, SPLRule *splr)
 	    splr->fromvalue *= ((double)units)/splr->fromunits;
 	}
 	entry = hbox_add_entry (hbox, splr, spl_ET_INTHELAST);
-	combobox = gtk_combo_box_new_text ();
+	combobox = gtk_combo_box_new ();
 	gtk_widget_show (combobox);
 	gtk_box_pack_start (GTK_BOX (hbox), combobox, TRUE, TRUE, 0);
 	g_object_set_data (G_OBJECT (combobox), "spl_rule", splr);
@@ -1025,17 +1045,20 @@ GtkWidget *spl_create_hbox (GtkWidget *spl_window, SPLRule *splr)
 	combobox = gtk_combo_box_new_text ();
 	gtk_widget_show (combobox);
 	gtk_box_pack_start (GTK_BOX (hbox), combobox, TRUE, TRUE, 0);
-	n = get_nr_of_playlists ();
-	pl_ids = g_array_sized_new (TRUE, TRUE, sizeof (guint64), n);
-	for (i=1; i<n; ++i)
+	pl_ids = g_array_sized_new (TRUE, TRUE, sizeof (guint64), 
+				    itdb_playlists_number (itdb));
+	gl=itdb->playlists;
+	while (gl && gl->next)
 	{
-	    Playlist *pl = get_playlist_by_nr (i);
+	    Playlist *pl = gl->next->data;
+	    g_return_val_if_fail (pl, NULL);
 	    if (pl != spl_orig)
 	    {
 		gtk_combo_box_append_text (GTK_COMBO_BOX (combobox),
 					   pl->name);
 		g_array_append_val (pl_ids, pl->id);
 	    }
+	    gl = gl->next;
 	}
 	g_object_set_data (G_OBJECT (combobox), "spl_rule", splr);
 	g_object_set_data_full (G_OBJECT (combobox), "spl_pl_ids",
@@ -1078,7 +1101,7 @@ static void spl_update_rule (GtkWidget *spl_window, SPLRule *splr)
 
     spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
     g_return_if_fail (spl);
-    table = GTK_TABLE (lookup_widget (spl_window, "spl_rules_table"));
+    table = g_object_get_data (G_OBJECT (spl_window), "spl_rules_table");
     g_return_if_fail (table);
 
     row = g_list_index (spl->splrules.rules, splr);
@@ -1091,7 +1114,7 @@ static void spl_update_rule (GtkWidget *spl_window, SPLRule *splr)
     combobox = g_object_get_data (G_OBJECT (table), name);
     if (!combobox)
     {  /* create combo for field */
-	combobox = gtk_combo_box_new_text ();
+	combobox = gtk_combo_box_new ();
 	gtk_widget_show (combobox);
 	gtk_table_attach (table, combobox, 0,1, row,row+1,
 			  0,0,         /* expand options */
@@ -1107,7 +1130,7 @@ static void spl_update_rule (GtkWidget *spl_window, SPLRule *splr)
 
     /* Combobox for action */
     /* ------------------- */
-    ft = itb_splr_get_field_type (splr);
+    ft = itdb_splr_get_field_type (splr);
     g_return_if_fail (ft != splft_unknown);
     snprintf (name, WNLEN, "spl_actioncombo%d", row);
     combobox = g_object_get_data (G_OBJECT (table), name);
@@ -1149,7 +1172,7 @@ static void spl_update_rule (GtkWidget *spl_window, SPLRule *splr)
     }
     if (!combobox)
     {  /* create combo for action */
-	combobox = gtk_combo_box_new_text ();
+	combobox = gtk_combo_box_new ();
 	gtk_widget_show (combobox);
 	gtk_table_attach (table, combobox, 1,2, row,row+1,
 			  GTK_FILL,0,   /* expand options */
@@ -1167,7 +1190,7 @@ static void spl_update_rule (GtkWidget *spl_window, SPLRule *splr)
 
     /* input fields (range, string, date...) */
     /* ------------------------------------- */
-    at = itb_splr_get_action_type (splr);
+    at = itdb_splr_get_action_type (splr);
     g_return_if_fail (at != splat_unknown);
     g_return_if_fail (at != splat_invalid);
     snprintf (name, WNLEN, "spl_actionhbox%d", row);
@@ -1224,7 +1247,7 @@ static void spl_display_rules (GtkWidget *spl_window)
     g_return_if_fail (spl_window);
     spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
     g_return_if_fail (spl);
-    align = lookup_widget (spl_window, "spl_rules_table_align");
+    align = glade_xml_get_widget (spl_window_xml, "spl_rules_table_align");
     g_return_if_fail (align);
     /* Destroy table if it already exists */
     table = g_object_get_data (G_OBJECT (spl_window),
@@ -1272,7 +1295,7 @@ static void spl_update_rules_from_row (GtkWidget *spl_window, gint row)
     g_return_if_fail (spl_window);
     spl =  g_object_get_data (G_OBJECT (spl_window), "spl_work");
     g_return_if_fail (spl);
-    table = lookup_widget (spl_window, "spl_rules_table");
+    table = g_object_get_data (G_OBJECT (spl_window), "spl_rules_table");
     g_return_if_fail (table);
 
     numrules = g_list_length (spl->splrules.rules);
@@ -1296,32 +1319,36 @@ static void spl_update_rules_from_row (GtkWidget *spl_window, gint row)
 
 /* Edit a smart playlist. If it is a new smartlist, it will be
  * inserted at position @pos when 'OK' is pressed. */
-void spl_edit_all (Playlist *spl, gint32 pos)
+void spl_edit_all (iTunesDB *itdb, Playlist *spl, gint32 pos)
 {
     GtkWidget *spl_window, *w;
     Playlist *spl_dup;
 
     g_return_if_fail (spl != NULL);
     g_return_if_fail (spl->is_spl);
+    g_return_if_fail (itdb != NULL);
 
-    spl_window = create_spl_window ();
-    g_assert (spl_window != NULL);
+    spl_window_xml = glade_xml_new (xml_file, "spl_window", NULL);
+    spl_window = glade_xml_get_widget (spl_window_xml, "spl_window");
+    
+    g_return_if_fail (spl_window != NULL);
 
     /* Duplicate playlist to work on */
-    spl_dup = pl_duplicate (spl);
+    spl_dup = itdb_playlist_duplicate (spl);
 
     /* Store pointers to original playlist and duplicate */
     g_object_set_data (G_OBJECT (spl_window), "spl_orig", spl);
     g_object_set_data (G_OBJECT (spl_window), "spl_work", spl_dup);
     g_object_set_data (G_OBJECT (spl_window), "spl_pos", (gpointer)pos);
+    g_object_set_data (G_OBJECT (spl_window), "spl_itdb", itdb);
     /* Set checkboxes and connect signal handlers */
-    if ((w = lookup_widget (spl_window, "spl_name_entry")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_name_entry")))
     {
 	if (spl_dup->name)
 	    gtk_entry_set_text (GTK_ENTRY (w), spl_dup->name);	
     }
 
-    if ((w = lookup_widget (spl_window, "spl_all_radio")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_all_radio")))
     {
 	gtk_toggle_button_set_active (
 	    GTK_TOGGLE_BUTTON (w),
@@ -1330,7 +1357,7 @@ void spl_edit_all (Playlist *spl, gint32 pos)
 			  G_CALLBACK (spl_all_radio_toggled),
 			  spl_window);
     }
-    if ((w = lookup_widget (spl_window, "spl_any_radio")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_any_radio")))
     {
 	gtk_toggle_button_set_active (
 	    GTK_TOGGLE_BUTTON (w),
@@ -1339,7 +1366,7 @@ void spl_edit_all (Playlist *spl, gint32 pos)
 			  G_CALLBACK (spl_any_radio_toggled),
 			  spl_window);
     }
-    if ((w = lookup_widget (spl_window, "spl_none_radio")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_none_radio")))
     {
 	gtk_toggle_button_set_active (
 	    GTK_TOGGLE_BUTTON (w), !spl_dup->splpref.checkrules);
@@ -1348,7 +1375,7 @@ void spl_edit_all (Playlist *spl, gint32 pos)
 			  spl_window);
     }
 
-    if ((w = lookup_widget (spl_window, "spl_matchcheckedonly_button")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_matchcheckedonly_button")))
     {
 	gtk_toggle_button_set_active (
 	    GTK_TOGGLE_BUTTON (w), spl_dup->splpref.matchcheckedonly);
@@ -1357,7 +1384,7 @@ void spl_edit_all (Playlist *spl, gint32 pos)
 			  spl_window);
     }
 
-    if ((w = lookup_widget (spl_window, "spl_liveupdate_button")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_liveupdate_button")))
     {
 	gtk_toggle_button_set_active (
 	    GTK_TOGGLE_BUTTON (w), spl_dup->splpref.liveupdate);
@@ -1367,12 +1394,12 @@ void spl_edit_all (Playlist *spl, gint32 pos)
     }
 
     /* Signals for Cancel, OK, Delete */
-    if ((w = lookup_widget (spl_window, "spl_cancel_button")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_cancel_button")))
     {
 	g_signal_connect (w, "clicked",
 		      G_CALLBACK (spl_cancel), spl_window);
     }
-    if ((w = lookup_widget (spl_window, "spl_ok_button")))
+    if ((w = glade_xml_get_widget (spl_window_xml, "spl_ok_button")))
     {
 	g_signal_connect (w, "clicked",
 		      G_CALLBACK (spl_ok), spl_window);
@@ -1393,15 +1420,17 @@ void spl_edit_all (Playlist *spl, gint32 pos)
 /* Edit an existing smart playlist */
 void spl_edit (Playlist *spl)
 {
-    spl_edit_all (spl, -1);
+    g_return_if_fail (spl);
+    g_return_if_fail (spl->itdb);
+    spl_edit_all (spl->itdb, spl, -1);
 }
 
 
 /* Edit a non-existing smartlist. If successful, it will be entered at
    position @pos. Default name is @name */
-void spl_edit_new (gchar *name, gint32 pos)
+void spl_edit_new (iTunesDB *itdb, gchar *name, gint32 pos)
 {
-    Playlist *spl = pl_new (name? name:_("New Playlist"), TRUE);
+    Playlist *spl = gp_playlist_new (name? name:_("New Playlist"), TRUE);
 
-    spl_edit_all (spl, pos);
+    spl_edit_all (itdb, spl, pos);
 }
