@@ -1,8 +1,10 @@
-/* Time-stamp: <2004-08-22 13:45:06 jcs>
+/* Time-stamp: <2005-06-25 11:14:46 jcs>
 |
 |  Copyright (C) 2002 Corey Donohoe <atmos at atmos.org>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +25,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: file_export.c,v 1.37 2004/08/22 09:33:43 jcsjcs Exp $
+|  $Id: file_export.c,v 1.50 2005/06/25 02:52:25 jcsjcs Exp $
 */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -33,8 +35,8 @@
 #include "prefs.h"
 #include "file.h"
 #include "charset.h"
-#include "interface.h"
-#include "support.h"
+#include "info.h"
+#include "md5.h"
 #include <limits.h>
 #include <stdio.h>
 #include <errno.h>
@@ -47,10 +49,11 @@
 /* Structure to keep all necessary information */
 struct fcd
 {
-    GList *tracks;  /* tracks to be written */
-    GtkWidget *fc;  /* file chooser */
-    GtkWidget *win; /* glade widget to lookup widgets by name */
-    gpointer user_data; 
+    GList *tracks;     /* tracks to be written */
+    GtkWidget *fc;     /* file chooser */
+    GList **filenames; /* pointer to GList to append the filenames used */
+    GladeXML *win_xml; /* Glade xml reference */
+    Track *track;      /* current track to export */
 };
 
 
@@ -72,8 +75,6 @@ const gchar *EXPORT_FILES_TPL="export_files_template";
 const gchar *EXPORT_FILES_TPL_DFLT="%o;%a - %t.mp3;%t.wav";
 
 
-
-
 /**
  * READ_WRITE_BLOCKSIZE - how many bytes we read per fread/fwrite call
  */
@@ -197,7 +198,19 @@ copy_file(gchar *file, gchar *dest)
     prefs_get_int_value (EXPORT_FILES_CHECK_EXISTING, &check_existing);
     
     if(check_existing && file_is_ok(file, dest))
+    {
+	gchar *buf = g_strdup_printf (_("Skipping existing file with same length: '%s'\n"), dest);
+	gtkpod_warning (buf);
+	g_free (buf);
     	return TRUE;
+    }
+
+    if (g_file_test (dest, G_FILE_TEST_EXISTS))
+    {
+	gchar *buf = g_strdup_printf (_("Overwriting existing file: '%s'\n"), dest);
+	gtkpod_warning (buf);
+	g_free (buf);
+    }
 	
     if((from = fopen(file, "r")))
     {
@@ -279,39 +292,70 @@ track_get_export_filename (Track *track)
  * @s - The Track reference we're manipulating
  * Returns - TRUE on success, FALSE on failure
  */
-static gboolean
-write_track(Track *s)
+static gboolean write_track (struct fcd *fcd)
 {
-    gchar *dest_file = NULL;
     gboolean result = FALSE;
+    gchar *dest_file = track_get_export_filename (fcd->track);
+
+    g_return_val_if_fail (fcd, FALSE);
+    g_return_val_if_fail (fcd->track, FALSE);
+    g_return_val_if_fail (fcd->track->itdb, FALSE);
     
-    if((dest_file = track_get_export_filename(s)))
+    if(dest_file)
     {
-	gchar *from_file = get_track_name_on_disk(s);
-	gchar *filename, *dest_dir;
+	gchar *from_file = NULL;
+	if (fcd->track->itdb->usertype & GP_ITDB_TYPE_IPOD)
+	{
+	    from_file = get_file_name_on_ipod (fcd->track);
+	}
+	else if (fcd->track->itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	{
+	    from_file = get_file_name_on_harddisk (fcd->track);
+	}
+	else
+	{
+	    g_return_val_if_reached (FALSE);
+	}
+	if (from_file)
+	{
+	    gchar *filename, *dest_dir;
+	    prefs_get_string_value (EXPORT_FILES_PATH, &dest_dir);
+	    filename = g_build_filename (dest_dir, dest_file, NULL);
 
-	prefs_get_string_value (EXPORT_FILES_PATH, &dest_dir);
-	filename = g_build_filename (dest_dir, dest_file, NULL);
-
-	if (mkdirhier(filename)) {
-	    if(copy_file(from_file, filename))
-	        result = TRUE;
+	    if (mkdirhier(filename))
+	    {
+		if(copy_file(from_file, filename))
+		{
+		    result = TRUE;
+		    if (fcd->filenames)
+		    {   /* append filename to list */
+			*fcd->filenames = g_list_append (*fcd->filenames,
+							 filename);
+			filename = NULL;
+		    }
+		}
+	    }
+	    g_free(from_file);
+	    g_free(dest_dir);
+	    g_free(filename);
+	}
+	else
+	{
+	    gtkpod_warning (_("Could find file for '%s' on the iPod\n"),
+			    get_track_info (fcd->track, FALSE));
 	}
-	g_free(from_file);
 	g_free(dest_file);
-	g_free(dest_dir);
-	g_free(filename);
     }
     return(result);
 }
 
 #ifdef G_THREADS_ENABLED
 /* Threaded write_track */
-static gpointer th_write_track (gpointer s)
+static gpointer th_write_track (gpointer fcd)
 {
     gboolean result = FALSE;
 
-    result = write_track ((Track *)s);
+    result = write_track (fcd);
 
     g_mutex_lock (mutex);
     mutex_data = TRUE;   /* signal that thread will end */
@@ -340,7 +384,9 @@ static void export_files_write (struct fcd *fcd)
     if (!cond) cond = g_cond_new ();
 #endif
 
-    g_return_if_fail (fcd && fcd->win && fcd->fc);
+    g_return_if_fail (fcd && fcd->fc);
+
+    block_widgets ();
 
     abort = FALSE;
     n = g_list_length (fcd->tracks);
@@ -406,13 +452,14 @@ static void export_files_write (struct fcd *fcd)
 	start = time(NULL);
 	for(l = fcd->tracks; l && !abort; l = l->next)
 	{
-	    Track *s = (Track*)l->data;
+	    Track *tr = (Track*)l->data;
 	    gchar *buf;
 
-	    copied += s->size;
+	    fcd->track = tr;
+	    copied += tr->size;
 #ifdef G_THREADS_ENABLED
 	    mutex_data = FALSE;
-	    thread = g_thread_create (th_write_track, s, TRUE, NULL);
+	    thread = g_thread_create (th_write_track, fcd, TRUE, NULL);
 	    if (thread)
 	    {
 		g_mutex_lock (mutex);
@@ -430,14 +477,16 @@ static void export_files_write (struct fcd *fcd)
 	    }
 	    else {
 		g_warning ("Thread creation failed, falling back to default.\n");
-		result &= write_track (s);
+		result &= write_track (fcd);
 	    }
 #else
-	    result &= write_track (s);
+	    result &= write_track (fcd);
+	    while (widgets_blocked && gtk_events_pending ())
+		gtk_main_iteration ();
 #endif
 	    if (!result)
 	    {
-		gtkpod_warning (_("Failed to write '%s-%s'\n"), s->artist, s->title);	
+		gtkpod_warning (_("Failed to write '%s-%s'\n"), tr->artist, tr->title);	
 	    }
 
 	    ++count;
@@ -474,17 +523,6 @@ static void export_files_write (struct fcd *fcd)
 	if (!result || abort)
 	    gtkpod_statusbar_message (_("Some tracks were not copied."));
     }
-}
-
-/******************************************************************
-   export_files_cleanup - free memory taken up by the fcd structure.
- ******************************************************************/
-static void export_files_cleanup (struct fcd *fcd)
-{
-    g_return_if_fail (fcd);
-    g_list_free (fcd->tracks);
-    if (fcd->win)  gtk_widget_destroy (fcd->win);
-    g_free (fcd);
     release_widgets ();
 }
 
@@ -493,109 +531,338 @@ static void export_files_cleanup (struct fcd *fcd)
    export_files_retrieve_options - retrieve options and store
    them in the prefs.
  ******************************************************************/
-static void export_files_retrieve_options (struct fcd *fcd)
+static void export_files_store_option_settings (struct fcd *fcd)
 {
-    g_return_if_fail (fcd && fcd->win && fcd->fc);
+    g_return_if_fail (fcd && fcd->win_xml && fcd->fc);
 
-    option_get_toggle_button (fcd->win,
+    option_get_toggle_button (fcd->win_xml,
 			      EXPORT_FILES_SPECIAL_CHARSET);
-    option_get_toggle_button (fcd->win,
+    option_get_toggle_button (fcd->win_xml,
 			      EXPORT_FILES_CHECK_EXISTING);
-    option_get_string (fcd->win, EXPORT_FILES_TPL, NULL);
+    option_get_string (fcd->win_xml, EXPORT_FILES_TPL, NULL);
     option_get_filename (GTK_FILE_CHOOSER (fcd->fc),
 			 EXPORT_FILES_PATH, NULL);
 }
 
 
-/******************************************************************
-   export_files_response - handle the response codes accordingly.
- ******************************************************************/
-static void export_files_response (GtkDialog *fc,
-				   gint response,
-				   struct fcd *fcd)
-{
-/*     printf ("received response code: %d\n", response); */
-    switch (response)
-    {
-    case RESPONSE_APPLY:
-	export_files_retrieve_options (fcd);
-	break;
-    case GTK_RESPONSE_ACCEPT:
-	export_files_retrieve_options (fcd);
-	export_files_write (fcd);
-	export_files_cleanup (fcd);
-	gtk_widget_destroy (GTK_WIDGET (fc));
-	break;
-    case GTK_RESPONSE_CANCEL:
-	export_files_cleanup (fcd);
-	gtk_widget_destroy (GTK_WIDGET (fc));
-	break;
-    case GTK_RESPONSE_DELETE_EVENT:
-	export_files_cleanup (fcd);
-	break;
-    default:
-	fprintf (stderr, "Programming error: export_files_response(): unknown response '%d'\n", response);
-	break;
-    }
-}
-
-
 /******************************************************************
    export_files_init - Export files off of your ipod to an arbitrary
    directory, specified by the file chooser dialog
-   @tracks - GList with data of type (Track*) we want to write 
+
+   @tracks    - GList with data of type (Track*) we want to write 
+   @filenames - a pointer to a GList where to store the filenames used
+                to write the tracks (or NULL)
+   @display   - TRUE: display a list of tracks to be exported
+   @message   - message to be displayed above the display of tracks
+
  ******************************************************************/
-void export_files_init (GList *tracks)
+void export_files_init (GList *tracks, GList **filenames,
+			gboolean display, gchar *message)
 {
-    GtkWidget *win = create_export_files_options ();
-    GtkWidget *options = lookup_widget (win, "options_frame");
-    struct fcd *fcd = g_malloc0 (sizeof (struct fcd));
-    GtkWidget *fc = gtk_file_chooser_dialog_new (
+    gint response;
+    GtkWidget *win, *options, *message_box; 
+    struct fcd *fcd;
+    GtkWidget *fc;
+    GladeXML *export_files_xml;
+
+    /* no export possible if in offline mode */
+    if (tracks && prefs_get_offline ())
+    {
+	Track *tr = tracks->data;
+	g_return_if_fail (tr);
+	g_return_if_fail (tr->itdb);
+	if (tr->itdb->usertype & GP_ITDB_TYPE_IPOD)
+	{
+	    GtkWidget *dialog = gtk_message_dialog_new (
+		GTK_WINDOW (gtkpod_window),
+		GTK_DIALOG_DESTROY_WITH_PARENT,
+		GTK_MESSAGE_WARNING,
+		GTK_BUTTONS_OK,
+		_("Export from iPod database not possible in offline mode."));
+	    gtk_dialog_run (GTK_DIALOG (dialog));
+	    gtk_widget_destroy (dialog);
+	    return;
+	}
+    }
+
+    fc = gtk_file_chooser_dialog_new (
 	_("Select Export Destination Directory"),
 	NULL,
 	GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-	GTK_STOCK_APPLY, RESPONSE_APPLY,
 	GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 	NULL);
 
+    export_files_xml = glade_xml_new (xml_file, "export_files_options", NULL);
+    win = glade_xml_get_widget (export_files_xml, "export_files_options");
+    options = glade_xml_get_widget (export_files_xml, "options_frame");
+    message_box = glade_xml_get_widget (export_files_xml, "message_box");
+
     /* Information needed to clean up later */
+    fcd = g_malloc0 (sizeof (struct fcd));
     fcd->tracks = g_list_copy (tracks);
-    fcd->win = win;
+    fcd->win_xml = export_files_xml;
+    fcd->filenames = filenames;
     fcd->fc = fc;
 
     /* according to GTK FAQ: move a widget to a new parent */
     gtk_widget_ref (options);
     gtk_container_remove (GTK_CONTAINER (win), options);
-    gtk_widget_unref (options);
-
     /* set extra options */
     gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc),
 				       options);
+    gtk_widget_unref (options);
+
+    gtk_widget_destroy (win);
+
+
+    /* set message text */
+    if (display)
+    {
+	GList *gl;
+	GtkWidget *label = glade_xml_get_widget (export_files_xml,
+						 "message");
+	GtkWidget *tv = glade_xml_get_widget (export_files_xml,
+					      "textview");
+	GtkTextBuffer *tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW(tv));
+	if (message)  gtk_label_set_text (GTK_LABEL (label), message);
+	else          gtk_widget_hide (label);
+	if (!tb)
+	{   /* set up textbuffer */
+	    tb = gtk_text_buffer_new (NULL);
+	    gtk_text_view_set_buffer (GTK_TEXT_VIEW (tv), tb);
+	    gtk_text_view_set_editable (GTK_TEXT_VIEW (tv), FALSE);
+	    gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW(tv), FALSE);
+	}
+	for (gl=tracks; gl; gl=gl->next)
+	{
+	    GtkTextIter ti;
+	    gchar *text;
+	    Track *tr = gl->data;
+	    g_return_if_fail (tr);
+
+	    text = get_track_info (tr, FALSE);
+	    /* append info to the end, incl. newline */
+	    gtk_text_buffer_get_end_iter (tb, &ti);
+	    gtk_text_buffer_insert (tb, &ti, text, -1);
+	    gtk_text_buffer_get_end_iter (tb, &ti);
+	    gtk_text_buffer_insert (tb, &ti, "\n", -1);
+	    g_free (text);
+	}
+    }
+    else
+    {
+	gtk_widget_hide (message_box);
+    }
 
     /* set last folder */
     option_set_folder (GTK_FILE_CHOOSER (fc),
 		       EXPORT_FILES_PATH);
     /* set toggle button "charset" */
-    option_set_toggle_button (win, EXPORT_FILES_SPECIAL_CHARSET, FALSE);
+    option_set_toggle_button (export_files_xml, EXPORT_FILES_SPECIAL_CHARSET, FALSE);
     /* set toggle button "check for existing files" */
-    option_set_toggle_button (win, EXPORT_FILES_CHECK_EXISTING, TRUE);
+    option_set_toggle_button (export_files_xml, EXPORT_FILES_CHECK_EXISTING, TRUE);
 
     /* set last template */
-    option_set_string (win, EXPORT_FILES_TPL, EXPORT_FILES_TPL_DFLT);
+    option_set_string (export_files_xml, EXPORT_FILES_TPL, EXPORT_FILES_TPL_DFLT);
 
-    /* catch response codes */
-    g_signal_connect (fc, "response",
-		      G_CALLBACK (export_files_response),
-		      fcd);
+    response = gtk_dialog_run (GTK_DIALOG (fc));
 
-    gtk_widget_show (fc);
-    block_widgets ();
+    switch (response)
+    {
+    case GTK_RESPONSE_ACCEPT:
+	export_files_store_option_settings (fcd);
+	export_files_write (fcd);
+	break;
+    case GTK_RESPONSE_CANCEL:
+	break;
+    default:
+	break;
+    }
+
+    g_list_free (fcd->tracks);
+    g_free (fcd);
+    gtk_widget_destroy (GTK_WIDGET (fc));
+}
+
+
+/*------------------------------------------------------------------
+
+  Code for DND: export when dragging from the iPod to the local
+  database.
+
+  ------------------------------------------------------------------*/
+
+/* If tracks are dragged from the iPod to the local database, the
+   tracks need to be copied from the iPod to the harddisk. This
+   function will ask where to copy them to, and add the tracks to the
+   MPL of @itdb_d.
+   A list of tracks that needs to be processed by the drag is
+   returned.
+
+   If tracks are not dragged from the iPod to the local database, a
+   copy of @tracks is returned.
+
+   The returned GList must be g_list_free()'ed after it is no longer
+   used. */
+GList *export_trackglist_when_necessary (iTunesDB *itdb_s,
+					 iTunesDB *itdb_d,
+					 GList *tracks)
+{
+    GList *gl;
+    GList *existing_tracks = NULL;
+    GList *new_tracks = NULL;
+    GList *added_tracks = NULL;
+
+    g_return_val_if_fail (itdb_s, NULL);
+    g_return_val_if_fail (itdb_d, NULL);
+    g_return_val_if_fail (gtkpod_window, NULL);
+
+    if (!(itdb_s->usertype & GP_ITDB_TYPE_IPOD) ||
+	!(itdb_d->usertype & GP_ITDB_TYPE_LOCAL))
+    {   /* drag is not from iPod to local database -> return copy of
+	 * @tracks */
+	return g_list_copy (tracks);
+    }
+
+    /* drag is from iPod to local database */
+
+    /* no drag possible if in offline mode */
+    if (prefs_get_offline ())
+    {
+	GtkWidget *dialog = gtk_message_dialog_new (
+	    GTK_WINDOW (gtkpod_window),
+	    GTK_DIALOG_DESTROY_WITH_PARENT,
+	    GTK_MESSAGE_WARNING,
+	    GTK_BUTTONS_OK,
+	    _("Drag from iPod database not possible in offline mode."));
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	return NULL;
+    }
+
+    /* make a list of tracks that already exist in itdb_d and of those
+       that do not yet exist */
+    for (gl=tracks; gl; gl=gl->next)
+    {
+	Track *otr;
+	Track *tr = gl->data;
+	g_return_val_if_fail (tr, NULL);
+
+	otr = md5_track_exists (itdb_d, tr);
+
+	if (otr)
+	{
+	    existing_tracks = g_list_append (existing_tracks, otr);
+	}
+	else
+	{
+	    new_tracks = g_list_append (new_tracks, tr);
+	}
+    }
+
+    /* if new tracks exist, copy them from the iPod to the harddisk */
+    if (new_tracks)
+    {
+	GList *filenames = NULL;
+	Playlist *mpl = itdb_playlist_mpl (itdb_d);
+	g_return_val_if_fail (mpl, NULL);
+
+	export_files_init (new_tracks, &filenames, TRUE,
+			   _("The following tracks have to be copied to your harddisk"));
+	/* add copied tracks to MPL of @itdb_d */
+	while (new_tracks && filenames)
+	{
+	    Track *dtr, *added_track;
+	    ExtraTrackData *edtr;
+	    Track *tr = new_tracks->data;
+	    gchar *filename = filenames->data;
+	    g_return_val_if_fail (tr, NULL);
+	    g_return_val_if_fail (filename, NULL);
+
+	    /* duplicate track */
+	    dtr = itdb_track_duplicate (tr);
+	    edtr = dtr->userdata;
+	    g_return_val_if_fail (edtr, NULL);
+
+	    /* set filename */
+	    g_free (edtr->pc_path_utf8);
+	    g_free (edtr->pc_path_locale);
+	    edtr->pc_path_utf8 = charset_to_utf8 (filename);
+	    edtr->pc_path_locale = filename;
+
+	    /* free ipod path */
+	    g_free (dtr->ipod_path);
+	    dtr->ipod_path = g_strdup ("");
+
+	    /* add track to itdb_d */
+	    added_track = gp_track_add (itdb_d, dtr);
+	    /* this cannot happen because we checked that the track is
+	       not in itdb_d before! */
+	    g_return_val_if_fail (added_track == dtr, NULL);
+	    /* add track to MPL */
+	    gp_playlist_add_track (mpl, dtr, TRUE);
+
+	    /* add track to added_tracks */
+	    added_tracks = g_list_append (added_tracks, dtr);
+
+	    /* remove the links from the GLists */
+	    new_tracks = g_list_delete_link (new_tracks, new_tracks);
+	    filenames = g_list_delete_link (filenames, filenames);
+	}
+
+	if (filenames)
+	{
+	    GList *gl;
+	    gtkpod_warning (_("Some tracks were not copied to your harddisk. Only the copied tracks will be included in the current drag and drop operation.\n\n"));
+	    for (gl=filenames; gl; gl=gl->next)
+	    {
+		g_free (gl->data);
+	    }
+	    g_list_free (filenames);
+	    filenames = NULL;
+	}
+	/* new_tracks must always be shorter than filenames! */
+	g_return_val_if_fail (!new_tracks, NULL);
+    }
+
+    /* return a list containing the existing tracks and the added
+       tracks */
+    return g_list_concat (existing_tracks, added_tracks);
 }
 
 
 
 
+/* same as export_trackglist_when_necessary() but the tracks are
+   represented as pointers in ASCII format. This function parses the
+   tracks in @data and calls export_trackglist_when_necessary() */
+GList *export_tracklist_when_necessary (iTunesDB *itdb_s,
+					iTunesDB *itdb_d,
+					gchar *data)
+{
+    GList *result;
+    Track *tr;
+    GList *tracks = NULL;
+    gchar *datap = data;
+
+    g_return_val_if_fail (itdb_s, NULL);
+    g_return_val_if_fail (itdb_d, NULL);
+    g_return_val_if_fail (data, NULL);
+
+    /* parse tracks and create GList */
+    while (parse_tracks_from_string (&datap, &tr))
+    {
+	tracks = g_list_append (tracks, tr);
+    }
+
+    result = export_trackglist_when_necessary (itdb_s, itdb_d, tracks);
+
+    g_list_free (tracks);
+ 
+    return result;
+}
+
 
 
 /*------------------------------------------------------------------
@@ -637,7 +904,6 @@ static void export_playlist_file_cleanup (struct fcd *fcd)
 {
     g_return_if_fail (fcd);
     g_list_free (fcd->tracks);
-    if (fcd->win)  gtk_widget_destroy (fcd->win);
     g_free (fcd);
     release_widgets ();
 }
@@ -649,15 +915,15 @@ static void export_playlist_file_cleanup (struct fcd *fcd)
  ******************************************************************/
 static void export_playlist_file_retrieve_options (struct fcd *fcd)
 {
-    g_return_if_fail (fcd && fcd->win && fcd->fc);
+    g_return_if_fail (fcd && fcd->fc);
 
-    option_get_radio_button (fcd->win,
+    option_get_radio_button (fcd->win_xml,
 			     EXPORT_PLAYLIST_FILE_TYPE,
 			     ExportPlaylistFileTypeW);
-    option_get_radio_button (fcd->win,
+    option_get_radio_button (fcd->win_xml,
 			     EXPORT_PLAYLIST_FILE_SOURCE,
 			     ExportPlaylistFileSourceW);
-    option_get_string (fcd->win, EXPORT_PLAYLIST_FILE_TPL, NULL);
+    option_get_string (fcd->win_xml, EXPORT_PLAYLIST_FILE_TPL, NULL);
     option_get_folder (GTK_FILE_CHOOSER (fcd->fc),
 		       EXPORT_PLAYLIST_FILE_PATH, NULL);
 }
@@ -710,8 +976,8 @@ static void export_playlist_file_write (struct fcd *fcd)
 	    gchar *infotext_utf8 = get_string_from_template (track,
 							     template,
 							     FALSE);
-	    gchar *filename = get_track_name_from_source (track,
-							  source);
+	    gchar *filename = get_file_name_from_source (track,
+							 source);
 	    gchar *infotext;
 
 	    if (infotext_utf8)
@@ -818,8 +1084,8 @@ static void export_playlist_file_response (GtkDialog *fc,
  ******************************************************************/
 void export_playlist_file_init (GList *tracks)
 {
-    GtkWidget *win = create_export_playlist_file_options ();
-    GtkWidget *options = lookup_widget (win, "options_frame");
+    GtkWidget *win; 
+    GtkWidget *options; 
     struct fcd *fcd = g_malloc0 (sizeof (struct fcd));
     GtkWidget *fc = gtk_file_chooser_dialog_new (
 	_("Create Playlist File"),
@@ -829,32 +1095,38 @@ void export_playlist_file_init (GList *tracks)
 	GTK_STOCK_APPLY, RESPONSE_APPLY,
 	GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 	NULL);
+    GladeXML *export_playlist_xml;
+    
+    export_playlist_xml = glade_xml_new (xml_file, "export_playlist_file_options", NULL);
+    win = glade_xml_get_widget (export_playlist_xml, "export_playlist_file_options");
+
+    options = glade_xml_get_widget (export_playlist_xml, "options_frame");
 
     /* Information needed to clean up later */
     fcd->tracks = g_list_copy (tracks);
-    fcd->win = win;
     fcd->fc = fc;
 
     /* according to GTK FAQ: move a widget to a new parent */
     gtk_widget_ref (options);
     gtk_container_remove (GTK_CONTAINER (win), options);
-    gtk_widget_unref (options);
-
     /* set extra options */
     gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc),
 				       options);
+    gtk_widget_unref (options);
+
+    gtk_widget_destroy (win);
 
     /* set last folder */
     option_set_folder (GTK_FILE_CHOOSER (fc),
 		       EXPORT_PLAYLIST_FILE_PATH);
     /* set last type */
-    option_set_radio_button (win, EXPORT_PLAYLIST_FILE_TYPE,
+    option_set_radio_button (export_playlist_xml, EXPORT_PLAYLIST_FILE_TYPE,
 			     ExportPlaylistFileTypeW, 0);
     /* set last source */
-    option_set_radio_button (win, EXPORT_PLAYLIST_FILE_SOURCE,
+    option_set_radio_button (export_playlist_xml, EXPORT_PLAYLIST_FILE_SOURCE,
 			     ExportPlaylistFileSourceW, 0);
     /* set last template */
-    option_set_string (win, EXPORT_PLAYLIST_FILE_TPL,
+    option_set_string (export_playlist_xml, EXPORT_PLAYLIST_FILE_TPL,
 		       EXPORT_PLAYLIST_FILE_TPL_DFLT);
 
     /* catch response codes */
