@@ -1,8 +1,9 @@
-/* Time-stamp: <2005-04-01 23:22:57 jcs>
+/* Time-stamp: <2005-06-05 23:43:58 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: info.c,v 1.16 2004/12/06 15:21:05 jcsjcs Exp $
+|  $Id: info.c,v 1.31 2005/06/05 14:48:37 jcsjcs Exp $
 */
 
 /* This file provides functions for the info window as well as for the
@@ -33,16 +34,17 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <gtk/gtk.h>
 #include "info.h"
-#include "interface.h"
 #include "misc.h"
+#include "misc_track.h"
 #include "prefs.h"
-#include "track.h"
-#include "support.h"
 
 /* pointer to info window */
 static GtkWidget *info_window = NULL;
 
+GladeXML *info_xml; 
+
 /* stuff for statusbar */
 static GtkWidget *gtkpod_statusbar = NULL;
 static GtkWidget *gtkpod_tracks_statusbar = NULL;
@@ -69,7 +71,9 @@ static void fill_in_info (GList *tl, guint32 *tracks,
 {
     GList *gl;
 
-    if (!tracks || !playtime || !filesize) return; /* sanity */
+    g_return_if_fail (tracks);
+    g_return_if_fail (playtime);
+    g_return_if_fail (filesize);
 
     *tracks = 0;
     *playtime = 0;
@@ -86,7 +90,11 @@ static void fill_in_info (GList *tl, guint32 *tracks,
 
 static void fill_label_uint (gchar *w_name, guint32 nr)
 {    
-    GtkWidget *w = lookup_widget (info_window, w_name);
+    GtkWidget *w;
+
+    g_return_if_fail (info_window);
+    g_return_if_fail (w_name);
+    w = glade_xml_get_widget (info_xml, w_name);
     if (w)
     {
 	gchar *str = g_strdup_printf ("%u", nr);
@@ -97,7 +105,11 @@ static void fill_label_uint (gchar *w_name, guint32 nr)
 
 static void fill_label_time (gchar *w_name, guint32 secs)
 {    
-    GtkWidget *w = lookup_widget (info_window, w_name);
+    GtkWidget *w;
+
+    g_return_if_fail (info_window);
+    g_return_if_fail (w_name);
+    w = glade_xml_get_widget (info_xml, w_name);
     if (w)
     {
 	gchar *str = g_strdup_printf ("%u:%02u:%02u",
@@ -111,7 +123,11 @@ static void fill_label_time (gchar *w_name, guint32 secs)
 
 static void fill_label_size (gchar *w_name, gdouble size)
 {    
-    GtkWidget *w = lookup_widget (info_window, w_name);
+    GtkWidget *w;
+
+    g_return_if_fail (info_window);
+    g_return_if_fail (w_name);
+    w = glade_xml_get_widget (info_xml, w_name);
     if (w)
     {
 	gchar *str = get_filesize_as_string (size);
@@ -122,7 +138,11 @@ static void fill_label_size (gchar *w_name, gdouble size)
 
 static void fill_label_string (gchar *w_name, const char *str)
 {    
-    GtkWidget *w = lookup_widget (info_window, w_name);
+    GtkWidget *w;
+
+    g_return_if_fail (info_window);
+    g_return_if_fail (w_name);
+    w = glade_xml_get_widget (info_xml, w_name);
     if (w)
     {
 	gtk_label_set_text (GTK_LABEL (w), str);
@@ -130,11 +150,20 @@ static void fill_label_string (gchar *w_name, const char *str)
 }
 
 
+
 /* open info window */
 void info_open_window (void)
 {
-    if (info_window)  return;            /* already open */
-    info_window = create_gtkpod_info ();
+    if (info_window)
+    {   /* info window already open -- raise to the top */
+	gdk_window_raise (info_window->window);
+	return;
+    }
+    
+    info_xml = glade_xml_new (xml_file, "gtkpod_info", NULL);
+    glade_xml_signal_autoconnect (info_xml);
+    info_window = glade_xml_get_widget (info_xml, "gtkpod_info");
+    
     if (info_window)
     {
 	gint defx, defy;
@@ -154,11 +183,12 @@ void info_close_window (void)
     GtkWidget *win;
 
     if (!info_window) return; /* not open */
+
     info_update_default_sizes ();
     win = info_window;
     info_window = NULL;
     gtk_widget_destroy (win);
-    prefs_set_info_window (FALSE); /* notify prefs */
+     prefs_set_info_window (FALSE); /* notify prefs */
     /* set the menu item for the info window correctly */
     display_set_info_window_menu ();
 }
@@ -183,18 +213,18 @@ void info_update (void)
     info_update_totals_view ();
 }
 
-static void info_update_track_view_total (void)
+static void info_update_track_view_displayed (void)
 {
     guint32 tracks, playtime; /* playtime in secs */
     gdouble  filesize;        /* in bytes */
     GList *displayed;
 
-    if (!info_window) return; /* not open */
+    g_return_if_fail (info_window);
     displayed = display_get_selected_members (prefs_get_sort_tab_num()-1);
     fill_in_info (displayed, &tracks, &playtime, &filesize);
-    fill_label_uint ("tracks_total", tracks);
-    fill_label_time ("playtime_total", playtime);
-    fill_label_size ("filesize_total", filesize);
+    fill_label_uint ("tracks_displayed", tracks);
+    fill_label_time ("playtime_displayed", playtime);
+    fill_label_size ("filesize_displayed", filesize);
 }
 
 void info_update_track_view_selected (void)
@@ -216,7 +246,7 @@ void info_update_track_view_selected (void)
 void info_update_track_view (void)
 {
     if (!info_window) return; /* not open */
-    info_update_track_view_total ();
+    info_update_track_view_displayed ();
     info_update_track_view_selected ();
 }
 
@@ -236,20 +266,84 @@ void info_update_playlist_view (void)
 }
 
 
+/* Get the local itdb */
+static iTunesDB *get_itdb_local (void)
+{
+    struct itdbs_head *itdbs_head;
+    GList *gl;
+
+    g_return_val_if_fail (gtkpod_window, NULL);
+    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
+				    "itdbs_head");
+    if (!itdbs_head) return NULL;
+    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
+    {
+	iTunesDB *itdb = gl->data;
+	g_return_val_if_fail (itdb, NULL);
+	if (itdb->usertype & GP_ITDB_TYPE_LOCAL)
+	    return itdb;
+    }
+    return NULL;
+}
+
+
+/* Get the iPod itdb */
+/* FIXME: This function must be expanded if support for several iPods
+   is implemented */
+static iTunesDB *get_itdb_ipod (void)
+{
+    struct itdbs_head *itdbs_head;
+    GList *gl;
+
+    g_return_val_if_fail (gtkpod_window, NULL);
+    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
+				    "itdbs_head");
+    if (!itdbs_head) return NULL;
+    for (gl=itdbs_head->itdbs; gl; gl=gl->next)
+    {
+	iTunesDB *itdb = gl->data;
+	g_return_val_if_fail (itdb, NULL);
+	if (itdb->usertype & GP_ITDB_TYPE_IPOD)
+	    return itdb;
+    }
+    return NULL;
+}
+
+
 /* update "totals" view section */
 void info_update_totals_view (void)
 {
     guint32 tracks=0, playtime=0; /* playtime in secs */
     gdouble  filesize=0;          /* in bytes */
     Playlist *pl;
+    iTunesDB *itdb;
 
     if (!info_window) return; /* not open */
-    pl = get_playlist_by_nr (0);
-    if (pl)  fill_in_info (pl->members, &tracks, &playtime, &filesize);
-    fill_label_uint ("total_playlists", get_nr_of_playlists ()-1);
-    fill_label_uint ("total_tracks", tracks);
-    fill_label_time ("total_playtime", playtime);
-    fill_label_size ("total_filesize", filesize);
+    
+    itdb = get_itdb_ipod ();
+    if (itdb)
+    {
+	pl = itdb_playlist_mpl (itdb);
+	g_return_if_fail (pl);
+	fill_in_info (pl->members, &tracks, &playtime, &filesize);
+	fill_label_uint ("total_playlists_ipod",
+			 itdb_playlists_number (itdb)-1);
+	fill_label_uint ("total_tracks_ipod", tracks);
+	fill_label_time ("total_playtime_ipod", playtime);
+	fill_label_size ("total_filesize_ipod", filesize);
+    }
+    itdb = get_itdb_local ();
+    if (itdb)
+    {
+	pl = itdb_playlist_mpl (itdb);
+	g_return_if_fail (pl);
+	fill_in_info (pl->members, &tracks, &playtime, &filesize);
+	fill_label_uint ("total_playlists_local",
+			 itdb_playlists_number (itdb)-1);
+	fill_label_uint ("total_tracks_local", tracks);
+	fill_label_time ("total_playtime_local", playtime);
+	fill_label_size ("total_filesize_local", filesize);
+    }
     info_update_totals_view_space ();
 }
 
@@ -258,32 +352,36 @@ void info_update_totals_view_space (void)
 {
     gdouble nt_filesize, del_filesize;
     guint32 nt_tracks, del_tracks;
-
+    iTunesDB *itdb;
 
     if (!info_window) return;
-    nt_filesize = get_filesize_of_nontransferred_tracks (&nt_tracks);
-    fill_label_uint ("non_transfered_tracks", nt_tracks);
-    fill_label_size ("non_transfered_filesize", nt_filesize);
-    del_filesize = get_filesize_of_deleted_tracks (&del_tracks);
-    fill_label_uint ("deleted_tracks", del_tracks);
-    fill_label_size ("deleted_filesize", del_filesize);
-    if (!prefs_get_offline ())
+    itdb = get_itdb_ipod ();
+    if (itdb)
     {
-	if (ipod_connected ())
+	gp_info_nontransferred_tracks (itdb, &nt_filesize, &nt_tracks);
+	fill_label_uint ("non_transferred_tracks", nt_tracks);
+	fill_label_size ("non_transferred_filesize", nt_filesize);
+	gp_info_deleted_tracks (itdb, &del_filesize, &del_tracks);
+	fill_label_uint ("deleted_tracks", del_tracks);
+	fill_label_size ("deleted_filesize", del_filesize);
+	if (!prefs_get_offline ())
 	{
-	    gdouble free_space = get_ipod_free_space()
-		+ del_filesize - nt_filesize;
-	    fill_label_size ("free_space", free_space);
+	    if (ipod_connected ())
+	    {
+		gdouble free_space = get_ipod_free_space()
+		    + del_filesize - nt_filesize;
+		fill_label_size ("free_space", free_space);
+	    }
+	    else
+	    {
+		fill_label_string ("free_space", _("n/c"));
+	    }
 	}
 	else
 	{
-	    fill_label_string ("free_space", _("n/c"));
+	    fill_label_string ("free_space", _("offline"));
 	}
     }
-    else
-    {
-	fill_label_string ("free_space", _("offline"));
-    }
 }
 
 
@@ -296,7 +394,7 @@ void info_update_totals_view_space (void)
 void
 gtkpod_statusbar_init(void)
 {
-    gtkpod_statusbar = lookup_widget (gtkpod_window, "gtkpod_status");
+    gtkpod_statusbar = glade_xml_get_widget (main_window_xml, "gtkpod_status");
 }
 
 static gint
@@ -342,7 +440,7 @@ void
 gtkpod_tracks_statusbar_init()
 {
     gtkpod_tracks_statusbar =
-	lookup_widget (gtkpod_window, "tracks_statusbar");
+	glade_xml_get_widget (main_window_xml, "tracks_statusbar");
     gtkpod_tracks_statusbar_update();
 }
 
@@ -352,11 +450,26 @@ gtkpod_tracks_statusbar_update(void)
     if(gtkpod_tracks_statusbar)
     {
 	gchar *buf;
+	iTunesDB *itdb;
+	Playlist *pl;
+	pl = pm_get_selected_playlist ();
+	/* select of which iTunesDB data should be displayed */
+	if (pl)
+	{
+	    itdb = pl->itdb;
+	}
+	else
+	{
+	    itdb = get_itdb_ipod ();
+	}
+	/* gets called before itdbs are setup up -> fail silently */
+/*	g_return_if_fail (itdb);*/
+	if (!itdb) return;
 	
-	buf = g_strdup_printf (_(" P:%d S:%d/%d"),
-			       get_nr_of_playlists () - 1,
+	buf = g_strdup_printf (_(" P:%d T:%d/%d"),
+			       itdb_playlists_number (itdb) - 1,
 			       tm_get_nr_of_tracks (),
-			       get_nr_of_tracks ());
+			       itdb_tracks_number (itdb));
 	gtk_statusbar_pop(GTK_STATUSBAR(gtkpod_tracks_statusbar), 1);
 	gtk_statusbar_push(GTK_STATUSBAR(gtkpod_tracks_statusbar), 1,  buf);
 	g_free (buf);
@@ -484,7 +597,7 @@ get_drive_stats_from_df(const gchar *mp)
 	if (df_str == NULL) df_str = "df -k -P";
 	if (strlen (df_str))
 	{
-	    snprintf(bufc, PATH_MAX, "%s %s", df_str, mp);
+	    snprintf(bufc, PATH_MAX, "%s \"%s\"", df_str, mp);
 	    fp = popen(bufc, "r");
 	    if(fp)
 	    {
@@ -644,11 +757,14 @@ gtkpod_space_statusbar_update(void)
 	{
 	    if (ipod_connected ())
 	    {
-		gdouble left, pending;
+		gdouble left, pending, deleted;
+		iTunesDB *itdb = get_itdb_ipod ();
+		/* we may be called before any itdb is present */
+		if (!itdb) return TRUE;
 
-		left = get_ipod_free_space() + 
-		    get_filesize_of_deleted_tracks (NULL);
-		pending = get_filesize_of_nontransferred_tracks(NULL);
+		gp_info_deleted_tracks (itdb, &deleted, NULL);
+		gp_info_nontransferred_tracks (itdb, &pending, NULL);
+		left = get_ipod_free_space() + deleted;
 		if((left-pending) > 0)
 		{
 		    str = get_filesize_as_string(left - pending);
@@ -681,7 +797,7 @@ gtkpod_space_statusbar_update(void)
 void
 gtkpod_space_statusbar_init(void)
 {
-    gtkpod_space_statusbar = lookup_widget (gtkpod_window, "space_statusbar");
+    gtkpod_space_statusbar = glade_xml_get_widget (main_window_xml, "space_statusbar");
 
     if (!space_mutex)
     {
