@@ -1,8 +1,9 @@
-/* Time-stamp: <2004-07-22 23:49:32 jcs>
+/* Time-stamp: <2005-06-17 22:25:30 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 | 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 | 
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 | 
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: info.h,v 1.8 2004/07/22 15:13:23 jcsjcs Exp $
+|  $Id: info.h,v 1.10 2005/06/17 13:30:03 jcsjcs Exp $
 */
 
 #ifndef __INFO_H__
@@ -33,8 +34,7 @@
 #  include <config.h>
 #endif
  
-#include <gtk/gtk.h>
-#include "track.h"
+#include <glib.h>
 
 /* info window */
 void info_open_window (void);
