@@ -1,9 +1,10 @@
 /* -*- coding: utf-8; -*-
-|  Time-stamp: <2005-03-23 22:00:19 jcs>
+|  Time-stamp: <2005-07-15 23:37:16 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-|
+| 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
@@ -24,7 +25,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.c,v 1.162 2004/12/13 15:57:06 jcsjcs Exp $
+|  $Id: misc.c,v 1.181 2005/07/15 14:39:37 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -37,10 +38,10 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <unistd.h>
-#include "interface.h"
+#include "charset.h"
 #include "misc.h"
 #include "prefs.h"
-#include "support.h"
+#include "misc_track.h"
 
 
 #define DEBUG_MISC 0
@@ -53,6 +54,9 @@
 
 static GtkWidget *about_window = NULL;
 
+/* ATTENTION: directly used as callback in gtkpod.glade -- if you
+   change the arguments of this function make sure you define a
+   separate callback for gtkpod.glade */
 void open_about_window ()
 {
   GtkLabel *about_label;
@@ -60,10 +64,17 @@ void open_about_window ()
   GtkTextView *textview;
   GtkTextIter ti;
   GtkTextBuffer *tb;
+  GladeXML *about_xml;
 
   if (about_window != NULL) return;
-  about_window = create_gtkpod_about_window ();
-  about_label = GTK_LABEL (lookup_widget (about_window, "about_label"));
+  /* about_window = create_gtkpod_about_window (); */
+
+  about_xml = glade_xml_new (xml_file, "gtkpod_about_window", NULL);
+  glade_xml_signal_autoconnect (about_xml);
+  about_window = glade_xml_get_widget (about_xml, "gtkpod_about_window");
+  
+  
+  about_label = GTK_LABEL (glade_xml_get_widget (about_xml, "about_label"));
   label_text = g_strdup_printf (_("gtkpod Version %s: Cross-Platform Multi-Lingual Interface to Apple's iPod(tm)."), VERSION);
   gtk_label_set_text (about_label, label_text);
   g_free (label_text);
@@ -128,6 +139,18 @@ Chris Micacchi: when sorting ignore 'the' and similar at the beginning of the ti
 Steve Jay: use statvfs() instead of df (better portability, faster)\n"),
 		       "\n",
 		       _("\
+Christoph Kunz: address compatibility issues when writing id3v2.4 type mp3 tags\n"),
+		       "\n",
+		       _("\
+James Ligget: replacement of old GTK file selection dialogs with new GTK filechooser dialogs\n"),
+		       "\n",
+		       _("\
+Daniel Kercher: sync scripts for abook and webcalendar\n"),
+		       "\n",
+		       _("\
+Clinton Gormley: sync scripts for thunderbird\n"),
+		       "\n",
+		       _("\
 Icons of buttons were made by Nicolas Chariot.\n\
 \n\
 \n"),
@@ -146,7 +169,7 @@ This program borrows code from the following projects:\n"),
 The GUI was created with the help of glade-2 (http://glade.gnome.org/).\n"),
 		       NULL };
       gchar **strp = text;
-      textview = GTK_TEXT_VIEW (lookup_widget (about_window, "credits_textview"));
+      textview = GTK_TEXT_VIEW (glade_xml_get_widget (about_xml, "credits_textview"));
       tb = gtk_text_view_get_buffer (textview);
       while (*strp)
       {
@@ -161,6 +184,8 @@ The GUI was created with the help of glade-2 (http://glade.gnome.org/).\n"),
 French:   David Le Brun (david at dyn-ns dot net)\n"),
 				     _("\
 German:   Jorg Schuler (jcsjcs at users dot sourceforge dot net)\n"),
+			             _("\
+Hebrew: Assaf Gillat (gillata at gmail dot com)\n"),
 				     _("\
 Italian:  Edward Matteucci (edward_matteucc at users dot sourceforge dot net)\n"),
 				     _("\
@@ -171,7 +196,7 @@ Japanese: Kentaro Fukuchi (fukuchi at users dot sourceforge dot net)\n"),
 Swedish: Stefan Asserhall (stefan asserhall at comhem dot se)\n"),
 				     NULL };
       gchar **strp = text;
-      textview = GTK_TEXT_VIEW (lookup_widget (about_window, "translators_textview"));
+      textview = GTK_TEXT_VIEW (glade_xml_get_widget (about_xml, "translators_textview"));
       tb = gtk_text_view_get_buffer (textview);
       while (*strp)
       {
@@ -184,6 +209,27 @@ Swedish: Stefan Asserhall (stefan asserhall at comhem dot se)\n"),
   gtk_widget_show (about_window);
 }
 
+
+/* callback for close button */
+void
+on_about_window_close_button           (GtkButton       *button,
+					gpointer         user_data)
+{
+  close_about_window (); /* in misc.c */
+}
+
+
+/* callback for window close button */
+gboolean
+on_about_window_close                  (GtkWidget       *widget,
+					GdkEvent        *event,
+					gpointer         user_data)
+{
+  close_about_window (); /* in misc.c */
+  return FALSE;
+}
+
+
 void close_about_window (void)
 {
   g_return_if_fail (about_window != NULL);
@@ -219,31 +265,37 @@ float get_ms_since (GTimeVal *old_time, gboolean update)
     return result;
 }
 
-/* parse a bunch of ipod ids delimited by \n
+/* parse a bunch of track pointers delimited by \n
  * @s - address of the character string we're parsing (gets updated)
- * @id - pointer the ipod id parsed from the string
+ * @track - pointer the track pointer parsed from the string
  * Returns FALSE when the string is empty, TRUE when the string can still be
  *	parsed
  */
 gboolean
-parse_ipod_id_from_string(gchar **s, guint32 *id)
+parse_tracks_from_string(gchar **s, Track **track)
 {
-    if(s && (*s))
+    g_return_val_if_fail (track, FALSE);
+    *track = NULL;
+    g_return_val_if_fail (s, FALSE);
+
+    if(*s)
     {
 	gchar *str = *s;
 	gchar *strp = strchr (str, '\n');
+	int tokens;
 
 	if (strp == NULL)
 	{
-	    *id = 0;
+	    *track = NULL;
 	    *s = NULL;
 	    return FALSE;
 	}
-	*id = (guint32)atoi(str);
+	tokens = sscanf (str, "%p", track);
 	++strp;
 	if (*strp) *s = strp;
 	else       *s = NULL;
-	return TRUE;
+	if (tokens == 1) 	return TRUE;
+	else                    return FALSE;
     }
     return FALSE;
 }
@@ -286,282 +338,6 @@ GList *glist_duplicate (GList *list)
 }
 
 
-
-
-/*------------------------------------------------------------------*\
- *                                                                  *
- *             DND to playlists                                     *
- *                                                                  *
-\*------------------------------------------------------------------*/
-
-/* DND: add a list of iPod IDs to Playlist @pl */
-void add_idlist_to_playlist (Playlist *pl, gchar *string)
-{
-    guint32 id = 0;
-    gchar *str = g_strdup (string);
-
-    if (!pl) return;
-    while(parse_ipod_id_from_string(&str,&id))
-    {
-	add_trackid_to_playlist(pl, id, TRUE);
-    }
-    data_changed();
-    g_free (str);
-}
-
-/* DND: add a list of files to Playlist @pl.  @pl: playlist to add to
-   or NULL. If NULL, a "New Playlist" will be created for adding
-   tracks and when adding a playlist file, a playlist with the name of the
-   playlist file will be added.
-   @trackaddfunc: passed on to add_track_by_filename() etc. */
-void add_text_plain_to_playlist (Playlist *pl, gchar *str, gint pl_pos,
-				 AddTrackFunc trackaddfunc, gpointer data)
-{
-    gchar **files = NULL, **filesp = NULL;
-    Playlist *pl_playlist = pl; /* playlist for playlist file */
-
-    if (!str)  return;
-
-    /*   printf("pl: %x, pl_pos: %d\n%s\n", pl, pl_pos, str);*/
-
-    block_widgets ();
-
-    files = g_strsplit (str, "\n", -1);
-    if (files)
-    {
-	filesp = files;
-	while (*filesp)
-	{
-	    gboolean added = FALSE;
-	    gint file_len = -1;
-
-	    gchar *file = NULL;
-	    gchar *decoded_file = NULL;
-
-	    file = *filesp;
-	    /* file is in uri form (the ones we're looking for are
-	       file:///), file can include the \n or \r\n, which isn't
-	       a valid character of the filename and will cause the
-	       uri decode / file test to fail, so we'll cut it off if
-	       its there. */
-	    file_len = strlen (file);
-	    if (file_len && (file[file_len-1] == '\n'))
-	    {
-		file[file_len-1] = 0;
-		--file_len;
-	    }
-	    if (file_len && (file[file_len-1] == '\r'))
-	    {
-		file[file_len-1] = 0;
-		--file_len;
-	    }
-
-	    decoded_file = filename_from_uri (file, NULL, NULL);
-	    if (decoded_file != NULL)
-	    {
-		if (g_file_test (decoded_file, G_FILE_TEST_IS_DIR))
-		{   /* directory */
-		    if (!pl)
-		    {  /* no playlist yet -- create new one */
-			pl = add_new_pl_user_name (NULL, pl_pos);
-			if (!pl)  break; /* while (*filesp) */
-		    }
-		    add_directory_by_name (decoded_file, pl,
-					   prefs_get_add_recursively (),
-					   trackaddfunc, data);
-		    added = TRUE;
-		}
-		if (g_file_test (decoded_file, G_FILE_TEST_IS_REGULAR))
-		{   /* regular file */
-		    gint ftype = determine_file_type (decoded_file);
-		    switch (ftype)
-		    {
-		    case FILE_TYPE_MP3:
-		    case FILE_TYPE_M4A:
-		    case FILE_TYPE_M4P:
-		    case FILE_TYPE_M4B:
-		    case FILE_TYPE_WAV:
-			if (!pl)
-			{  /* no playlist yet -- create new one */
-			    pl = add_new_pl_user_name (NULL,
-						       pl_pos);
-			    if (!pl)  break; /* while (*filesp) */
-			}
-			add_track_by_filename (decoded_file, pl,
-					       prefs_get_add_recursively (),
-					       trackaddfunc, data);
-			added = TRUE;
-			break;
-		    case FILE_TYPE_M3U:
-		    case FILE_TYPE_PLS:
-			add_playlist_by_filename (decoded_file,
-						  pl_playlist,
-						  trackaddfunc, data);
-			added = TRUE;
-			break;
-		    case FILE_TYPE_ERROR:
-		    case FILE_TYPE_UNKNOWN:
-			break;
-		    }
-		}
-		g_free (decoded_file);
-	    }
-	    if (!added)
-	    {
-		if (strlen (*filesp) != 0)
-		    gtkpod_warning (_("drag and drop: ignored '%s'\n"), *filesp);
-	    }
-	    ++filesp;
-	}
-	g_strfreev (files);
-    }
-    /* display log of non-updated tracks */
-    display_non_updated (NULL, NULL);
-    /* display log updated tracks */
-    display_updated (NULL, NULL);
-    /* display log of detected duplicates */
-    remove_duplicate (NULL, NULL);
-
-    release_widgets ();
-}
-
-
-/* ------------------------------------------------------------
- *
- * Code to "eject" the iPod.
- *
- * Large parts of this code are Copyright (C) 1994-2001 Jeff Tranter
- * (tranter at pobox.com).  Repackaged to only include scsi eject code
- * by Alex Tribble (prat at rice.edu). Integrated into gtkpod by Jorg
- * Schuler (jcsjcs at users.sourceforge.net)
- *
- * ------------------------------------------------------------ */
-
-/* This code does not work with FreeBSD -- let me know if you know a
- * way around it. */
-
-#if (HAVE_LINUX_CDROM_H && HAVE_SCSI_SCSI_H && HAVE_SCSI_SCSI_IOCTL_H)
-#include <linux/cdrom.h>
-#include <scsi/scsi.h>
-#include <scsi/scsi_ioctl.h>
-#include <sys/mount.h>
-#include <sys/types.h>
-#include <errno.h>
-#include <fcntl.h>
-
-static gchar *getdevicename(const gchar *mount)
-{
-    if(mount) {
-	gchar device[256], point[256];
-	FILE *fp = fopen("/proc/mounts","r");
-	if (!fp)
-	    return NULL;
-	do {
-	    fscanf(fp, "%255s %255s %*s %*s %*s %*s", device, point);
-	    if( strcmp(mount,point) == 0 ) {
-		fclose(fp);
-		return strdup(device);
-	    }
-	} while(!feof(fp));
-	fclose(fp);
-    }
-    return NULL;
-}
-
-/*
- * Eject using CDROMEJECT ioctl. Return TRUE if successful, FALSE otherwise.
- */
-static gboolean EjectCdrom(int fd)
-{
-	int status;
-
-	status = ioctl(fd, CDROMEJECT);
-	return (status == 0);
-}
-
-/*
- * Eject using SCSI commands. Return TRUE if successful, FALSE otherwise.
- */
-static gboolean EjectScsi(int fd)
-{
-    int status;
-    struct sdata {
-	int inlen;
-	int outlen;
-	char cmd[256];
-    } scsi_cmd;
-
-    scsi_cmd.inlen = 0;
-    scsi_cmd.outlen = 0;
-    scsi_cmd.cmd[0] = ALLOW_MEDIUM_REMOVAL;
-    scsi_cmd.cmd[1] = 0;
-    scsi_cmd.cmd[2] = 0;
-    scsi_cmd.cmd[3] = 0;
-    scsi_cmd.cmd[4] = 0;
-    scsi_cmd.cmd[5] = 0;
-    status = ioctl(fd, SCSI_IOCTL_SEND_COMMAND, (void *)&scsi_cmd);
-    if (status != 0)	return FALSE;
-
-    scsi_cmd.inlen = 0;
-    scsi_cmd.outlen = 0;
-    scsi_cmd.cmd[0] = START_STOP;
-    scsi_cmd.cmd[1] = 0;
-    scsi_cmd.cmd[2] = 0;
-    scsi_cmd.cmd[3] = 0;
-    scsi_cmd.cmd[4] = 1;
-    scsi_cmd.cmd[5] = 0;
-    status = ioctl(fd, SCSI_IOCTL_SEND_COMMAND, (void *)&scsi_cmd);
-    if (status != 0)	return FALSE;
-
-    scsi_cmd.inlen = 0;
-    scsi_cmd.outlen = 0;
-    scsi_cmd.cmd[0] = START_STOP;
-    scsi_cmd.cmd[1] = 0;
-    scsi_cmd.cmd[2] = 0;
-    scsi_cmd.cmd[3] = 0;
-    scsi_cmd.cmd[4] = 2;
-    scsi_cmd.cmd[5] = 0;
-    status = ioctl(fd, SCSI_IOCTL_SEND_COMMAND, (void *)&scsi_cmd);
-    if (status != 0)	return FALSE;
-
-    status = ioctl(fd, BLKRRPART);
-    return (status == 0);
-}
-
-static void eject_ipod(gchar *ipod_device)
-{
-     if(ipod_device)
-     {
-	  int fd = open (ipod_device, O_RDONLY|O_NONBLOCK);
-	  if (fd == -1)
-	  {
-/*	      fprintf(stderr, "Unable to open '%s' to eject.\n",ipod_device); */
-	      /* just ignore the error, i.e. if we don't have write access */
-	  }
-	  else
-	  {
-	      /* 2G seems to need EjectCdrom(), 3G EjectScsi()? */
-	      if (!EjectCdrom (fd))
-		  EjectScsi (fd);
-	      close (fd);
-	  }
-     }
-}
-#else
-static gchar *getdevicename(const gchar *mount)
-{
-    return NULL;
-}
-static void eject_ipod(gchar *ipod_device)
-{
-}
-#endif
-
-/* ------------------------------------------------------------
- *                    end of eject code
- * ------------------------------------------------------------ */
-
-
 /***************************************************************************
  * Mount Calls
  *
@@ -571,6 +347,32 @@ static void eject_ipod(gchar *ipod_device)
  * does not check prefs to see if the current prefs want gtkpod itself to
  * mount the ipod drive, that should be checked before making this call.
  */
+#include <sys/mount.h>
+#include <errno.h>
+#include <stdio.h>
+
+
+static gchar *getdevicename(const gchar *mount)
+{
+    if(mount) {
+        gchar device[256], point[256];
+        FILE *fp = fopen("/proc/mounts","r");
+        if (!fp)
+            return NULL;
+        do {
+            fscanf(fp, "%255s %255s %*s %*s %*s %*s", device, point);
+            if( strcmp(mount,point) == 0 ) {
+                fclose(fp);
+                return strdup(device);
+            }
+        } while(!feof(fp));
+        fclose(fp);
+    }
+    return NULL;
+}
+
+
+
 void
 mount_ipod(void)
 {
@@ -585,7 +387,7 @@ mount_ipod(void)
 	{
 	    case 0: /* child */
 		execl(MOUNT_BIN, "mount", str, NULL);
-		exit(0);
+		exit (1);
 		break;
 	    case -1: /* parent and error */
 		break;
@@ -598,40 +400,85 @@ mount_ipod(void)
 }
 
 /**
- * umount_ipod - attempt to unmount the ipod from prefs_get_ipod_mount()
+ * unmount_ipod - attempt to eject the ipod from prefs_get_ipod_mount()
  * This does not check prefs to see if the current prefs want gtkpod itself
- * to unmount the ipod drive, that should be checked before making this
+ * to eject the ipod drive, that should be checked before making this
  * call.
- * After unmounting, an attempt to send the "eject" signal is
- * made. Errors are ignored.
  */
 void
 unmount_ipod(void)
 {
-    const gchar *str = prefs_get_ipod_mount ();
-    gchar *ipod_device = getdevicename (prefs_get_ipod_mount());
-    if(str)
+    const gchar *mp = prefs_get_ipod_mount ();
+    if (mp)
     {
 	pid_t pid, tpid;
 	int status;
+	gchar *eject_bin;
+	gchar *ipod_device = getdevicename (mp);
 
+	/* umount */
 	pid = fork ();
 	switch (pid)
 	{
 	    case 0: /* child */
-		execl(UMOUNT_BIN, "umount", str, NULL);
-		exit(0);
+		execl (UMOUNT_BIN, "umount", mp, NULL);
+		exit (1); /* this is only reached in case of an error */
 		break;
 	    case -1: /* parent and error */
 		break;
 	    default: /* parent -- let's wait for the child to terminate */
 		tpid = waitpid (pid, &status, 0);
-		eject_ipod (ipod_device);
-		/* we could evaluate tpid and status now */
+		if (status != 0)
+		{
+		   	gchar *buf;
+		    gchar *str_utf8 = charset_to_utf8 (mp);
+		    GtkWidget *dialog;
+		    if (ipod_device)
+			buf = g_strdup_printf (
+			    _("Unmounting of '%s' (%s) unsuccessful."),
+			    str_utf8, ipod_device);
+		    else
+			buf = g_strdup_printf (
+			    _("Unmounting of '%s' unsuccessful."),
+			    str_utf8);
+		    g_free (str_utf8);
+		    dialog = gtk_message_dialog_new (
+			GTK_WINDOW (gtkpod_window),
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_MESSAGE_WARNING,
+			GTK_BUTTONS_OK,
+			buf);
+		    gtk_dialog_run (GTK_DIALOG (dialog));
+		    gtk_widget_destroy (dialog);
+		    g_free (buf);
+		}
 		break;
 	}
+	/* eject -- only if ipod_device and are EJECT_BIN available */
+	eject_bin = which ("eject");
+	if (ipod_device && eject_bin)
+	{
+	    pid = fork ();
+	    switch (pid)
+	    {
+	    case 0: /* child */
+		execl (eject_bin, "eject", ipod_device, NULL);
+		exit (1); /* this is only reached in case of an error */
+		break;
+	    case -1: /* parent and error */
+		break;
+	    default: /* parent -- let's wait for the child to terminate */
+		tpid = waitpid (pid, &status, 0);
+		if (status != 0)
+		{
+		    /* fail silently */
+		}
+		break;
+	    }
+	    g_free (eject_bin);
+	}
+	g_free (ipod_device);
     }
-    g_free (ipod_device);
 }
 
 
@@ -706,28 +553,70 @@ gint compare_string (gchar *str1, gchar *str2)
 	return compare_string_case_insensitive (str1, str2);
 }
 
-#define NUM_ARTICLES 	7
-static char* articles[NUM_ARTICLES] = {
-	"the ",
-	"a ",
-	"le ", 
-	"la ", 
-	"les ", 
-	"lo ", 
-	"los ", 
+struct csfk
+{
+    gint length;
+    gchar *key;
 };
 
+static GList *csfk_list = NULL;
+
+
+/* needs to be called everytime the sort_ign_strings in the prefs were
+   changed */
+void compare_string_fuzzy_generate_keys (void)
+{
+    GList *gl;
+    gint i;
+    /* remove old keys */
+    for (gl=csfk_list; gl; gl=gl->next)
+    {
+	struct csfk *csfk = gl->data;
+	g_return_if_fail (csfk);
+	g_free (csfk->key);
+	g_free (csfk);
+    }
+    g_list_free (csfk_list);
+    csfk_list = NULL;
+
+    /* create new keys */
+    for (i=0; ;++i)
+    {
+	gchar *buf = g_strdup_printf ("sort_ign_string_%d", i);
+	gchar *str = prefs_get_string (buf);
+	struct csfk *csfk;
+	gchar *tempStr;
+
+	g_free (buf);
+
+	/* end loop if no string is set or if the the string
+	 * corresponds to the end marker */
+	if (!str)  break;  
+	if (strcmp (str, SORT_IGNORE_STRINGS_END) == 0)
+	{
+	    g_free (str);
+	    break;
+	}
+
+	csfk = g_malloc (sizeof (struct csfk));
+	tempStr = g_utf8_casefold (str, -1 );
+	csfk->length = g_utf8_strlen (tempStr, -1 );
+	csfk->key = g_utf8_collate_key (tempStr, -1 );
+	g_free (tempStr);
+	g_free (str);
+
+	csfk_list = g_list_append (csfk_list, csfk);
+    }
+}
+
+
 /* compare @str1 and @str2 case-sensitively or case-insensitively
  * depending on prefs settings, and ignoring certain initial articles 
  * ("the", "le"/"la", etc) */
 gint compare_string_fuzzy (gchar *str1, gchar *str2)
 {
-    static gint   articleKeysGenerated = FALSE;
-    static gint   articleLengths[NUM_ARTICLES];
-    static gchar *articleKeys[NUM_ARTICLES];
     gchar *tempStr;
     gint   result;
-    gint   i;
     
     gchar *cleanStr1 = g_utf8_casefold (str1, -1);
     gchar *cleanStr2 = g_utf8_casefold (str2, -1);
@@ -737,45 +626,51 @@ gint compare_string_fuzzy (gchar *str1, gchar *str2)
     gchar *pcleanStr1 = cleanStr1;
     gchar *pcleanStr2 = cleanStr2;
     
+    GList *gl;
+
     /* If the article collations keys have not been generated, 
      * do that first
      */
-    if ( ! articleKeysGenerated ) {
-	for ( i = 0; i < NUM_ARTICLES; i += 1 ) {
-	    tempStr = g_utf8_casefold( articles[i], -1 );
-	    articleLengths[i] = g_utf8_strlen( tempStr, -1 );
-	    articleKeys[i] = g_utf8_collate_key( tempStr, -1 );
-	    g_free( tempStr );
-	}
-	articleKeysGenerated = TRUE;
-    } 
-    
+    if (!csfk_list)
+	compare_string_fuzzy_generate_keys ();
+
+    if (!csfk_list)
+	return compare_string (str1, str2);
+
     /* Check the beginnings of both strings for any of the 
      * articles we should ignore
      */
-    for ( i = 0; i < NUM_ARTICLES; i += 1 ) {
-    	tempStr = g_utf8_collate_key( cleanStr1, articleLengths[i] );
-	if ( strcmp( tempStr, articleKeys[i] ) == 0 ) {
+    for (gl=csfk_list; gl; gl=gl->next)
+    {
+	struct csfk *csfk = gl->data;
+	g_return_val_if_fail (csfk, 0);
+    	tempStr = g_utf8_collate_key (cleanStr1, csfk->length);
+	if (strcmp (tempStr, csfk->key) == 0)
+	{
 	    /* Found article, bump pointers ahead appropriate distance
 	     */
-	    pstr1 += articleLengths[i];
-	    pcleanStr1 = g_utf8_offset_to_pointer( cleanStr1, articleLengths[i] );
-	    g_free( tempStr );
+	    pstr1 += csfk->length;
+	    pcleanStr1 = g_utf8_offset_to_pointer (cleanStr1, csfk->length);
+	    g_free (tempStr);
 	    break;
 	}
-	g_free( tempStr );
+	g_free (tempStr);
     }
-    for ( i = 0; i < NUM_ARTICLES; i += 1 ) {
-    	tempStr = g_utf8_collate_key( cleanStr2, articleLengths[i] );
-	if ( strcmp( tempStr, articleKeys[i] ) == 0 ) {
+    for (gl=csfk_list; gl; gl=gl->next)
+    {
+	struct csfk *csfk = gl->data;
+	g_return_val_if_fail (csfk, 0);
+    	tempStr = g_utf8_collate_key (cleanStr2, csfk->length);
+	if (strcmp (tempStr, csfk->key) == 0)
+	{
 	    /* Found article, bump pointers ahead apropriate distance
 	     */
-	    pstr2 += articleLengths[i];
-	    pcleanStr2 = g_utf8_offset_to_pointer( cleanStr2, articleLengths[i] );
-	    g_free( tempStr );
+	    pstr2 += csfk->length;
+	    pcleanStr2 = g_utf8_offset_to_pointer (cleanStr2, csfk->length);
+	    g_free (tempStr);
 	    break;
 	}
-	g_free( tempStr );
+	g_free (tempStr);
     }
 
     if (prefs_get_case_sensitive ())
@@ -834,103 +729,6 @@ gunichar2 *utf16_strdup (gunichar2 *utf16)
 }
 
 
-/*------------------------------------------------------------------*\
- *                                                                  *
- *  Generic functions to "do" things on selected playlist / entry   *
- *  / tracks                                                         *
- *                                                                  *
-\*------------------------------------------------------------------*/
-
-/* Make a list of all selected tracks and call @do_func with that list
-   as argument */
-void do_selected_tracks (void (*do_func)(GList *trackids))
-{
-    GList *selected_trackids = NULL;
-
-    if (!do_func) return;
-
-    /* I'm using ids instead of "Track *" -pointer because it would be
-     * possible that a track gets removed during the process */
-    selected_trackids = tm_get_selected_trackids();
-    do_func (selected_trackids);
-    g_list_free (selected_trackids);
-}
-
-
-/* Make a list of all tracks in the currently selected entry of sort
-   tab @inst and call @do_func with that list as argument */
-void do_selected_entry (void (*do_func)(GList *trackids), gint inst)
-{
-    GList *selected_trackids = NULL;
-    TabEntry *entry;
-    GList *gl;
-
-    if (!do_func) return;
-
-    if ((inst < 0) || (inst > prefs_get_sort_tab_num ()))   return;
-
-    entry = st_get_selected_entry (inst);
-    if (entry == NULL)
-    {  /* no entry selected */
-	gtkpod_statusbar_message (_("No entry selected."));
-	return;
-    }
-    for (gl=entry->members; gl; gl=gl->next)
-    { /* make a list with all trackids in this entry */
-	Track *track = (Track *)gl->data;
-	if (track)
-	    selected_trackids = g_list_append (selected_trackids,
-					      (gpointer)track->ipod_id);
-    }
-    do_func (selected_trackids);
-    g_list_free (selected_trackids);
-}
-
-
-/* Make a list of the tracks in the current playlist and call @do_func
-   with that list as argument */
-void do_selected_playlist (void (*do_func)(GList *trackids))
-{
-    GList *selected_trackids = NULL;
-    Playlist *pl;
-    GList *gl;
-
-    if (!do_func) return;
-
-    pl = pm_get_selected_playlist();
-    if (!pl)
-    { /* no playlist selected */
-	gtkpod_statusbar_message (_("No playlist selected."));
-	return;
-    }
-    for (gl=pl->members; gl; gl=gl->next)
-    { /* make a list with all trackids in this entry */
-	Track *track = (Track *)gl->data;
-	if (track)
-	    selected_trackids = g_list_append (selected_trackids,
-					      (gpointer)track->ipod_id);
-    }
-    do_func (selected_trackids);
-    g_list_free (selected_trackids);
-}
-
-
-/* return some sensible input about the "track". Yo must free the
- * return string after use. */
-gchar *get_track_info (Track *track)
-{
-    if (!track) return NULL;
-    if (track->pc_path_utf8 && strlen(track->pc_path_utf8))
-	return g_path_get_basename (track->pc_path_utf8);
-    else if ((track->title && strlen(track->title)))
-	return g_strdup (track->title);
-    else if ((track->album && strlen(track->album)))
-	return g_strdup (track->album);
-    else if ((track->artist && strlen(track->artist)))
-	return g_strdup (track->artist);
-    return g_strdup_printf ("iPod ID: %d", track->ipod_id);
-}
-
 
 
 /*------------------------------------------------------------------*\
@@ -946,7 +744,7 @@ gchar *get_track_info (Track *track)
    (integer value). If no parameter is set in the prefs, use
    @dflt. The corresponding widget names are stored in an array
    @widgets and are member of @win */
-void option_set_radio_button (GtkWidget *win,
+void option_set_radio_button (GladeXML *win_xml,
 			      const gchar *prefs_string,
 			      const gchar **widgets,
 			      gint dflt)
@@ -954,7 +752,7 @@ void option_set_radio_button (GtkWidget *win,
     gint wnum, num=0;
     GtkWidget *w;
 
-    g_return_if_fail (win && prefs_string && widgets);
+    g_return_if_fail (win_xml && prefs_string && widgets);
 
     /* number of available widgets */
     num=0;
@@ -971,7 +769,7 @@ void option_set_radio_button (GtkWidget *win,
 	prefs_set_int_value (prefs_string, 0);
 	wnum = 0;
     }
-    w = lookup_widget (win, widgets[wnum]);
+    w = glade_xml_get_widget (win_xml, widgets[wnum]);
     if (w)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
 }
@@ -979,17 +777,17 @@ void option_set_radio_button (GtkWidget *win,
 
 /* Retrieve which toggle button was activated and store the state in
  * the prefs */
-gint option_get_radio_button (GtkWidget *win,
+gint option_get_radio_button (GladeXML *win_xml,
 			      const gchar *prefs_string,
 			      const gchar **widgets)
 {
     gint i;
 
-    g_return_val_if_fail (win && prefs_string && widgets, 0);
+    g_return_val_if_fail (win_xml && prefs_string && widgets, 0);
 
     for (i=0; widgets[i]; ++i)
     {
-	GtkWidget *w = lookup_widget (win, widgets[i]);
+	GtkWidget *w = glade_xml_get_widget (win_xml, widgets[i]);
 	if (w)
 	{
 	    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)))
@@ -1077,21 +875,21 @@ void option_get_filename (GtkFileChooser *fc,
 
 /* Set the string entry @name to the prefs value stored in @name or
    to @default if @name is not yet defined. */
-void option_set_string (GtkWidget *win,
+void option_set_string (GladeXML *win_xml,
 			const gchar *name,
 			const gchar *dflt)
 {
     gchar *string;
     GtkWidget *entry;
 
-    g_return_if_fail (win && name && dflt);
+    g_return_if_fail (win_xml && name && dflt);
 
     prefs_get_string_value (name, &string);
 
     if (!string)
 	string = g_strdup (dflt);
 
-    entry = lookup_widget (win, name);
+    entry = glade_xml_get_widget (win_xml, name);
 
     if (entry)
 	gtk_entry_set_text(GTK_ENTRY(entry), string);
@@ -1103,15 +901,16 @@ void option_set_string (GtkWidget *win,
  * to the prefs (@name) */
 /* If @value is != NULL, a copy of the string is placed into
    @value. It has to be g_free()d after use */
-void option_get_string (GtkWidget *win,
+void option_get_string (GladeXML *win_xml,
 			const gchar *name,
 			gchar **value)
 {
     GtkWidget *entry;
 
-    g_return_if_fail (win && name);
+    g_return_if_fail (win_xml && name);
+
+    entry = glade_xml_get_widget (win_xml, name);
 
-    entry = lookup_widget (win, name);
     if (entry)
     {
 	const gchar *str = gtk_entry_get_text (GTK_ENTRY (entry));
@@ -1123,19 +922,19 @@ void option_get_string (GtkWidget *win,
 
 /* Set the state of toggle button @name to the prefs value stored in
    @name or to @default if @name is not yet defined. */
-void option_set_toggle_button (GtkWidget *win,
+void option_set_toggle_button (GladeXML *win_xml,
 			       const gchar *name,
 			       gboolean dflt)
 {
     gboolean active;
     GtkWidget *button;
 
-    g_return_if_fail (win && name);
+    g_return_if_fail (win_xml && name);
 
     if (!prefs_get_int_value (name, &active))
 	active = dflt;
 
-    button = lookup_widget (win, name);
+    button = glade_xml_get_widget (win_xml, name);
 
     if (button)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button),
@@ -1145,15 +944,15 @@ void option_set_toggle_button (GtkWidget *win,
 /* Retrieve the current state of the toggle button @name and write it
  * to the prefs (@name) */
 /* Return value: the current state */
-gboolean option_get_toggle_button (GtkWidget *win,
+gboolean option_get_toggle_button (GladeXML *win_xml,
 				   const gchar *name)
 {
     gboolean active = FALSE;
     GtkWidget *button;
 
-    g_return_val_if_fail (win && name, active);
+    g_return_val_if_fail (win_xml && name, active);
 
-    button = lookup_widget (win, name);
+    button = glade_xml_get_widget (win_xml, name);
 
     if (button)
     {
@@ -1226,10 +1025,12 @@ static gchar *select_template (Track *track, const gchar *p)
     gchar **templates, **tplp;
     gchar *tname, *ext = NULL;
     gchar *result;
+    ExtraTrackData *etr;
 
-    if (!track) return (strdup (""));
-
-    tname = get_track_name_on_disk (track);
+    g_return_val_if_fail (track, strdup (""));
+    etr = track->userdata;
+    g_return_val_if_fail (etr, strdup (""));
+    tname = get_file_name (track);
     if (!tname) return (NULL);         /* this should not happen... */
     ext = strrchr (tname, '.');        /* pointer to filename extension */
 
@@ -1240,7 +1041,7 @@ static gchar *select_template (Track *track, const gchar *p)
 	if (strcmp (*tplp, "%o") == 0)
 	{   /* this is only a valid extension if the original filename
 	       is present */
-	    if (track->pc_path_locale && strlen(track->pc_path_locale))  break;
+	    if (etr->pc_path_locale && strlen(etr->pc_path_locale))  break;
 	}
 	else if (strrchr (*tplp, '.') == NULL)
 	{   /* this templlate does not have an extension and therefore
@@ -1282,14 +1083,17 @@ gchar *get_string_from_template (Track *track,
     gchar *p, *res_utf8;
     gchar *basename = NULL;
     gchar *template;
+    ExtraTrackData *etr;
 
     g_return_val_if_fail (track && full_template, NULL);
+    etr = track->userdata;
+    g_return_val_if_fail (etr, NULL);
 
     template = select_template (track, full_template);
 
     if (!template)
     {
-	gchar *fn = get_track_name_on_disk (track);
+	gchar *fn = get_file_name (track);
 	gtkpod_warning (_("Template ('%s') does not match file type '%s'\n"), full_template, fn ? fn:"");
 	g_free (fn);
 	return NULL;
@@ -1311,8 +1115,8 @@ gchar *get_string_from_template (Track *track,
     result = g_string_new ("");
 
     /* try to get the original filename */
-    if (track->pc_path_utf8)
-	basename = g_path_get_basename (track->pc_path_utf8);
+    if (etr->pc_path_utf8)
+	basename = g_path_get_basename (etr->pc_path_utf8);
 
     p=template;
     while (*p != '\0') {
@@ -1334,20 +1138,20 @@ gchar *get_string_from_template (Track *track,
 		    tmp = pl->name;
 		break;
 	    case 'a':
-		tmp = track_get_item_utf8 (track, T_ARTIST);
+		tmp = track_get_item (track, T_ARTIST);
 		break;
 	    case 'A':
-		tmp = track_get_item_utf8 (track, T_ALBUM);
+		tmp = track_get_item (track, T_ALBUM);
 		break;
 	    case 't':
-		tmp = track_get_item_utf8 (track, T_TITLE);
+		tmp = track_get_item (track, T_TITLE);
 		break;
 	    case 'c':
-		tmp = track_get_item_utf8 (track, T_COMPOSER);
+		tmp = track_get_item (track, T_COMPOSER);
 		break;
 	    case 'g':
 	    case 'G':
-		tmp = track_get_item_utf8 (track, T_GENRE);
+		tmp = track_get_item (track, T_GENRE);
 		break;
 	    case 'C':
 		if (track->cds == 0)
@@ -1436,3 +1240,30 @@ gchar *get_string_from_template (Track *track,
 }
 
 
+/**
+ * which - run the shell command which, useful for querying default values
+ * for executable,
+ * @name - the executable we're trying to find the path for
+ * Returns the path to the executable, NULL on not found
+ */
+gchar *which (const gchar *exe)
+{
+    FILE *fp = NULL;
+    gchar *result = NULL;
+    gchar buf[PATH_MAX];
+    gchar *which_exec = NULL;
+
+    g_return_val_if_fail (exe, NULL);
+
+    memset(&buf[0], 0, PATH_MAX);
+    which_exec = g_strdup_printf("which %s", exe);
+    if((fp = popen(which_exec, "r")))
+    {
+        int read_bytes = 0;
+        if((read_bytes = fread(buf, sizeof(gchar), PATH_MAX, fp)) > 0)
+            result = g_strndup(buf, read_bytes-1);
+        pclose(fp);
+    }
+    g_free(which_exec);
+    return(result);
+}
