@@ -1,8 +1,9 @@
-/* Time-stamp: <2004-12-29 18:25:55 jcs>
+/* Time-stamp: <2005-07-09 14:32:12 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-|
+| 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc.h,v 1.84 2004/12/29 10:00:31 jcsjcs Exp $
+|  $Id: misc.h,v 1.100 2005/07/09 06:53:26 jcsjcs Exp $
 */
 
 #ifndef __MISC_H__
@@ -33,11 +34,13 @@
 #  include <config.h>
 #endif
 
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
+#include <glade/glade.h>
 #include <stdio.h>
 #include "file.h"
-#include "track.h"
-#include "playlist.h"
+#include "display_itdb.h"
+#include "itdb.h"
 #include "display.h"
 #include "confirmation.h"
 #include "time.h"
@@ -63,11 +66,13 @@ enum { RESPONSE_APPLY = 5 };
 
 /* indicates whether widgets are currently blocked */
 extern gboolean widgets_blocked;
+/* full path to 'gtkpod.glade', defined in main.c */
+extern gchar *xml_file;
 
 gchar *get_user_string (gchar *title, gchar *message, gchar *dflt,
 			gchar *opt_msg, gboolean *opt_state);
-Playlist *add_new_pl_user_name (gchar *dflt, gint position);
-void add_new_pl_or_spl_user_name (gchar *dflt, gint position);
+Playlist *add_new_pl_user_name (iTunesDB *itdb, gchar *dflt, gint32 pos);
+void add_new_pl_or_spl_user_name (iTunesDB *itdb, gchar *dflt, gint32 pos);
 void create_add_files_fileselector (void);
 void create_add_playlists_fileselector (void);
 gchar *concat_dir (G_CONST_RETURN gchar *dir, G_CONST_RETURN gchar *file);
@@ -77,24 +82,26 @@ float get_ms_since (GTimeVal *old_time, gboolean update);
 gint get_sort_tab_number (gchar *text);
 void open_about_window (void);
 void close_about_window (void);
-gboolean parse_ipod_id_from_string(gchar **s, guint32 *id);
-void add_idlist_to_playlist (Playlist *pl, gchar *str);
-void add_text_plain_to_playlist (Playlist *pl, gchar *str, gint position,
-				 AddTrackFunc trackaddfunc, gpointer data);
+gboolean parse_tracks_from_string(gchar **s, Track **track);
 void cleanup_backup_and_extended_files (void);
 gboolean gtkpod_main_quit(void);
 void gtkpod_main_window_set_active(gboolean active);
 
 T_item TM_to_T (TM_item sm);
 T_item ST_to_T (ST_CAT_item st);
-gchar *get_track_info (Track *track);
+const gchar *get_tm_string (TM_item tm);
+const gchar *get_tm_tooltip (TM_item tm);
+const gchar *get_t_string (T_item t);
+const gchar *get_t_tooltip (T_item t);
 
-void ipod_directories_head (void);
-void delete_playlist_head (gboolean delete_full);
-void delete_track_head (gboolean delete_full);
-void delete_entry_head (gint inst, gboolean delete_full);
+gchar *get_track_info (Track *track, gboolean prefer_filename);
 
-void delete_populate_settings (Playlist *pl, GList *selected_trackids,
+void ipod_directories_head (const gchar *mountpoint);
+void delete_playlist_head (DeleteAction deleteaction);
+void delete_track_head (DeleteAction deleteaction);
+void delete_entry_head (gint entry_inst, DeleteAction deleteaction);
+
+void delete_populate_settings (struct DeleteData *dd,
 			       gchar **label, gchar **title,
 			       gboolean *confirm_again,
 			       ConfHandlerOpt *confirm_again_handler,
@@ -108,7 +115,7 @@ void mount_ipod(void);
 void unmount_ipod(void);
 void call_script (gchar *script);
 
-void delete_track_ok (gpointer user_data1, gpointer user_data2);
+void delete_track_ok (struct DeleteData *dd);
 
 void gtkpod_warning (const gchar *format, ...);
 
@@ -125,6 +132,7 @@ void time_set_time (Track *track, time_t t, TM_item tm_item);
 gchar *get_filesize_as_string (double size);
 
 gint compare_string (gchar *str1, gchar *str2);
+void compare_string_fuzzy_generate_keys (void);
 gint compare_string_fuzzy (gchar *str1, gchar *str2);
 gint compare_string_case_insensitive (gchar *str1, gchar *str2);
 
@@ -132,40 +140,36 @@ gchar *filename_from_uri (const char *uri,
 			  char      **hostname,
 			  GError    **error);
 
-void generate_category_playlists (T_item cat);
+void generate_category_playlists (iTunesDB *itdb, T_item cat);
 Playlist *generate_displayed_playlist (void);
 Playlist *generate_selected_playlist (void);
 void randomize_current_playlist (void);
-Playlist *generate_random_playlist (void);
-Playlist *generate_not_listed_playlist (void);
-Playlist *generate_playlist_with_name (GList *tracks, gchar *pl_name,
-				       gboolean del_old);
-Playlist *generate_new_playlist (GList *tracks);
-void most_listened_pl (void);
-void never_listened_pl (void);
-void last_listened_pl(void);
-void most_rated_pl(void);
-void since_last_pl(void);
-void each_rating_pl (void);
+Playlist *generate_random_playlist (iTunesDB *itdb);
+Playlist *generate_not_listed_playlist (iTunesDB *itdb);
+Playlist *generate_playlist_with_name (iTunesDB *itdb, GList *tracks,
+				       gchar *pl_name, gboolean del_old);
+Playlist *generate_new_playlist (iTunesDB *itdb, GList *tracks);
+void most_listened_pl (iTunesDB *itdb);
+void never_listened_pl (iTunesDB *itdb);
+void last_listened_pl(iTunesDB *itdb);
+void most_rated_pl(iTunesDB *itdb);
+void since_last_pl(iTunesDB *itdb);
+void each_rating_pl (iTunesDB *itdb);
 
 guint32 utf16_strlen (gunichar2 *utf16);
 gunichar2 *utf16_strdup (gunichar2 *utf16);
 
-void check_db (void);
-
-void do_selected_tracks (void (*do_func)(GList *trackids));
-void do_selected_entry (void (*do_func)(GList *trackids), gint inst);
-void do_selected_playlist (void (*do_func)(GList *trackids));
+void check_db (iTunesDB *db);
 
 guint32 replaygain_to_soundcheck (gdouble gain);
 gdouble soundcheck_to_replaygain (guint32 soundcheck);
 
 
-void option_set_radio_button (GtkWidget *win,
+void option_set_radio_button (GladeXML *win_xml,
 			      const gchar *prefs_string,
 			      const gchar **widgets,
 			      gint dflt);
-gint option_get_radio_button (GtkWidget *win,
+gint option_get_radio_button (GladeXML *win_xml,
 			      const gchar *prefs_string,
 			      const gchar **widgets);
 void option_set_folder (GtkFileChooser *fc,
@@ -178,16 +182,16 @@ void option_set_filename (GtkFileChooser *fc,
 void option_get_filename (GtkFileChooser *fc,
 			  const gchar *prefs_string,
 			  gchar **value);
-void option_set_string (GtkWidget *win,
+void option_set_string (GladeXML *win_xml,
 			const gchar *name,
 			const gchar *dflt);
-void option_get_string (GtkWidget *win,
+void option_get_string (GladeXML *win_xml,
 			const gchar *name,
 			gchar **value);
-void option_set_toggle_button (GtkWidget *win,
+void option_set_toggle_button (GladeXML *win_xml,
 			       const gchar *name,
 			       gboolean dflt);
-gboolean option_get_toggle_button (GtkWidget *win,
+gboolean option_get_toggle_button (GladeXML *win_xml,
 				   const gchar *name);
 
 gchar *get_string_from_template (Track *track,
@@ -196,4 +200,5 @@ gchar *get_string_from_template (Track *track,
 
 GList *glist_duplicate (GList *list);
 
+gchar *which (const gchar *exe);
 #endif
