@@ -1,8 +1,9 @@
-/* Time-stamp: <2004-12-30 14:39:54 jcs>
+/* Time-stamp: <2005-07-02 00:37:36 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-|
+| 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: misc_conversion.c,v 1.9 2004/12/30 04:34:04 jcsjcs Exp $
+|  $Id: misc_conversion.c,v 1.19 2005/07/01 16:53:44 jcsjcs Exp $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -38,22 +39,97 @@
 #include <string.h>
 
 #include "charset.h"
-#include "itunesdb.h"
+#include "itdb.h"
 #include "misc.h"
+#include "display.h"
 #include "prefs.h"
-#include "support.h"
 #include <time.h>
 
 
 #define DEBUG_MISC 0
 
+/* Note: the toggle buttons for tag_autoset and display_col in the
+ * prefs_window are named after the the TM_COLUM_* numbers defined in
+ * display.h (Title: tag_autoset0, Artist: tag_autoset1
+ * etc.). ign_field is named after T_*. Since the labels to the
+ * buttons are set in prefs_window.c when creating the window, you
+ * only need to name the buttons in the intended order using
+ * glade-2. There is no need to label them. */
+/* Strings associated to the column headers */
+static const gchar *t_strings[] = {
+    N_("All"),               /*  0 */
+    N_("Album"),
+    N_("Artist"),
+    N_("Title"),
+    N_("Genre"),
+    N_("Comment"),           /*  5 */
+    N_("Composer"),
+    N_("Kind"),
+    N_("PC File"),
+    N_("iPod File"),
+    N_("iPod ID"),           /* 10 */
+    N_("Track Nr (#)"),
+    N_("Transferred"),
+    N_("File Size"),
+    N_("Play Time"),
+    N_("Bitrate"),           /* 15 */
+    N_("Samplerate"),
+    N_("BPM"),
+    N_("Playcount"),
+    N_("Rating"),
+    N_("Date added"),        /* 20 */
+    N_("Date played"),
+    N_("Date modified"),
+    N_("Volume"),
+    N_("Soundcheck"),
+    N_("Year"),              /* 25 */
+    N_("CD Nr"),
+    N_("Grouping"),
+    N_("Compilation"),
+    NULL };
+
+/* Tooltips for prefs window */
+static const gchar *t_tooltips[] = {
+    NULL,                                              /*  0 */
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,                                              /*  5 */
+    NULL,
+    NULL,
+    N_("Name of file on PC, if available"),
+    N_("Name of file on the iPod"),
+    NULL,                                              /* 10 */
+    N_("Track Nr. and total number of tracks on CD"),
+    N_("Whether the file has already been "
+       "transferred to the iPod or not"),
+    NULL,
+    NULL,
+    NULL,                                              /* 15 */
+    NULL,
+    N_("Supposedly something that tells the iPod to "
+       "increase or decrease the playback speed"),
+    N_("Number of times the track has been played"),
+    N_("Star rating from 0 to 5"),
+    N_("Date and time track has been added"),          /* 20 */
+    N_("Date and time track has last been played"),
+    N_("Date and time track has last been modified"),
+    N_("Manual volume adjust"),
+    N_("Volume adjust in dB (replay gain) -- "
+       "you need to activate 'soundcheck' on the iPod"),
+    NULL,                                              /* 25 */
+    N_("CD Nr. and total number of CDS in set"),
+    NULL,
+    NULL };
+
 
 /* translates a TM_COLUMN_... (defined in display.h) into a
- * T_... (defined in track.h). Returns -1 in case a translation is not
+ * T_... (defined in display.h). Returns -1 in case a translation is not
  * possible */
-T_item TM_to_T (TM_item sm)
+T_item TM_to_T (TM_item tm)
 {
-    switch (sm)
+    switch (tm)
     {
     case TM_COLUMN_TITLE:         return T_TITLE;
     case TM_COLUMN_ARTIST:        return T_ARTIST;
@@ -75,7 +151,7 @@ T_item TM_to_T (TM_item sm)
     case TM_COLUMN_BPM:           return T_BPM;
     case TM_COLUMN_PLAYCOUNT:     return T_PLAYCOUNT;
     case TM_COLUMN_RATING:        return T_RATING;
-    case TM_COLUMN_TIME_CREATED:  return T_TIME_CREATED;
+    case TM_COLUMN_TIME_ADDED:    return T_TIME_ADDED;
     case TM_COLUMN_TIME_PLAYED:   return T_TIME_PLAYED;
     case TM_COLUMN_TIME_MODIFIED: return T_TIME_MODIFIED;
     case TM_COLUMN_VOLUME:        return T_VOLUME;
@@ -89,7 +165,7 @@ T_item TM_to_T (TM_item sm)
 
 
 /* translates a ST_CAT_... (defined in display.h) into a
- * T_... (defined in track.h). Returns -1 in case a translation is not
+ * T_... (defined in display.h). Returns -1 in case a translation is not
  * possible */
 T_item ST_to_T (ST_CAT_item st)
 {
@@ -108,6 +184,51 @@ T_item ST_to_T (ST_CAT_item st)
 }
 
 
+/* return descriptive string (non-localized -- pass through gettext()
+ * for the localized version) for tm_item (usually used to name
+ * buttons or column headers). */
+const gchar *get_tm_string (TM_item tm)
+{
+    T_item t = TM_to_T (tm);
+
+    if (t != -1)   return t_strings[t];
+    else           return ("");
+}
+
+
+/* return string (non-localized -- pass through gettext()
+ * for the localized version) for tm_item that can be used as a
+ * tooltip */
+const gchar *get_tm_tooltip (TM_item tm)
+{
+    T_item t = TM_to_T (tm);
+
+    if (t != -1)   return t_tooltips[t];
+    else           return ("");
+}
+
+
+/* return descriptive string (non-localized -- pass through gettext()
+ * for the localized version) for tm_item (usually used to name
+ * buttons or column headers). */
+const gchar *get_t_string (T_item t)
+{
+    if (t != -1)   return t_strings[t];
+    else           return ("");
+}
+
+
+/* return string (non-localized -- pass through gettext()
+ * for the localized version) for tm_item that can be used as a
+ * tooltip */
+const gchar *get_t_tooltip (T_item t)
+{
+    if ((t >= 0) && (t<T_ITEM_NUM))   return t_tooltips[t];
+    else                              return ("");
+}
+
+
+
 /*------------------------------------------------------------------*\
  *                                                                  *
  *                       Timestamp stuff                            *
@@ -138,13 +259,14 @@ static gchar *time_to_string_format (time_t t, const gchar *format)
 {
     gchar buf[PATH_MAX+1];
     struct tm tm;
+    size_t size;
 
     g_return_val_if_fail (format, NULL);
 
     if (t)
     {
 	localtime_r (&t, &tm);
-	size_t size = strftime (buf, PATH_MAX, format, &tm);
+	size = strftime (buf, PATH_MAX, format, &tm);
 	buf[size] = 0;
 	return g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
     }
@@ -241,8 +363,8 @@ time_t time_get_time (Track *track, TM_item tm_item)
 
     if (track) switch (tm_item)
     {
-    case TM_COLUMN_TIME_CREATED:
-	mactime = track->time_created;
+    case TM_COLUMN_TIME_ADDED:
+	mactime = track->time_added;
 	break;
     case TM_COLUMN_TIME_PLAYED:
 	mactime = track->time_played;
@@ -254,7 +376,7 @@ time_t time_get_time (Track *track, TM_item tm_item)
 	mactime = 0;
 	break;
     }
-    return (itunesdb_time_mac_to_host (mactime));
+    return (itdb_time_mac_to_host (mactime));
 }
 
 
@@ -268,12 +390,12 @@ gchar *time_field_to_string (Track *track, TM_item tm_item)
 /* get the timestamp TM_COLUMN_TIME_CREATE/PLAYED/MODIFIED */
 void time_set_time (Track *track, time_t time, TM_item tm_item)
 {
-    guint32 mactime = itunesdb_time_host_to_mac (time);
+    guint32 mactime = itdb_time_host_to_mac (time);
 
     if (track) switch (tm_item)
     {
-    case TM_COLUMN_TIME_CREATED:
-	track->time_created = mactime;
+    case TM_COLUMN_TIME_ADDED:
+	track->time_added = mactime;
 	break;
     case TM_COLUMN_TIME_PLAYED:
 	track->time_played = mactime;
@@ -384,7 +506,7 @@ g_unescape_uri_string (const char *escaped,
       *out++ = c;
     }
 
-  g_assert (out - result <= len);
+  g_return_val_if_fail (out - result <= len, NULL);
   *out = '\0';
 
   if (in != in_end || !g_utf8_validate (result, -1, NULL))
