@@ -1,9 +1,10 @@
-/* Time-stamp: <2004-11-20 01:38:07 jcs>
+/* Time-stamp: <2005-06-17 22:12:13 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-|
-|  URL:  http://gtkpod.sourceforge.net/
+| 
+|  URL: http://www.gtkpod.org/
+|  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
 |  it under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: mp3file.c,v 1.45 2004/12/29 00:28:55 ln2 Exp $
+|  $Id: mp3file.c,v 1.54 2005/06/17 13:30:04 jcsjcs Exp $
 */
 
 /* The code in the first section of this file is taken from the
@@ -59,6 +60,9 @@ struct _File_Tag
     gchar *comment;        /* Comment */
     gchar *composer;	   /* Composer */
     guint32 songlen;       /* Length of file in ms */
+    /* CD/disc number handling */
+    gchar *cdnostring;    /* Position of disc in the album */
+    gchar *cdno_total;    /* The number of discs in the album (ex: 1/2) */
 };
 
 /* This code is taken from the mp3info code. Only the code needed for
@@ -105,10 +109,9 @@ struct _File_Tag
 
 #include "mp3file.h"
 #include "charset.h"
-#include "itunesdb.h"
+#include "itdb.h"
 #include "file.h"
 #include "misc.h"
-#include "support.h"
 
 
 /* MIN_CONSEC_GOOD_FRAMES defines how many consecutive valid MP3 frames
@@ -133,21 +136,21 @@ typedef struct {
     guint  copyright;
     guint  original;
     guint  emphasis;
-} mp3header;
+} MP3Header;
 
 typedef struct {
     gchar *filename;
     FILE *file;
     off_t datasize;
     gint header_isvalid;
-    mp3header header;
+    MP3Header header;
     gint id3_isvalid;
     gint vbr;
     float vbr_average;
     gint milliseconds;
     gint frames;
     gint badframes;
-} mp3info;
+} MP3Info;
 
 /* This is for xmms code */
 static guint get_track_time(gchar *path);
@@ -159,7 +162,7 @@ static guint get_track_time(gchar *path);
    start of first section
 
    ------------------------------------------------------------ */
-void get_mp3_info(mp3info *mp3);
+void get_mp3_info(MP3Info *mp3);
 
 gint frequencies[3][4] = {
    {22050,24000,16000,50000},  /* MPEG 2.0 */
@@ -194,17 +197,17 @@ gchar *emphasis_text[] = {
 };
 
 
-static gint mp3file_header_bitrate(mp3header *h) {
+static gint mp3file_header_bitrate(MP3Header *h) {
     return bitrate[h->version & 1][3-h->layer][h->bitrate];
 }
 
 
-static gint mp3file_header_frequency(mp3header *h) {
+static gint mp3file_header_frequency(MP3Header *h) {
     return frequencies[h->version][h->freq];
 }
 
 
-gint frame_length(mp3header *header) {
+gint frame_length(MP3Header *header) {
 	return header->sync == 0xFFE ?
 		    (frame_size_index[3-header->layer]*((header->version&1)+1)*
 		    mp3file_header_bitrate(header)/(float)mp3file_header_frequency(header))+
@@ -216,7 +219,7 @@ gint frame_length(mp3header *header) {
    positive value = Frame Length of this header
    0 = No, we did not retrieve a valid frame header
 */
-gint get_header(FILE *file,mp3header *header)
+gint get_header(FILE *file,MP3Header *header)
 {
     guchar buffer[FRAME_HEADER_SIZE];
     gint fl;
@@ -251,7 +254,7 @@ gint get_header(FILE *file,mp3header *header)
     return ((fl=frame_length(header)) >= MIN_FRAME_SIZE ? fl : 0);
 }
 
-gint sameConstant(mp3header *h1, mp3header *h2) {
+gint sameConstant(MP3Header *h1, MP3Header *h2) {
     if((*(guint*)h1) == (*(guint*)h2)) return 1;
 
     if((h1->version       == h2->version         ) &&
@@ -267,10 +270,10 @@ gint sameConstant(mp3header *h1, mp3header *h2) {
 }
 
 
-gint get_first_header(mp3info *mp3, long startpos)
+gint get_first_header(MP3Info *mp3, long startpos)
 {
   gint k, l=0,c;
-  mp3header h, h2;
+  MP3Header h, h2;
   long valid_start=0;
 
   fseek(mp3->file,startpos,SEEK_SET);
@@ -288,7 +291,7 @@ gint get_first_header(mp3info *mp3, long startpos)
 	  }
 	  if(k == MIN_CONSEC_GOOD_FRAMES) {
 		fseek(mp3->file,valid_start,SEEK_SET);
-		memcpy(&(mp3->header),&h2,sizeof(mp3header));
+		memcpy(&(mp3->header),&h2,sizeof(MP3Header));
 		mp3->header_isvalid=1;
 		return 1;
 	  }
@@ -305,10 +308,10 @@ gint get_first_header(mp3info *mp3, long startpos)
 /* get_next_header() - read header at current position or look for
    the next valid header if there isn't one at the current position
 */
-gint get_next_header(mp3info *mp3)
+gint get_next_header(MP3Info *mp3)
 {
   gint l=0,c,skip_bytes=0;
-  mp3header h;
+  MP3Header h;
 
    while(1) {
      while((c=fgetc(mp3->file)) != 255 && (ftell(mp3->file) < mp3->datasize)) skip_bytes++;
@@ -329,7 +332,7 @@ gint get_next_header(mp3info *mp3)
 }
 
 
-void get_mp3_info(mp3info *mp3)
+void get_mp3_info(MP3Info *mp3)
 {
   gint frame_type[15]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
   float milliseconds=0,total_rate=0;
@@ -337,7 +340,7 @@ void get_mp3_info(mp3info *mp3)
   gint vbr_median=-1;
   guint bitrate;
   gint counter=0;
-  mp3header header;
+  MP3Header header;
   struct stat filestat;
   off_t data_start=0;
 
@@ -352,7 +355,7 @@ void get_mp3_info(mp3info *mp3)
 	      frame_type[15-bitrate]++;
 	  frames++;
       }
-      memcpy(&header,&(mp3->header),sizeof(mp3header));
+      memcpy(&header,&(mp3->header),sizeof(MP3Header));
       for(counter=0;counter<15;counter++) {
 	  if(frame_type[counter]) {
 	      float header_bitrate; /* introduced by JCS to speed up */
@@ -379,24 +382,6 @@ void get_mp3_info(mp3info *mp3)
 
 
 
-/* Returns a filled-in mp3info struct that must be g_free'd after use */
-static mp3info *mp3file_get_info (gchar *filename)
-{
-    mp3info *mp3 = NULL;
-    FILE *fp;
-
-    if ( !( fp=fopen(filename,"r") ) ) {
-	gtkpod_warning (_("Error opening MP3 file '%s'"),filename);
-    } else {
-	mp3 = g_malloc0 (sizeof (mp3info));
-	mp3->filename=filename;
-	mp3->file=fp;
-	get_mp3_info (mp3);
-	fclose (fp);
-    }
-    return mp3;
-}
-
 
 /* ------------------------------------------------------------
 
@@ -1003,37 +988,30 @@ static gchar* id3_get_string (struct id3_tag *tag, char *frame_name)
     return utf8;
 }
 
-static void id3_set_string (struct id3_tag *tag, const char *frame_name, const char *data, enum id3_field_textencoding encoding)
+static void id3_set_string (struct id3_tag *tag,
+			    const char *frame_name,
+			    const char *data,
+			    enum id3_field_textencoding encoding)
 {
     int res;
     struct id3_frame *frame;
     union id3_field *field;
     id3_ucs4_t *ucs4;
 
-    if (data == NULL)
-	return;
-
-/*    printf ("updating id3 (enc: %d): %s: %s\n", encoding, frame_name, data);*/
-
-    /*
-     * An empty string removes the frame altogether.
-     */
-    if (strlen(data) == 0)
+    /* clear the frame, because of bug in libid3tag see
+       http://www.mars.org/mailman/public/mad-dev/2004-October/001113.html
+    */
+    while ((frame = id3_tag_findframe (tag, frame_name, 0)))
     {
-/*	printf("removing ID3 frame: %s\n", frame_name);*/
-	while ((frame = id3_tag_findframe (tag, frame_name, 0)))
-	    id3_tag_detachframe (tag, frame);
-	return;
+	id3_tag_detachframe (tag, frame);
     }
 
-    frame = id3_tag_findframe (tag, frame_name, 0);
-    if (!frame)
-    {
-/*	puts("new frame!");*/
-	frame = id3_frame_new (frame_name);
-	id3_tag_attachframe (tag, frame);
-    }
+    if ((data == NULL) || (strlen(data) == 0))
+	return;
 
+    frame = id3_frame_new (frame_name);
+    id3_tag_attachframe (tag, frame);
+   
     if (frame_name == ID3_FRAME_COMMENT)
     {
 	field = id3_frame_field (frame, 3);
@@ -1048,6 +1026,10 @@ static void id3_set_string (struct id3_tag *tag, const char *frame_name, const c
     /* Use the specified text encoding */
     id3_field_settextencoding(field, encoding);
 
+
+    /* maybe could be optimized see
+       http://www.mars.org/mailman/public/mad-dev/2002-October/000739.html
+    */
     if (frame_name == ID3_FRAME_GENRE)
     {
 	id3_ucs4_t *tmp_ucs4 = id3_utf8_ucs4duplicate ((id3_utf8_t *)data);
@@ -1123,8 +1105,8 @@ gboolean id3_tag_read (gchar *filename, File_Tag *tag)
     gchar* string;
     gchar* string2;
 
-    if (!filename || !tag)
-	return FALSE;
+    g_return_val_if_fail (filename, FALSE);
+    g_return_val_if_fail (tag, FALSE);
 
     memset (tag, 0, sizeof (File_Tag));
 
@@ -1166,6 +1148,20 @@ gboolean id3_tag_read (gchar *filename, File_Tag *tag)
 	    tag->trackstring = g_strdup_printf ("%.2d", atoi (string));
 	    g_free(string);
 	}
+	
+	/* CD/disc number tag handling */
+	string = id3_get_string (id3tag, "TPOS");
+	if (string)
+	{
+	    string2 = strchr(string,'/');
+	    if (string2)
+	    {
+		tag->cdno_total = g_strdup_printf ("%.2d", atoi (string2+1));
+		*string2 = '\0';
+	    }
+	    tag->cdnostring = g_strdup_printf ("%.2d", atoi (string));
+	    g_free(string);
+	}
     }
 
     id3_file_close (id3file);
@@ -1244,12 +1240,23 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
 	    (encoding == ID3_FIELD_TEXTENCODING_ISO_8859_1))
 	    encoding = ID3_FIELD_TEXTENCODING_UTF_8;
 
+	/* always render id3v1 to prevent dj studio from crashing */
+	id3_tag_options(id3tag, ID3_TAG_OPTION_ID3V1, ~0);
+
+        /* turn off frame compression and crc information to let
+	   itunes read tags see
+	   http://www.mars.org/mailman/public/mad-dev/2002-October/000742.html
+	*/
+	id3_tag_options(id3tag, ID3_TAG_OPTION_COMPRESSION, 0);
+	id3_tag_options(id3tag, ID3_TAG_OPTION_CRC, 0);
+
 	id3_set_string (id3tag, ID3_FRAME_TITLE, track->title, encoding);
 	id3_set_string (id3tag, ID3_FRAME_ARTIST, track->artist, encoding);
 	id3_set_string (id3tag, ID3_FRAME_ALBUM, track->album, encoding);
 	id3_set_string (id3tag, ID3_FRAME_GENRE, track->genre, encoding);
 	id3_set_string (id3tag, ID3_FRAME_COMMENT, track->comment, encoding);
 	id3_set_string (id3tag, "TCOM", track->composer, encoding);
+
 	if (track->tracks)
 	    string1 = g_strdup_printf ("%d/%d",
 				       track->track_nr, track->tracks);
@@ -1257,7 +1264,15 @@ gboolean mp3_write_file_info (gchar *filename, Track *track)
 	    string1 = g_strdup_printf ("%d", track->track_nr);
 	id3_set_string (id3tag, ID3_FRAME_TRACK, string1, encoding);
 	g_free(string1);
-    }
+
+	if (track->cds)
+	    string1 = g_strdup_printf ("%d/%d",
+				       track->cd_nr, track->cds);
+	else
+	    string1 = g_strdup_printf ("%d", track->cd_nr);
+	id3_set_string (id3tag, "TPOS", string1, encoding);
+	g_free(string1);
+    } 
 
     if (id3_file_update(id3file) != 0)
     {
@@ -1337,6 +1352,11 @@ static gint lame_vcmp(gchar a[5], gchar b[5]) {
 static void read_lame_replaygain(char buf[], Track *track, int gain_adjust) {
 	char oc, nc;
 	gint gain;
+	ExtraTrackData *etr;
+
+	g_return_if_fail (track);
+	etr = track->userdata;
+	g_return_if_fail (etr);
 
 	/* check originator */
 	oc = (buf[0] & 0x1c) >> 2;
@@ -1359,17 +1379,17 @@ static void read_lame_replaygain(char buf[], Track *track, int gain_adjust) {
 
 	switch (nc) {
 		case 0x20:
-			if (track->radio_gain_set) return;
-			track->radio_gain = (gdouble)gain / 10;
-			track->radio_gain_set = TRUE;
-/*			printf("radio_gain (lame): %i\n", track->radio_gain); */
+			if (etr->radio_gain_set) return;
+			etr->radio_gain = (gdouble)gain / 10;
+			etr->radio_gain_set = TRUE;
+/*			printf("radio_gain (lame): %i\n", etr->radio_gain); */
 			break;
 		case 0x40:
-			if (track->audiophile_gain_set) return;
-			track->audiophile_gain = (gdouble)gain / 10;
-			track->audiophile_gain_set = TRUE;
+			if (etr->audiophile_gain_set) return;
+			etr->audiophile_gain = (gdouble)gain / 10;
+			etr->audiophile_gain_set = TRUE;
 /*			printf("audiophile_gain (lame): %i\n", 
-					track->audiophile_gain);*/
+					etr->audiophile_gain);*/
 			break;
 	}
 }
@@ -1398,7 +1418,7 @@ static inline guint32 parse_lame_uint32(char *buf) {
  * TODO: Check CRC.
  */
 
-gboolean mp3_get_track_lame_replaygain(gchar *path, Track *track)
+gboolean mp3_get_track_lame_replaygain (gchar *path, Track *track)
 {
 	struct {
 		/* All members are defined in terms of chars so padding does not
@@ -1413,23 +1433,30 @@ gboolean mp3_get_track_lame_replaygain(gchar *path, Track *track)
 
 	FILE *file = NULL;
 	char buf[4], version[5];
-
 	int gain_adjust = 0;
 	int sideinfo;
 	guint32 ps;
+	ExtraTrackData *etr;
 
-	track->radio_gain = 0;
-	track->audiophile_gain = 0;
-	track->peak_signal = 0;
-	track->radio_gain_set = FALSE;
-	track->audiophile_gain_set = FALSE;
-	track->peak_signal_set = FALSE;
+	g_return_val_if_fail (track, FALSE);
+	etr = track->userdata;
+	g_return_val_if_fail (etr, FALSE);
+
+	etr->radio_gain = 0;
+	etr->audiophile_gain = 0;
+	etr->peak_signal = 0;
+	etr->radio_gain_set = FALSE;
+	etr->audiophile_gain_set = FALSE;
+	etr->peak_signal_set = FALSE;
 	
 	if (!path)
 		goto rg_fail;
 	
 	file = fopen (path, "r");
-	
+
+	if (!file)
+	    goto rg_fail;
+
 	/* Skip ID3 header if appropriate */
 	if (fread(&id3head, 1, sizeof(id3head), file) != 
 			sizeof(id3head))
@@ -1514,18 +1541,18 @@ gboolean mp3_get_track_lame_replaygain(gchar *path, Track *track)
 	/* Don't know when fixed-point PeakSingleAmplitude
 	 * was introduced exactly. 3.94b will be used for now.) */
 	if ((lame_vcmp(version, "3.94b") >= 0)) {
-		if ((!track->peak_signal_set) && ps) {
-			track->peak_signal = ps;
-			track->peak_signal_set = TRUE;
+		if ((!etr->peak_signal_set) && ps) {
+			etr->peak_signal = ps;
+			etr->peak_signal_set = TRUE;
 /*			printf("peak_signal (lame): %f\n", (double)
-					track->peak_signal / 0x800000);*/
+					etr->peak_signal / 0x800000);*/
 		}
 	} else {
 		float f = *((float *) (void *) (&ps)) * 0x800000;
-		track->peak_signal = (guint32) f;
+		etr->peak_signal = (guint32) f;
 		/* I would like to see an example of that. */
 /*		printf("peak_signal (lame floating point): %f. PLEASE report.\n", 
-				(double) track->peak_signal / 0x800000);*/
+				(double) etr->peak_signal / 0x800000);*/
 	}
 
 	/*
@@ -1542,13 +1569,13 @@ gboolean mp3_get_track_lame_replaygain(gchar *path, Track *track)
 		goto rg_fail;
 
 	/* radio gain */
-	read_lame_replaygain(buf, track, gain_adjust);
+	read_lame_replaygain (buf, track, gain_adjust);
 
 	if (fread(&buf[0], 1, 2, file) != 2)
 		goto rg_fail;
 
 	/* audiophile gain */
-	read_lame_replaygain(buf, track, gain_adjust);
+	read_lame_replaygain (buf, track, gain_adjust);
 
 	fclose(file);
 	return TRUE;
@@ -1591,12 +1618,18 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 	guint32 entry_length = 0;
 	guint32 entries;
 	double d;
+	ExtraTrackData *etr;
+
+	g_return_val_if_fail (track, FALSE);
+	etr = track->userdata;
+	g_return_val_if_fail (etr, FALSE);
+	g_return_val_if_fail (path, FALSE);
 
-	if (!path)
-		goto rg_fail;
-	
 	file = fopen (path, "r");
 
+	if (!file)
+	    goto rg_fail;
+
 	/* check for ID3v1 Tag */
 	if (fseek(file, -ID3V1_SIZE, SEEK_END) ||
 			fread(&buf, 1, 3, file) != 3)
@@ -1658,7 +1691,7 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 		goto rg_fail;
 	
 	for (i = 0; i < entries; i++) {
-		if (track->radio_gain_set && track->peak_signal_set) break;
+		if (etr->radio_gain_set && etr->peak_signal_set) break;
 		pos = pos2 + entry_length;
 		if (pos > data_length - 10) break;
 		
@@ -1673,7 +1706,7 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 		if (entry_length + 1 > sizeof(buf))
 			continue;
 /* 		printf ("%s:%d:%d\n",&dbuf[pos], pos2, pos); */
-		if (!track->radio_gain_set && !strcasecmp(&dbuf[pos],
+		if (!etr->radio_gain_set && !strcasecmp(&dbuf[pos],
 					"REPLAYGAIN_TRACK_GAIN")) {
 			memcpy(buf, &dbuf[pos2], entry_length);
 			buf[entry_length] = '\0';
@@ -1682,14 +1715,14 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 /* 			printf("%f\n", d); */
 			if ((ep == buf + entry_length - 3) 
 					&& (!strncasecmp(ep, " dB", 3))) {
-			    track->radio_gain = d;
-				track->radio_gain_set = TRUE;
-/*				printf("radio_gain (ape): %i\n", track->radio_gain);*/
+			    etr->radio_gain = d;
+				etr->radio_gain_set = TRUE;
+/*				printf("radio_gain (ape): %i\n", etr->radio_gain);*/
 			}
 			
 			continue;
 		}
-		if (!track->peak_signal_set && !strcasecmp(&dbuf[pos],
+		if (!etr->peak_signal_set && !strcasecmp(&dbuf[pos],
 					"REPLAYGAIN_TRACK_PEAK")) {
 			memcpy(buf, &dbuf[pos2], entry_length);
 			buf[entry_length] = '\0';
@@ -1697,9 +1730,9 @@ gboolean mp3_get_track_ape_replaygain(gchar *path, Track *track)
 			d = g_ascii_strtod(buf, &ep);
 			if (ep == buf + entry_length) {
 				d *= 0x800000;
-				track->peak_signal = (guint32) floor(d + 0.5);
-				track->peak_signal_set = TRUE;
-/*				printf("peak_signal (ape): %f\n", (double) track->peak_signal / 0x800000);*/
+				etr->peak_signal = (guint32) floor(d + 0.5);
+				etr->peak_signal_set = TRUE;
+/*				printf("peak_signal (ape): %f\n", (double) etr->peak_signal / 0x800000);*/
 			}
 
 			continue;
@@ -1828,20 +1861,26 @@ static gboolean mp3_calc_gain (gchar *path, Track *track)
 
 gboolean mp3_get_gain (gchar *path, Track *track) 
 {
-	track->radio_gain_set = FALSE;
-	track->audiophile_gain_set = FALSE;
-	track->peak_signal_set = FALSE;
+    ExtraTrackData *etr;
 
-	mp3_get_track_lame_replaygain(path, track);
-/* 	printf ("%d:%d\n", track->radio_gain_set, track->peak_signal_set); */
-	if (track->radio_gain_set && track->peak_signal_set) return TRUE;
-	mp3_get_track_ape_replaygain(path, track);
-	if (track->radio_gain_set) return TRUE;
+    g_return_val_if_fail (track, FALSE);
+    etr = track->userdata;
+    g_return_val_if_fail (etr, FALSE);
+
+    etr->radio_gain_set = FALSE;
+    etr->audiophile_gain_set = FALSE;
+    etr->peak_signal_set = FALSE;
+
+    mp3_get_track_lame_replaygain (path, track);
+/*    printf ("%d:%d\n", track->radio_gain_set, track->peak_signal_set); */
+    if (etr->radio_gain_set && etr->peak_signal_set) return TRUE;
+    mp3_get_track_ape_replaygain (path, track);
+    if (etr->radio_gain_set) return TRUE;
 	    
-	if (mp3_calc_gain(path, track))
-	    mp3_get_track_ape_replaygain(path, track);
-	if (track->radio_gain_set) return TRUE;
-	return FALSE;
+    if (mp3_calc_gain (path, track))
+	mp3_get_track_ape_replaygain (path, track);
+    if (etr->radio_gain_set) return TRUE;
+    return FALSE;
 }
 
 
@@ -1855,17 +1894,23 @@ gboolean mp3_get_gain (gchar *path, Track *track)
 
 gboolean mp3_read_gain (gchar *path, Track *track) 
 {
-	track->radio_gain_set = FALSE;
-	track->audiophile_gain_set = FALSE;
-	track->peak_signal_set = FALSE;
+    ExtraTrackData *etr;
 
-	mp3_get_track_lame_replaygain(path, track);
-	if (track->radio_gain_set && track->peak_signal_set) return TRUE;
-	
-	mp3_get_track_ape_replaygain(path, track);
-	if (track->radio_gain_set) return TRUE;
-	    
-	return FALSE;
+    g_return_val_if_fail (track, FALSE);
+    etr = track->userdata;
+    g_return_val_if_fail (etr, FALSE);
+
+    etr->radio_gain_set = FALSE;
+    etr->audiophile_gain_set = FALSE;
+    etr->peak_signal_set = FALSE;
+
+    mp3_get_track_lame_replaygain (path, track);
+    if (etr->radio_gain_set && etr->peak_signal_set) return TRUE;
+
+    mp3_get_track_ape_replaygain (path, track);
+    if (etr->radio_gain_set) return TRUE;
+    
+    return FALSE;
 }
 
 
@@ -1882,9 +1927,32 @@ Track *mp3_get_file_info (gchar *name)
 {
     Track *track = NULL;
     File_Tag filetag;
-    mp3info *mp3info;
+    MP3Info *mp3i=NULL;
+    FILE *file;
 
-    track = itunesdb_new_track ();
+    g_return_val_if_fail (name, NULL);
+
+    /* Attempt to open the file */
+    file = fopen (name, "r");
+    if (file)
+    {
+	mp3i = g_malloc0 (sizeof (MP3Info));
+	mp3i->filename = name;
+	mp3i->file = file;
+	get_mp3_info (mp3i);
+	mp3i->file = NULL;
+	fclose (file);
+    }
+    else
+    {
+	gchar *fbuf = charset_to_utf8 (name);
+    	gtkpod_warning(_("ERROR while opening file: '%s' (%s).\n"),
+		       fbuf, g_strerror(errno));
+	g_free (fbuf);
+	return NULL;
+    }
+
+    track = gp_track_new ();
     if (prefs_get_readtags() && (id3_tag_read (name, &filetag) == TRUE))
     {
 	track->fdesc = g_strdup ("MPEG audio file");
@@ -1947,18 +2015,37 @@ Track *mp3_get_file_info (gchar *name)
 	    track->tracks = atoi(filetag.track_total);
 	    g_free (filetag.track_total);
 	}
+	/* CD/disc number handling */
+	if (filetag.cdnostring == NULL)
+	{
+	    track->cd_nr = 0;
+	}
+	else
+	{
+	    track->cd_nr = atoi(filetag.cdnostring);
+	    g_free (filetag.cdnostring);
+	}
+	
+	if (filetag.cdno_total == NULL)
+	{
+	    track->cds = 0;
+	}
+	else
+	{
+	    track->cds = atoi(filetag.cdno_total);
+	    g_free (filetag.cdno_total);
+	}
     }
 
     mp3_read_gain (name, track);
 
     /* Get additional info (play time and bitrate */
-    mp3info = mp3file_get_info (name);
-    if (mp3info)
+    if (mp3i)
     {
-	track->tracklen = mp3info->milliseconds;
-	track->bitrate = (gint)(mp3info->vbr_average);
- 	track->samplerate = mp3file_header_frequency (&mp3info->header);
-	g_free (mp3info);
+	track->tracklen = mp3i->milliseconds;
+	track->bitrate = (gint)(mp3i->vbr_average);
+ 	track->samplerate = mp3file_header_frequency (&mp3i->header);
+	g_free (mp3i);
     }
     /* Fall back to xmms code if tracklen is 0 */
     if (track->tracklen == 0)
@@ -1973,9 +2060,8 @@ Track *mp3_get_file_info (gchar *name)
 	/* Tracks with zero play length are ignored by iPod... */
 	gtkpod_warning (_("File \"%s\" has zero play length. Ignoring.\n"),
 			name);
-	free_track (track);
+	gp_track_free (track);
 	track = NULL;
     }
-
     return track;
 }
