@@ -1,8 +1,9 @@
-/* Time-stamp: <2005-01-02 18:27:53 jcs>
+/* Time-stamp: <2005-07-16 16:11:00 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
 |
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: prefs.c,v 1.125 2004/12/29 10:00:32 jcsjcs Exp $
+|  $Id: prefs.c,v 1.146 2005/07/16 07:13:50 jcsjcs Exp $
 */
 
 /* -------------------------------------------------------------------
@@ -42,11 +43,10 @@
  *   prefs_set_new_option() should verify that the value passed is
  *   valid.
  *
- * - add a function prefs_window_set_new_option to
- *   prefs_window.[ch]. This function is called from the callback
- *   functions in callback.c to set the state of the option. The value
- *   is applied to the actual prefs when pressing the "OK" or "Apply"
- *   button in the prefs window.
+ * - add a callback on_new_option_*() to prefs_windows.c to set the
+ *   new value in the temporary prefs struct.
+ *   The value is applied to the actual prefs when pressing the "OK"
+ *   or "Apply" button in the prefs window.
  *
  * - if your option is a pointer to data, make sure the data is copied
  *   in clone_prefs() in prefs.c
@@ -99,10 +99,9 @@
 #include "clientserver.h"
 #include "display.h"
 #include "info.h"
-#include "md5.h"
 #include "misc.h"
+#include "misc_track.h"
 #include "prefs.h"
-#include "support.h"
 
 /* static function declarations */
 static void prefs_write_hash_values (FILE *fp);
@@ -136,16 +135,17 @@ static void usage (FILE *file)
   usage_fpf(file, _("  --offline:    same as '-o'.\n"));
 }
 
+
 struct cfg *cfg_new(void)
 {
     struct cfg *mycfg = NULL;
-    gchar buf[PATH_MAX], *str;
+    gchar curdir[PATH_MAX], *str;
     gint i;
 
     mycfg = g_malloc0 (sizeof (struct cfg));
-    if(getcwd(buf, PATH_MAX))
+    if(getcwd(curdir, PATH_MAX))
     {
-	mycfg->last_dir.browse = g_strdup_printf ("%s/", buf);
+	mycfg->last_dir.browse = g_strdup_printf ("%s/", curdir);
     }
     else
     {
@@ -162,11 +162,14 @@ struct cfg *cfg_new(void)
     {
 	mycfg->ipod_mount = g_strdup("/mnt/ipod");
     }
+
     mycfg->charset = NULL;
     mycfg->deletion.track = TRUE;
     mycfg->deletion.ipod_file = TRUE;
+    mycfg->deletion.local_file = TRUE;
+    mycfg->deletion.database = TRUE;
     mycfg->deletion.syncing = TRUE;
-    mycfg->md5tracks = FALSE;
+    mycfg->md5tracks = TRUE;
     mycfg->update_existing = FALSE;
     mycfg->block_display = FALSE;
     mycfg->autoimport = FALSE;
@@ -184,8 +187,8 @@ struct cfg *cfg_new(void)
 	mycfg->st[i].sp_played_state = g_strdup (">4w");
 	mycfg->st[i].sp_modified = FALSE;
 	mycfg->st[i].sp_modified_state = g_strdup ("<1d");
-	mycfg->st[i].sp_created = FALSE;
-	mycfg->st[i].sp_created_state = g_strdup ("<1d");
+	mycfg->st[i].sp_added = FALSE;
+	mycfg->st[i].sp_added_state = g_strdup ("<1d");
 	mycfg->st[i].sp_autodisplay = FALSE;
     }
     mycfg->mpl_autoselect = TRUE;
@@ -288,22 +291,24 @@ struct cfg *cfg_new(void)
     mycfg->mserv_use = FALSE;
     mycfg->mserv_report_probs = TRUE;
     mycfg->mserv_username = g_strdup ("");
+
     return(mycfg);
 }
 
 
 /* Compare strlen(@arg) chars of @arg with @line. Return strlen (@arg)
-   in @off */
+   in @off if @off != NULL */
 static gint arg_comp (const gchar *line, const gchar *arg, gint *off)
 {
     if (arg && line)
     {
-	*off = strlen (arg);
-	return g_ascii_strncasecmp (line, arg, *off);
+	gint len = strlen (arg);
+	if (off) *off = len;
+	return g_ascii_strncasecmp (line, arg, len);
     }
     else
     {
-	*off = 0;
+	if (*off)   *off = 0;
 	return 0;
     }
 }
@@ -327,13 +332,43 @@ static gint correct_sort (gint sort)
     return sort;
 }
 
+/* default ignore strings -- must end with a space */
+static char* sort_ign_strings[] =
+{
+    "the ",
+    "a ",
+/*    "le ", 
+    "la ", 
+    "les ", 
+    "lo ", 
+    "los ",
+    "der ",
+    "die ",
+    "das ",*/ /* will make sorting very slow -- only add the words you
+                 really want to skip */
+    SORT_IGNORE_STRINGS_END,  /* end marker */
+    NULL,
+};
+
 
 static void
 read_prefs_from_file_desc(FILE *fp)
 {
     gchar buf[PATH_MAX];
     gchar *line, *arg, *bufp;
-    gint len, off;
+    gint len, off, i;
+
+    /* set ignore strings */
+    for (i=0; sort_ign_strings[i]; ++i)
+    {
+	bufp = g_strdup_printf ("sort_ign_string_%d", i);
+	prefs_set_string_value (bufp, sort_ign_strings[i]);
+	g_free (bufp);
+    }
+    /* set ignore fields (ignore above words for artist) */
+    bufp = g_strdup_printf ("sort_ign_field_%d", T_ARTIST);
+    prefs_set_int_value (bufp, 1);
+    g_free (bufp);
 
     if(fp)
     {
@@ -471,14 +506,18 @@ read_prefs_from_file_desc(FILE *fp)
 	  {
 	      prefs_set_track_playlist_deletion((gboolean)atoi(arg));
 	  }
+	  else if(g_ascii_strcasecmp (line, "delete_database") == 0)
+	  {
+	      prefs_set_track_database_deletion((gboolean)atoi(arg));
+	  }
+	  else if(g_ascii_strcasecmp (line, "delete_local_file") == 0)
+	  {
+	      prefs_set_track_local_file_deletion((gboolean)atoi(arg));
+	  }
 	  else if(g_ascii_strcasecmp (line, "delete_playlist") == 0)
 	  {
 	      /* ignore -- no longer supported as of 0.61-CVS */
 	  }
-	  else if(g_ascii_strcasecmp (line, "delete_ipod") == 0)
-	  {
-	      prefs_set_track_ipod_file_deletion((gboolean)atoi(arg));
-	  }
 	  else if(g_ascii_strcasecmp (line, "sync_remove_confirm") == 0)
 	  {
 	      prefs_set_sync_remove_confirm((gboolean)atoi(arg));
@@ -523,10 +562,11 @@ read_prefs_from_file_desc(FILE *fp)
 	      gint i = atoi (line+off);
 	      prefs_set_sp_cond (i, T_TIME_MODIFIED, atoi (arg));
 	  }
-	  else if(arg_comp (line, "sp_created_cond", &off) == 0)
+	  else if((arg_comp (line, "sp_created_cond", &off) == 0) ||
+		  arg_comp (line, "sp_added_cond", &off) == 0)
 	  {
 	      gint i = atoi (line+off);
-	      prefs_set_sp_cond (i, T_TIME_CREATED, atoi (arg));
+	      prefs_set_sp_cond (i, T_TIME_ADDED, atoi (arg));
 	  }
 	  else if(arg_comp (line, "sp_rating_state", &off) == 0)
 	  {
@@ -553,10 +593,11 @@ read_prefs_from_file_desc(FILE *fp)
 	      gint i = atoi (line+off);
 	      prefs_set_sp_entry (i, T_TIME_MODIFIED, arg);
 	  }
-	  else if(arg_comp (line, "sp_created_state", &off) == 0)
+	  else if((arg_comp (line, "sp_created_state", &off) == 0) ||
+		  (arg_comp (line, "sp_added_state", &off) == 0))
 	  {
 	      gint i = atoi (line+off);
-	      prefs_set_sp_entry (i, T_TIME_CREATED, arg);
+	      prefs_set_sp_entry (i, T_TIME_ADDED, arg);
 	  }
 	  else if(arg_comp (line, "sp_autodisplay", &off) == 0)
 	  {
@@ -862,7 +903,16 @@ read_prefs_from_file_desc(FILE *fp)
 	  {   /* All leftover options will be stored into the prefs
 		 setting hash (generic options -- should have had this
 		 idea much sooner... */
-	      prefs_set_string_value (line, arg);
+	      gboolean skip = FALSE;
+	      if (cfg->version < 0.91)
+	      {
+		  if(arg_comp (line, "itdb_", NULL) == 0)
+		  {   /* set incorrectly in 0.90 -- delete */
+		      skip = TRUE;
+		  }
+	      }
+	      if (!skip)
+		  prefs_set_string_value (line, arg);
 	  }
 	  g_free(line);
 	}
@@ -993,7 +1043,7 @@ gboolean read_prefs (GtkWidget *gtkpod, int argc, char *argv[])
 	  prefs_set_ipod_mount (optarg);
 	  break;
       case GP_AUTO:
-	  prefs_set_autoimport (TRUE);
+	  prefs_set_autoimport_commandline (TRUE);
 	  break;
       case GP_OFFLINE:
 	  prefs_set_offline (TRUE);
@@ -1004,7 +1054,7 @@ gboolean read_prefs (GtkWidget *gtkpod, int argc, char *argv[])
 	  exit(1);
       }
   }
-  menu = GTK_CHECK_MENU_ITEM (lookup_widget (gtkpod, "offline_menu"));
+  menu = GTK_CHECK_MENU_ITEM (glade_xml_get_widget (main_window_xml, "offline_menu"));
   gtk_check_menu_item_set_active (menu, prefs_get_offline ());
   return result;
 }
@@ -1050,6 +1100,8 @@ write_prefs_to_file_desc(FILE *fp)
     fprintf(fp, _("# delete confirmation\n"));
     fprintf(fp, "delete_file=%d\n",prefs_get_track_playlist_deletion());
     fprintf(fp, "delete_ipod=%d\n",prefs_get_track_ipod_file_deletion());
+    fprintf(fp, "delete_database=%d\n",prefs_get_track_database_deletion());
+    fprintf(fp, "delete_local_file=%d\n",prefs_get_track_local_file_deletion());
     fprintf(fp, "sync_remove_confirm=%d\n",prefs_get_sync_remove_confirm());
     fprintf(fp, "autoimport=%d\n",prefs_get_autoimport());
     fprintf(fp, _("# sort tab: select 'All', last selected page (=category)\n"));
@@ -1067,8 +1119,8 @@ write_prefs_to_file_desc(FILE *fp)
 	fprintf(fp, "sp_played_state%d=%s\n", i, prefs_get_sp_entry (i, T_TIME_PLAYED));
 	fprintf(fp, "sp_modified_cond%d=%d\n", i, prefs_get_sp_cond (i, T_TIME_MODIFIED));
 	fprintf(fp, "sp_modified_state%d=%s\n", i, prefs_get_sp_entry (i, T_TIME_MODIFIED));
-	fprintf(fp, "sp_created_cond%d=%d\n", i, prefs_get_sp_cond (i, T_TIME_CREATED));
-	fprintf(fp, "sp_created_state%d=%s\n", i, prefs_get_sp_entry (i, T_TIME_CREATED));
+	fprintf(fp, "sp_added_cond%d=%d\n", i, prefs_get_sp_cond (i, T_TIME_ADDED));
+	fprintf(fp, "sp_added_state%d=%s\n", i, prefs_get_sp_entry (i, T_TIME_ADDED));
 	fprintf(fp, "sp_autodisplay%d=%d\n", i, prefs_get_sp_autodisplay (i));
     }
     fprintf(fp, _("# autoselect master playlist?\n"));
@@ -1207,6 +1259,9 @@ void cfg_free(struct cfg *c)
 
 void sortcfg_free(struct sortcfg *c)
 {
+    g_return_if_fail (c);
+    g_list_free (c->tmp_sort_ign_fields);
+    g_free (c->tmp_sort_ign_strings);
     g_free (c);
 }
 
@@ -1269,23 +1324,39 @@ void prefs_set_ipod_mount(const gchar *mp)
     else cfg->ipod_mount = g_strdup(mp);
     /* need to notify the info thread of new mount point */
     space_set_ipod_mount (cfg->ipod_mount);
+    gp_itdb_set_mountpoint (cfg->ipod_mount);
 }
 
 
 /* If the status of md5 hash flag changes, free or re-init the md5
    hash table */
-void prefs_set_md5tracks(gboolean active)
+void prefs_set_md5tracks (gboolean active)
 {
+    struct itdbs_head *itdbs_head;
+
+    g_return_if_fail (gtkpod_window);
+    itdbs_head = g_object_get_data (G_OBJECT (gtkpod_window),
+				    "itdbs_head");
+/*    g_return_if_fail (itdbs_head);*/
+    /* gets called before itdbs are set up -> fail silently */
+    if (!itdbs_head)
+    {
+	cfg->md5tracks = active;
+	return;
+    }
+
     if (cfg->md5tracks && !active)
     { /* md5 checksums have been turned off */
-	cfg->md5tracks = active;
-	md5_unique_file_free ();
+	cfg->md5tracks = FALSE;
+	gp_md5_free_hash ();
     }
     if (!cfg->md5tracks && active)
     { /* md5 checksums have been turned on */
-	cfg->md5tracks = active; /* must be set before calling
-				   hash_tracks() */
-	hash_tracks ();
+	cfg->md5tracks = TRUE; /* must be set before calling
+				  gp_md5_hash_tracks() */
+	gp_md5_hash_tracks ();
+	/* display duplicates */
+	gp_duplicate_remove (NULL, NULL);
     }
 }
 
@@ -1366,6 +1437,27 @@ gboolean prefs_get_track_ipod_file_deletion(void)
     return(cfg->deletion.ipod_file);
 }
 
+void prefs_set_track_database_deletion(gboolean val)
+{
+    cfg->deletion.database = val;
+}
+
+gboolean prefs_get_track_database_deletion(void)
+{
+    return (cfg->deletion.database);
+}
+
+void prefs_set_track_local_file_deletion(gboolean val)
+{
+    cfg->deletion.local_file = val;
+}
+
+gboolean prefs_get_track_local_file_deletion(void)
+{
+    return cfg->deletion.local_file;
+}
+
+
 void prefs_set_sync_remove_confirm(gboolean val)
 {
     cfg->deletion.syncing = val;
@@ -1425,6 +1517,9 @@ struct sortcfg *clone_sortprefs(void)
     {
 	result = g_memdup (&cfg->sortcfg, sizeof (struct sortcfg));
     }
+    /* GLists are not copied */
+    result->tmp_sort_ign_fields = NULL;
+    result->tmp_sort_ign_strings = NULL;
     return(result);
 }
 
@@ -1438,6 +1533,16 @@ void prefs_set_autoimport(gboolean val)
     cfg->autoimport = val;
 }
 
+gboolean prefs_get_autoimport_commandline(void)
+{
+    return(cfg->autoimport_commandline);
+}
+
+void prefs_set_autoimport_commandline(gboolean val)
+{
+    cfg->autoimport_commandline = val;
+}
+
 /* "inst": the instance of the sort tab */
 gboolean prefs_get_st_autoselect (guint32 inst)
 {
@@ -2239,8 +2344,8 @@ void prefs_set_sp_cond (guint32 inst, T_item t_item, gboolean state)
 	case T_TIME_MODIFIED:
 	    cfg->st[inst].sp_modified = state;
 	    break;
-	case T_TIME_CREATED:
-	    cfg->st[inst].sp_created = state;
+	case T_TIME_ADDED:
+	    cfg->st[inst].sp_added = state;
 	    break;
 	default:
 	    /* programming error */
@@ -2273,8 +2378,8 @@ gboolean prefs_get_sp_cond (guint32 inst, T_item t_item)
 	    return cfg->st[inst].sp_played;
 	case T_TIME_MODIFIED:
 	    return cfg->st[inst].sp_modified;
-	case T_TIME_CREATED:
-	    return cfg->st[inst].sp_created;
+	case T_TIME_ADDED:
+	    return cfg->st[inst].sp_added;
 	default:
 	    /* programming error */
 	    fprintf (stderr, "prefs_get_sp_cond(): inst=%d !t_item=%d!\n",
@@ -2358,9 +2463,9 @@ void prefs_set_sp_entry (guint32 inst, T_item t_item, const gchar *str)
 	    g_free (cfg->st[inst].sp_modified_state);
 	    cfg->st[inst].sp_modified_state = cstr;
 	    break;
-	case T_TIME_CREATED:
-	    g_free (cfg->st[inst].sp_created_state);
-	    cfg->st[inst].sp_created_state = cstr;
+	case T_TIME_ADDED:
+	    g_free (cfg->st[inst].sp_added_state);
+	    cfg->st[inst].sp_added_state = cstr;
 	    break;
 	default:
 	    /* programming error */
@@ -2395,8 +2500,8 @@ gchar *prefs_get_sp_entry (guint32 inst, T_item t_item)
 	case T_TIME_MODIFIED:
 	    result = cfg->st[inst].sp_modified_state;
 	    break;
-	case T_TIME_CREATED:
-	    result = cfg->st[inst].sp_created_state;
+	case T_TIME_ADDED:
+	    result = cfg->st[inst].sp_added_state;
 	    break;
 	default:
 	    /* programming error */
@@ -2602,7 +2707,7 @@ void prefs_set_int64_value (const gchar *key, gint64 value)
 
     if (!key)  return;
 
-    str = g_strdup_printf ("%lld", value);
+    str = g_strdup_printf ("%llu", value);
     prefs_set_string_value (key, str);
     g_free (str);
 }
@@ -2614,6 +2719,8 @@ void prefs_set_int64_value (const gchar *key, gint64 value)
  * @value is set to NULL */
 gboolean prefs_get_string_value (const gchar *key, gchar **value)
 {
+    g_return_val_if_fail (value, FALSE);
+
     *value = NULL;
     if (prefs_hash && key)
     {
@@ -2623,12 +2730,25 @@ gboolean prefs_get_string_value (const gchar *key, gchar **value)
 }
 
 
+/* Same as prefs_get_string_value() but easier to use. The string is
+   simply returned, a returned NULL value means that the @key could
+   not be found. g_free() the result when it's no longer needed */
+gchar *prefs_get_string (const gchar *key)
+{
+    if (prefs_hash && key)
+	return g_strdup (g_hash_table_lookup (prefs_hash, key));
+    return NULL;
+}
+
+
 /* Retrieve an integer setting. @value will be filled with the stored
  * integer value. */
 /* Return value: TRUE, if @key could be retrieved. Otherwise FALSE and
  * @value is set to 0 */
 gboolean prefs_get_int_value (const gchar *key, gint *value)
 {
+    g_return_val_if_fail (value, FALSE);
+
     *value = 0;
     if (prefs_hash && key)
     {
@@ -2643,19 +2763,37 @@ gboolean prefs_get_int_value (const gchar *key, gint *value)
 }
 
 
-/* Retrieve an 64 bit integer setting. @value will be filled with the
+/* Retrieve an integer setting. */
+/* Return value: the set integer value or 0 if no value is set. */
+/* Use prefs_get_int_value if you need to know whether a value was
+   actually set in the prefs or not. */
+gint prefs_get_int (const gchar *key)
+{
+    if (prefs_hash && key)
+    {
+	gchar *str = g_hash_table_lookup (prefs_hash, key);
+	if (str)
+	    return atoi (str);
+    }
+    return 0;
+}
+
+
+/* Retrieve a 64 bit integer setting. @value will be filled with the
  * stored integer value. */
 /* Return value: TRUE, if @key could be retrieved. Otherwise FALSE and
  * @value is set to 0 */
 gboolean prefs_get_int64_value (const gchar *key, gint64 *value)
 {
+    g_return_val_if_fail (value, FALSE);
+
     *value = 0;
     if (prefs_hash && key)
     {
 	gchar *str = g_hash_table_lookup (prefs_hash, key);
 	if (str)
 	{
-	    *value = atoll (str);
+	    *value = (gint64)g_ascii_strtoull (str, NULL, 10);
 	    return TRUE;
 	}
     }
@@ -2663,12 +2801,19 @@ gboolean prefs_get_int64_value (const gchar *key, gint64 *value)
 }
 
 
-/* used by prefs_write_hash_value () */
-static void prefs_write_hash_func (gpointer key, gpointer value,
-				   gpointer fp)
+/* Retrieve a 64 bit integer setting. */
+/* Return value: the set integer value or 0 if no value is set. */
+/* Use prefs_get_int64_value if you need to know whether a value was
+   actually set in the prefs or not. */
+gint64 prefs_get_int64 (const gchar *key)
 {
-    g_return_if_fail (key && value && fp);
-    fprintf ((FILE *)fp, "%s=%s\n", (gchar *)key, (gchar *)value);
+    if (prefs_hash && key)
+    {
+	gchar *str = g_hash_table_lookup (prefs_hash, key);
+	if (str)
+	    return (gint64)g_ascii_strtoull (str, NULL, 10);
+    }
+    return 0;
 }
 
 
@@ -2676,6 +2821,13 @@ static void prefs_write_hash_func (gpointer key, gpointer value,
    @fp */
 static void prefs_write_hash_values (FILE *fp)
 {
+    void prefs_write_hash_func (gpointer key, gpointer value,
+				gpointer fp)
+	{
+	    g_return_if_fail (key && value && fp);
+	    fprintf ((FILE *)fp, "%s=%s\n", (gchar *)key, (gchar *)value);
+	}
+
     if (prefs_hash)
 	g_hash_table_foreach (prefs_hash, prefs_write_hash_func, fp);
 }
