@@ -1,8 +1,9 @@
-/* Time-stamp: <2004-12-29 18:13:36 jcs>
+/* Time-stamp: <2005-07-01 23:16:54 jcs>
 |
-|  Copyright (C) 2002-2003 Jorg Schuler <jcsjcs at users.sourceforge.net>
+|  Copyright (C) 2002-2005 Jorg Schuler <jcsjcs at users sourceforge net>
 |  Part of the gtkpod project.
-|
+| 
+|  URL: http://www.gtkpod.org/
 |  URL: http://gtkpod.sourceforge.net/
 |
 |  This program is free software; you can redistribute it and/or modify
@@ -23,7 +24,7 @@
 |
 |  This product is not supported/written/published by Apple!
 |
-|  $Id: prefs.h,v 1.90 2004/12/29 10:00:32 jcsjcs Exp $
+|  $Id: prefs.h,v 1.97 2005/07/01 16:53:44 jcsjcs Exp $
 */
 
 #ifndef __PREFS_H__
@@ -70,6 +71,8 @@ struct cfg
   gboolean tmp_disable_sort;/* tmp. disable sorting during change of slctn? */
   gboolean startup_messages;/* show startup messages/warnings? */
   gboolean autoimport;	    /* whether or not to automatically import files */
+  gboolean autoimport_commandline;/* whether or not to automatically
+				   * import files as set by commandline */
   struct
   {
     gboolean autoselect;     /* automatically select "All" in sort tab? */
@@ -85,8 +88,8 @@ struct cfg
     gchar *sp_played_state;  /* current "played" string */
     gboolean sp_modified;    /* consider last modified? */
     gchar *sp_modified_state;/* current "modified" string */
-    gboolean sp_created;     /* consider created? */
-    gchar *sp_created_state; /* current "created" string */
+    gboolean sp_added;       /* consider added? */
+    gchar *sp_added_state;   /* current "added" string */
     gboolean sp_autodisplay; /* display automatically? */
   } st[SORT_TAB_MAX];
   struct sortcfg
@@ -98,6 +101,8 @@ struct cfg
     gboolean pm_autostore;   /* save sort order automatically?    */
     gboolean tm_autostore;   /* save sort order automatically?    */
     gboolean case_sensitive; /* Should sorting be case-sensitive? */
+    GList *tmp_sort_ign_fields; /* used in prefs_window.c only     */
+    gchar *tmp_sort_ign_strings;/* used in prefs_window.c only     */
   } sortcfg;
   gboolean info_window;   /* is info window open (will then open on restart */
   gboolean mpl_autoselect;/* select mpl automatically? */
@@ -110,7 +115,7 @@ struct cfg
   } last_dir;	          /* last directories used by the fileselections */
   struct
   {
-      gboolean track, ipod_file, syncing;
+      gboolean track, ipod_file, local_file, database, syncing;
   } deletion;
   struct win_size size_gtkpod;  /* last size of gtkpod main window */
   struct win_size size_cal;     /* last size of calendar window */
@@ -167,6 +172,8 @@ enum
     SORT_NONE = 10*(GTK_SORT_ASCENDING+GTK_SORT_DESCENDING),
 };
 
+/* marker for end of sort_ign_string_%d prefs settings */
+#define SORT_IGNORE_STRINGS_END "----++++----"
 
 gchar *prefs_get_cfgdir (void);
 void prefs_print(void);
@@ -192,9 +199,6 @@ void prefs_set_autoimport(gboolean val);
 void prefs_set_st_autoselect (guint32 inst, gboolean autoselect);
 void prefs_set_mpl_autoselect (gboolean autoselect);
 void prefs_set_st_category (guint32 inst, guint category);
-void prefs_set_track_playlist_deletion(gboolean val);
-void prefs_set_track_ipod_file_deletion(gboolean val);
-void prefs_set_sync_remove_confirm(gboolean val);
 void prefs_set_md5tracks(gboolean active);
 void prefs_set_update_existing(gboolean active);
 void prefs_set_block_display(gboolean active);
@@ -241,9 +245,6 @@ gboolean prefs_get_autoimport(void);
 gboolean prefs_get_st_autoselect (guint32 inst);
 gboolean prefs_get_mpl_autoselect (void);
 guint prefs_get_st_category (guint32 inst);
-gboolean prefs_get_track_playlist_deletion(void);
-gboolean prefs_get_track_ipod_file_deletion(void);
-gboolean prefs_get_sync_remove_confirm(void);
 gboolean prefs_get_id3_write(void);
 gboolean prefs_get_id3_write_id3v24(void);
 const gchar *prefs_get_ipod_mount (void);
@@ -273,6 +274,18 @@ gboolean prefs_get_show_sync_dirs (void);
 void prefs_set_show_sync_dirs (gboolean val);
 gboolean prefs_get_sync_remove (void);
 void prefs_set_sync_remove (gboolean val);
+
+void prefs_set_sync_remove_confirm(gboolean val);
+gboolean prefs_get_sync_remove_confirm(void);
+void prefs_set_track_playlist_deletion(gboolean val);
+gboolean prefs_get_track_playlist_deletion(void);
+void prefs_set_track_ipod_file_deletion(gboolean val);
+gboolean prefs_get_track_ipod_file_deletion(void);
+void prefs_set_track_database_deletion(gboolean val);
+gboolean prefs_get_track_database_deletion(void);
+void prefs_set_track_local_file_deletion(gboolean val);
+gboolean prefs_get_track_local_file_deletion(void);
+
 gboolean prefs_get_display_toolbar (void);
 void prefs_set_display_toolbar (gboolean val);
 gboolean prefs_get_update_charset (void);
@@ -335,14 +348,19 @@ gboolean prefs_get_mserv_report_probs(void);
 void prefs_set_mserv_report_probs(gboolean val);
 const gchar *prefs_get_mserv_username(void);
 void prefs_set_mserv_username(const gchar *root);
+gboolean prefs_get_autoimport_commandline(void);
+void prefs_set_autoimport_commandline(gboolean val);
 
 gboolean prefs_get_disable_sorting(void);
 
 void prefs_set_string_value (const gchar *key, const gchar *value);
 void prefs_set_int_value (const gchar *key, gint value);
 void prefs_set_int64_value (const gchar *key, gint64 value);
+gchar *prefs_get_string (const gchar *key);
 gboolean prefs_get_string_value (const gchar *key, gchar **value);
 gboolean prefs_get_int_value (const gchar *key, gint *value);
+gint prefs_get_int (const gchar *key);
 gboolean prefs_get_int64_value (const gchar *key, gint64 *value);
+gint64 prefs_get_int64 (const gchar *key);
 
 #endif
